@@ -26,11 +26,10 @@ sub post_transaction {
   my $query;
   my $sth;
 
-  my $null;
-  ($null, $form->{department_id}) = split(/--/, $form->{department});
+  (undef, $form->{department_id}) = split(/--/, $form->{department});
   $form->{department_id} *= 1;
   
-  my %defaults = $form->get_defaults($dbh, \@{['fx%accno_id', 'cdt', 'precision']});
+  my %defaults = $form->get_defaults($dbh, \@{['fxgainloss_accno_id', 'cdt', 'precision']});
   $form->{precision} = $defaults{precision};
 
   my $ml = 1;
@@ -43,6 +42,8 @@ sub post_transaction {
   my $approved = ($form->{pending}) ? '0' : '1';
   my $action = ($approved) ? 'posted' : 'saved';
 
+  $form->{vc} =~ s/;//g;
+
   if ($form->{vc} eq 'vendor') {
     $table = 'ap';
     $ARAP = 'AP';
@@ -82,10 +83,10 @@ sub post_transaction {
 
     if ($form->{currency} ne $form->{defaultcurrency}) {
       push @{ $form->{acc_trans}{taxes} }, {
-	accno => $accno,
-	amount => $amount,
-	transdate => $form->{transdate},
-	fx_transaction => 1 };
+        accno => $accno,
+        amount => $amount,
+        transdate => $form->{transdate},
+        fx_transaction => 1 };
     }
 
   }
@@ -113,10 +114,10 @@ sub post_transaction {
     if ($amount{fxamount}{$i}) {
       
       if ($form->{taxincluded}) {
-	$amount = ($fxinvamount) ? $fxtax * $amount{fxamount}{$i} / $fxinvamount : 0;
-	$taxamount = $form->round_amount($amount - $fxdiff, $form->{precision});
-	$amount{fxamount}{$i} -= $taxamount;
-	$fxdiff = $form->round_amount($taxamount - ($amount - $fxdiff), 10);
+        $amount = ($fxinvamount) ? $fxtax * $amount{fxamount}{$i} / $fxinvamount : 0;
+        $taxamount = $form->round_amount($amount - $fxdiff, $form->{precision});
+        $amount{fxamount}{$i} -= $taxamount;
+        $fxdiff = $form->round_amount($taxamount - ($amount - $fxdiff), 10);
       }
 	
       # multiply by exchangerate
@@ -124,33 +125,33 @@ sub post_transaction {
       $amount{amount}{$i} = $form->round_amount($amount - $diff, $form->{precision});
       $diff = $amount{amount}{$i} - ($amount - $diff);
       
-      ($null, $project_id) = split /--/, $form->{"projectnumber_$i"};
+      (undef, $project_id) = split /--/, $form->{"projectnumber_$i"};
       $project_id ||= 'NULL';
       ($accno) = split /--/, $form->{"${ARAP}_amount_$i"};
 
       if ($keepcleared) {
-	$cleared = $form->dbquote($form->{"cleared_$i"}, SQL_DATE);
+        $cleared = $form->dbquote($form->{"cleared_$i"}, SQL_DATE);
       }
 
       push @{ $form->{acc_trans}{lineitems} }, {
-	accno => $accno,
-	amount => $amount{fxamount}{$i},
-	project_id => $project_id,
-	description => $form->{"description_$i"},
-	cleared => $cleared,
-	fx_transaction => 0,
-	id => $i };
+        accno => $accno,
+        amount => $amount{fxamount}{$i},
+        project_id => $project_id,
+        description => $form->{"description_$i"},
+        cleared => $cleared,
+        fx_transaction => 0,
+        id => $i };
 
       if ($form->{currency} ne $form->{defaultcurrency}) {
-	$amount = $amount{amount}{$i} - $amount{fxamount}{$i};
-	push @{ $form->{acc_trans}{lineitems} }, {
-	  accno => $accno,
-	  amount => $amount,
-	  project_id => $project_id,
-	  description => $form->{"description_$i"},
-	  cleared => $cleared,
-	  fx_transaction => 1,
-	  id => $i };
+        $amount = $amount{amount}{$i} - $amount{fxamount}{$i};
+        push @{ $form->{acc_trans}{lineitems} }, {
+          accno => $accno,
+          amount => $amount,
+          project_id => $project_id,
+          description => $form->{"description_$i"},
+          cleared => $cleared,
+          fx_transaction => 1,
+          id => $i };
       }
 
       $invnetamount += $amount{amount}{$i};
@@ -194,7 +195,7 @@ sub post_transaction {
   
   $paid = ($fxinvamount == $paid) ? $invamount : $form->round_amount($paid * $form->{exchangerate}, $form->{precision});
   
-  ($null, $form->{employee_id}) = split /--/, $form->{employee};
+  (undef, $form->{employee_id}) = split /--/, $form->{employee};
   unless ($form->{employee_id}) {
     ($form->{employee}, $form->{employee_id}) = $form->get_employee($dbh); 
   }
@@ -202,7 +203,7 @@ sub post_transaction {
   my $vth;
   
   # check if id really exists
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     $query = qq|SELECT id
                 FROM $table
  	        WHERE id = $form->{id}|;
@@ -212,9 +213,9 @@ sub post_transaction {
 
     if ($form->{id}) {
       # delete detail records
-      for (qw(acc_trans dpt_trans payment reference)) {
-	$query = qq|DELETE FROM $_ WHERE trans_id = $form->{id}|;
-	$dbh->do($query) || $form->dberror($query);
+      for (qw(acc_trans dpt_trans payment)) {
+        $query = qq|DELETE FROM $_ WHERE trans_id = $form->{id}|;
+        $dbh->do($query) || $form->dberror($query);
       }
     }
   }
@@ -254,7 +255,7 @@ sub post_transaction {
   $form->{amount} = $invamount;          # need for vr batch
   
   ($paymentaccno) = split /--/, $form->{"${ARAP}_paid_$form->{paidaccounts}"};
-  ($null, $paymentmethod_id) = split /--/, $form->{"paymentmethod_$form->{paidaccounts}"};
+  (undef, $paymentmethod_id) = split /--/, $form->{"paymentmethod_$form->{paidaccounts}"};
   $paymentmethod_id *= 1;
 
   if ($form->{vc} eq 'customer') {
@@ -295,7 +296,8 @@ sub post_transaction {
 	      exchangerate = $form->{exchangerate},
 	      dcn = |.$dbh->quote($form->{dcn}).qq|,
 	      bank_id = (SELECT id FROM chart WHERE accno = '$paymentaccno'),
-	      paymentmethod_id = $paymentmethod_id
+	      paymentmethod_id = $paymentmethod_id,
+	      language_code = '$form->{language_code}'
 	      WHERE id = $form->{id}|;
   $dbh->do($query) || $form->dberror($query);
 
@@ -444,7 +446,7 @@ sub post_transaction {
       # add payment
       ($accno) = split /--/, $form->{"${ARAP}_paid_$i"};
       
-      ($null, $paymentmethod_id) = split /--/, $form->{"paymentmethod_$i"};
+      (undef, $paymentmethod_id) = split /--/, $form->{"paymentmethod_$i"};
       $paymentmethod_id *= 1;
      
       if ($keepcleared) {
@@ -481,10 +483,9 @@ sub post_transaction {
 	  $amount = $form->round_amount(($form->round_amount($paid{fxamount}{$i} * $form->{exchangerate}, $form->{precision}) - $form->round_amount($paid{fxamount}{$i} * $form->{"exchangerate_$i"}, $form->{precision})) * -1, $form->{precision});
 	  
 	  if ($amount) {
-	    my $accno_id = (($amount * $ml * $arapml) > 0) ? $defaults{fxgain_accno_id} : $defaults{fxloss_accno_id};
 	    $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount,
 			transdate, fx_transaction, cleared, approved, vr_id)
-			VALUES ($form->{id}, $accno_id,
+			VALUES ($form->{id}, $defaults{fxgainloss_accno_id},
 			$amount * $ml * $arapml, '$form->{"datepaid_$i"}', '1',
 			$cleared, '$approved', $voucherid)|;
 	    $dbh->do($query) || $form->dberror($query);
@@ -543,7 +544,7 @@ sub post_transaction {
   $form->save_status($dbh);
 
   # save reference documents
-  $form->save_reference($dbh);
+  $form->save_reference($dbh, "${table}_$form->{type}");
   
   my %audittrail = ( tablename  => $table,
                      reference  => $form->{invnumber},
@@ -593,7 +594,7 @@ sub reverse_vouchers {
   $form->{voucher}{transaction} = $ref;
   $sth->finish;
 
-  if ($form->{batchid}) {
+  if ($form->{batchid} *= 1) {
     $form->update_balance($dbh,
 			  'br',
 			  'amount',
@@ -655,8 +656,11 @@ sub delete_transaction {
   # connect to database, turn AutoCommit off
   $dbh = $form->dbconnect_noauto($myconfig) unless $dbh;
   
+  $form->{vc} =~ s/;//g;
   my $table = ($form->{vc} eq 'customer') ? 'ar' : 'ap';
   
+  $form->{id} *= 1;
+  
   my %audittrail = ( tablename  => $table,
                      reference  => $form->{invnumber},
 		     formname   => 'transaction',
@@ -676,7 +680,7 @@ sub delete_transaction {
 
   my $id;
   for $id (qw(id glid)) {
-    for (qw(acc_trans dpt_trans payment status pay_trans reference)) {
+    for (qw(acc_trans dpt_trans payment status pay_trans)) {
       if ($form->{$id}) {
 	$query = qq|DELETE FROM $_ WHERE trans_id = $form->{$id}|;
 	$dbh->do($query) || $form->dberror($query);
@@ -690,28 +694,84 @@ sub delete_transaction {
     }
   }
   
+  $form->delete_references($dbh);
+
   $form->remove_locks($myconfig, $dbh, $table);
  
   # commit
-  my $rc;
+  my $rc = $dbh->commit;
   
-  if ($disconnect) {
-    $rc = $dbh->commit;
-    $dbh->disconnect;
-  }
+  $dbh->disconnect if $disconnect;
 
   $rc;
 
 }
 
 
+sub delete_transactions {
+  my ($self, $myconfig, $form, $spool) = @_;
+
+  # connect to database
+  my $dbh = $form->dbconnect_noauto($myconfig);
+
+  my $query = qq|SELECT * FROM |. lc $form->{ARAP} .
+              qq| WHERE id = ?|;
+  my $sth = $dbh->prepare($query) || $form->dberror;
+
+  my $ref;
+  my $deleted;
+
+  for (1 .. $form->{rowcount}) {
+    $form->{id} = $form->{"id_$_"};
+
+    if ($form->{id}) {
+
+      $sth->execute($form->{id});
+      $ref = $sth->fetchrow_hashref(NAME_lc);
+
+      if ($ref->{id}) {
+        $deleted = 0;
+
+        $form->{transdate} = $ref->{transdate};
+        $form->{invnumber} = $ref->{invnumber};
+        $form->{type} = 'transaction';
+
+        if ($ref->{invoice}) {
+          $form->{type} = 'invoice';
+
+          if ($form->{ARAP} eq 'AR') {
+            $deleted = IS->delete_invoice($myconfig, $form, $spool, $dbh);
+          } else {
+            $deleted = IR->delete_invoice($myconfig, $form, $spool, $dbh);
+          }
+        } else {
+          $deleted = AA->delete_transaction($myconfig, $form, $dbh);
+        }
+
+        if (!$deleted) {
+          $form->{notdeleted} .= "$ref->{invnumber}\n";
+          $sth->finish;
+          $dbh->disconnect;
+          return 0;
+        }
+      }
+
+      $sth->finish;
+    }
+  }
+  
+  $dbh->disconnect;
+
+  1;
+
+}
+
 
 sub transactions {
   my ($self, $myconfig, $form) = @_;
 
   # connect to database
   my $dbh = $form->dbconnect($myconfig);
-  my $null;
   my $var;
   my $paid = "a.paid";
   my $ml = 1;
@@ -722,24 +782,30 @@ sub transactions {
   
   my %defaults = $form->get_defaults($dbh, \@{['precision', 'company']});
   for (keys %defaults) { $form->{$_} = $defaults{$_} }
+
+  $form->{defaultcurrency} = substr($form->get_currencies($myconfig, $dbh),0,3);
   
+  $form->{vc} =~ s/;//g;
+
   if ($form->{vc} eq 'vendor') {
     $ml = -1;
     $ARAP = 'AP';
     $table = 'ap';
   }
-  
-  ($form->{transdatefrom}, $form->{transdateto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
+
+  unless ($form->{transdatefrom} || $form->{transdateto}) {
+    ($form->{transdatefrom}, $form->{transdateto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
+  }
  
   if ($form->{outstanding}) {
     $paid = qq|SELECT SUM(ac.amount) * -1 * $ml
                FROM acc_trans ac
-	       JOIN chart c ON (c.id = ac.chart_id)
-	       WHERE ac.trans_id = a.id
-	       AND ac.approved = '1'
-	       AND (c.link LIKE '%${ARAP}_paid%'
-	            OR c.link LIKE '%${ARAP}_discount%'
-		    OR c.link = '')|;
+               JOIN chart c ON (c.id = ac.chart_id)
+               WHERE ac.trans_id = a.id
+               AND ac.approved = '1'
+               AND (c.link LIKE '%${ARAP}_paid%'
+                    OR c.link LIKE '%${ARAP}_discount%'
+                    OR c.link = '')|;
     $paid .= qq|
                AND ac.transdate <= '$form->{transdateto}'| if $form->{transdateto};
     $form->{summary} = 1;
@@ -747,23 +813,23 @@ sub transactions {
   }
 
   my $taxfld = qq|SELECT sum(ac.amount)
-		  FROM acc_trans ac
-		  JOIN chart c ON (c.id = ac.chart_id)
-		  WHERE ac.trans_id = a.id
-		  AND ac.approved = '1'
-		  AND c.link LIKE '%_tax%'|;
+                  FROM acc_trans ac
+                  JOIN chart c ON (c.id = ac.chart_id)
+                  WHERE ac.trans_id = a.id
+                  AND ac.approved = '1'
+                  AND c.link LIKE '%_tax%'|;
   $taxfld .= qq|
-               AND ac.transdate <= '$form->{transdateto}'| if $form->{transdateto};
+                  AND ac.transdate <= '$form->{transdateto}'| if $form->{transdateto};
   
   if (!$form->{summary} || $form->{l_memo}) {
-    $acc_trans_flds = qq|, c.accno, ac.source,
+    $acc_trans_flds = qq|, ch.accno, ac.source,
 			 pr.projectnumber, ac.memo,
 			 ac.amount AS linetotal,
 			 i.description AS linedescription, ac.fx_transaction|;
 
     $acc_trans_join = qq|
 	    JOIN acc_trans ac ON (a.id = ac.trans_id)
-	    JOIN chart c ON (c.id = ac.chart_id)
+	    JOIN chart ch ON (ch.id = ac.chart_id)
 	    LEFT JOIN project pr ON (pr.id = ac.project_id)
 	    LEFT JOIN invoice i ON (i.id = ac.id|;
     $acc_trans_join .= qq| AND i.trans_id = a.id| if $form->{summary};
@@ -772,30 +838,31 @@ sub transactions {
     
   my $query = qq|SELECT a.id, a.invnumber, a.ordnumber, a.transdate,
                  a.duedate, ($taxfld) * $ml AS tax,
-		 a.amount, ($paid) AS paid,
-		 a.invoice, a.datepaid, a.terms, a.notes,
-		 a.shipvia, a.waybill, a.shippingpoint,
-		 e.name AS employee, vc.name, vc.$form->{vc}number,
-		 a.$form->{vc}_id, a.till, a.curr,
-		 a.exchangerate, d.description AS department,
-		 a.ponumber, a.warehouse_id, w.description AS warehouse,
-		 a.description, a.dcn, pm.description AS paymentmethod,
-		 a.datepaid - a.duedate AS paymentdiff,
-		 ad.address1, ad.address2, ad.city, ad.zipcode, ad.country
-		 $acc_trans_flds
-	         FROM $table a
-	      JOIN $form->{vc} vc ON (a.$form->{vc}_id = vc.id)
-	      JOIN address ad ON (ad.trans_id = vc.id)
-	      LEFT JOIN employee e ON (a.employee_id = e.id)
-	      LEFT JOIN department d ON (a.department_id = d.id)
-	      LEFT JOIN warehouse w ON (a.warehouse_id = w.id)
-	      LEFT JOIN paymentmethod pm ON (pm.id = a.paymentmethod_id)
-	      $acc_trans_join
-	      |;
+                 a.amount, ($paid) AS paid,
+                 a.invoice, a.datepaid, a.terms, a.notes,
+                 a.shipvia, a.waybill, a.shippingpoint,
+                 e.name AS employee, vc.name, vc.$form->{vc}number,
+                 a.$form->{vc}_id, a.till, a.curr,
+                 a.exchangerate, d.description AS department,
+                 a.ponumber, a.warehouse_id, w.description AS warehouse,
+                 a.description, a.dcn, pm.description AS paymentmethod,
+                 a.datepaid - a.duedate AS paymentdiff,
+                 ad.address1, ad.address2, ad.city, ad.zipcode, ad.country,
+                 c.description AS paymentaccount, vc.taxnumber
+                 $acc_trans_flds
+                 FROM $table a
+                 JOIN $form->{vc} vc ON (a.$form->{vc}_id = vc.id)
+                 JOIN address ad ON (ad.trans_id = vc.id)
+                 LEFT JOIN employee e ON (a.employee_id = e.id)
+                 LEFT JOIN department d ON (a.department_id = d.id)
+                 LEFT JOIN warehouse w ON (a.warehouse_id = w.id)
+                 LEFT JOIN paymentmethod pm ON (pm.id = a.paymentmethod_id)
+                 LEFT JOIN chart c ON (c.id = a.bank_id)
+                 $acc_trans_join
+|;
 
   my $where = "a.approved = '1'";
-
-  if ($form->{"$form->{vc}_id"}) {
+  if ($form->{"$form->{vc}_id"} *= 1) {
     $where .= qq| AND a.$form->{vc}_id = $form->{"$form->{vc}_id"}|;
   } else {
     if ($form->{$form->{vc}}) {
@@ -807,12 +874,30 @@ sub transactions {
       $where .= " AND lower(vc.$form->{vc}number) LIKE '$var'";
     }
   }
-  for (qw(warehouse department employee)) {
+  for (qw(warehouse employee)) {
     if ($form->{$_}) {
-      ($null, $var) = split /--/, $form->{$_};
+      (undef, $var) = split /--/, $form->{$_};
       $where .= " AND a.${_}_id = $var";
     }
   }
+  if ($form->{department}) {
+    ($var, undef) = split /--/, $form->{department};
+    my $query = qq|SELECT id
+                   FROM department
+                   WHERE description LIKE '$var%'|;
+    my $dth = $dbh->prepare($query);
+    my @ids;
+
+    $dth->execute || $form->dberror($query);
+    while (($var) = $dth->fetchrow_array) {
+      push @ids, $var;
+    }
+    if (@ids) {
+      $where .= " AND (";
+      for (@ids) { $where .= "a.department_id = $_ OR " }
+      $where = substr($where, 0, -4) . ")";
+    }
+  }
 
   for (qw(invnumber ordnumber)) {
     if ($form->{$_}) {
@@ -879,11 +964,11 @@ sub transactions {
   if ($form->{$ARAP}) {
     my ($accno) = split /--/, $form->{$ARAP};
     $where .= qq|
-                AND a.id IN (SELECT ac.trans_id
+                 AND a.id IN (SELECT ac.trans_id
 		             FROM acc_trans ac
-			     JOIN chart c ON (c.id = ac.chart_id)
-			     WHERE a.id = ac.trans_id
-			     AND c.accno = '$accno')
+                 JOIN chart c ON (c.id = ac.chart_id)
+                 WHERE a.id = ac.trans_id
+                 AND c.accno = '$accno')
 		|;
   }
   
@@ -891,10 +976,10 @@ sub transactions {
     $var = $form->like(lc $form->{memo});
     $where .= qq| AND (a.id IN (SELECT DISTINCT trans_id
                                 FROM acc_trans
-				WHERE lower(memo) LIKE '$var')
-		       OR a.id IN (SELECT DISTINCT trans_id
-		                FROM invoice
-				WHERE lower(description) LIKE '$var'))|;
+                 WHERE lower(memo) LIKE '$var')
+                 OR a.id IN (SELECT DISTINCT trans_id
+                             FROM invoice
+                             WHERE lower(description) LIKE '$var'))|;
   }
 
   $query .= " WHERE $where";
@@ -909,8 +994,11 @@ sub transactions {
 
   my $i = -1;
   my $sameid;
- 
+  my %curr;
+  
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
+    $curr{$ref->{curr}} = 1;
+    
     $ref->{exchangerate} ||= 1;
     if ($ref->{linetotal} <= 0) {
       $ref->{debit} = $ref->{linetotal} * -1;
@@ -930,19 +1018,19 @@ sub transactions {
       next if $form->round_amount($ref->{amount}, $form->{precision}) == $form->round_amount($ref->{paid}, $form->{precision});
     }
 
-    for (qw(address1 address2 city zipcode country)) { $ref->{address} .= "$ref->{$_} " }
+    for (qw(address1 address2)) { $ref->{address} .= "$ref->{$_} " }
     
     if ($form->{summary}) {
       if ($sameid != $ref->{id}) {
-	$i++;
-	push @{ $form->{transactions} }, $ref;
+        $i++;
+        push @{ $form->{transactions} }, $ref;
       } else {
         if ($ref->{memo} && ! $ref->{fx_transaction}) {
-	  if ($form->{transactions}[$i]->{memo}) {
-	    $form->{transactions}[$i]->{memo} .= "\n";
-	  }
-	  $form->{transactions}[$i]->{memo} .= $ref->{memo};
-	}
+          if ($form->{transactions}[$i]->{memo}) {
+            $form->{transactions}[$i]->{memo} .= "\n";
+          }
+          $form->{transactions}[$i]->{memo} .= $ref->{memo};
+        }
       }
     } else {
       push @{ $form->{transactions} }, $ref;
@@ -950,6 +1038,9 @@ sub transactions {
     $sameid = $ref->{id};
   }
   
+  delete $curr{$form->{defaultcurrency}};
+  $form->{currencies} = join ':', sort (keys %curr);
+  
   $sth->finish;
   $dbh->disconnect;
 
@@ -990,6 +1081,7 @@ sub get_name {
     $duedate = ($form->{transdate}) ? "to_date('$form->{transdate}', '$dateformat') + c.terms" : "current_date + c.terms";
   }
 
+  $form->{vc} =~ s/;//g;
   $form->{"$form->{vc}_id"} *= 1;
 
   my $arap = "ar";
@@ -1000,39 +1092,39 @@ sub get_name {
   
   # get customer/vendor
   # also get shipto if we did not convert an order or invoice
-  my $shipto = ", s.*" if ! $form->{shipto};
+  my $shipto = ", s.*" if ! $form->{linkshipto};
   my $shiptojoin = qq|LEFT JOIN shipto s ON (s.trans_id = c.id)|;
 
   my $query = qq|SELECT c.name AS $form->{vc}, c.$form->{vc}number,
                  c.discount, c.creditlimit, c.terms,
                  c.email, c.cc, c.bcc, c.taxincluded,
-		 ad.address1, ad.address2, ad.city, ad.state,
-		 ad.zipcode, ad.country, c.curr AS currency, c.language_code,
-	         $duedate AS duedate, c.notes AS intnotes,
-		 b.discount AS tradediscount, b.description AS business,
-		 e.name AS employee, e.id AS employee_id,
-		 c1.accno AS arap_accno, c1.description AS arap_accno_description, t1.description AS arap_accno_translation,
-		 c2.accno AS payment_accno, c2.description AS payment_accno_description, t2.description AS payment_accno_translation,
-		 c3.accno AS discount_accno, c3.description AS discount_accno_description, t3.description AS discount_accno_translation,
-		 c.cashdiscount, c.threshold, c.discountterms,
-		 c.remittancevoucher,
-		 pm.description AS payment_method, c.paymentmethod_id,
-		 pr.pricegroup
-		 $shipto
+                 ad.address1, ad.address2, ad.city, ad.state,
+                 ad.zipcode, ad.country, c.curr AS currency, c.language_code,
+                 $duedate AS duedate, c.notes AS intnotes,
+                 b.discount AS tradediscount, b.description AS business,
+                 e.name AS employee, e.id AS employee_id,
+                 c1.accno AS arap_accno, c1.description AS arap_accno_description, t1.description AS arap_accno_translation,
+                 c2.accno AS payment_accno, c2.description AS payment_accno_description, t2.description AS payment_accno_translation,
+                 c3.accno AS discount_accno, c3.description AS discount_accno_description, t3.description AS discount_accno_translation,
+                 c.cashdiscount, c.threshold, c.discountterms,
+                 c.remittancevoucher,
+                 pm.description AS payment_method, c.paymentmethod_id,
+                 pr.pricegroup
+                 $shipto
                  FROM $form->{vc} c
-		 JOIN address ad ON (ad.trans_id = c.id)
-		 LEFT JOIN business b ON (b.id = c.business_id)
-		 LEFT JOIN employee e ON (e.id = c.employee_id)
-		 LEFT JOIN chart c1 ON (c1.id = c.arap_accno_id)
-		 LEFT JOIN chart c2 ON (c2.id = c.payment_accno_id)
-		 LEFT JOIN chart c3 ON (c3.id = c.discount_accno_id)
-		 LEFT JOIN translation t1 ON (t1.trans_id = c1.id AND t1.language_code = '$myconfig->{countrycode}')
-		 LEFT JOIN translation t2 ON (t2.trans_id = c2.id AND t2.language_code = '$myconfig->{countrycode}')
-		 LEFT JOIN translation t3 ON (t3.trans_id = c3.id AND t3.language_code = '$myconfig->{countrycode}')
-		 LEFT JOIN paymentmethod pm ON (pm.id = c.paymentmethod_id)
-		 LEFT JOIN pricegroup pr ON (pr.id = c.pricegroup_id)
-		 $shiptojoin
-	         WHERE c.id = $form->{"$form->{vc}_id"}|;
+                 JOIN address ad ON (ad.trans_id = c.id)
+                 LEFT JOIN business b ON (b.id = c.business_id)
+                 LEFT JOIN employee e ON (e.id = c.employee_id)
+                 LEFT JOIN chart c1 ON (c1.id = c.arap_accno_id)
+                 LEFT JOIN chart c2 ON (c2.id = c.payment_accno_id)
+                 LEFT JOIN chart c3 ON (c3.id = c.discount_accno_id)
+                 LEFT JOIN translation t1 ON (t1.trans_id = c1.id AND t1.language_code = '$myconfig->{countrycode}')
+                 LEFT JOIN translation t2 ON (t2.trans_id = c2.id AND t2.language_code = '$myconfig->{countrycode}')
+                 LEFT JOIN translation t3 ON (t3.trans_id = c3.id AND t3.language_code = '$myconfig->{countrycode}')
+                 LEFT JOIN paymentmethod pm ON (pm.id = c.paymentmethod_id)
+                 LEFT JOIN pricegroup pr ON (pr.id = c.pricegroup_id)
+                 $shiptojoin
+                 WHERE c.id = $form->{"$form->{vc}_id"}|;
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
@@ -1072,18 +1164,18 @@ sub get_name {
 
 
   $query = qq|SELECT SUM(a.amount - a.paid)
-	      FROM $arap a
-	      WHERE a.amount != a.paid
-	      AND $form->{vc}_id = $form->{"$form->{vc}_id"}|;
+              FROM $arap a
+              WHERE a.amount != a.paid
+              AND $form->{vc}_id = $form->{"$form->{vc}_id"}|;
   my ($amount) = $dbh->selectrow_array($query);
   $form->{creditremaining} -= $amount;
 
   if ($form->{"$form->{vc}_id"}) {
     $query = qq|SELECT SUM(a.amount)
-		FROM oe a
-		WHERE a.quotation = '0'
-		AND a.closed = '0'
-		AND a.$form->{vc}_id = $form->{"$form->{vc}_id"}|;
+                FROM oe a
+                WHERE a.quotation = '0'
+                AND a.closed = '0'
+                AND a.$form->{vc}_id = $form->{"$form->{vc}_id"}|;
     ($amount) = $dbh->selectrow_array($query);
     $form->{creditremaining} -= $amount;
   }
@@ -1091,8 +1183,8 @@ sub get_name {
   # get taxes
   $query = qq|SELECT c.accno
               FROM chart c
-	      JOIN $form->{vc}tax ct ON (ct.chart_id = c.id)
-	      WHERE ct.$form->{vc}_id = $form->{"$form->{vc}_id"}|;
+              JOIN $form->{vc}tax ct ON (ct.chart_id = c.id)
+              WHERE ct.$form->{vc}_id = $form->{"$form->{vc}_id"}|;
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
@@ -1116,6 +1208,7 @@ sub get_name {
 	      JOIN tax t ON (c.id = t.chart_id)
 	      LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
 	      WHERE c.link LIKE '%$form->{ARAP}_tax%'
+              AND c.closed = '0'
 	      $where
 	      ORDER BY c.accno, t.validto|;
   $sth = $dbh->prepare($query);
@@ -1135,47 +1228,34 @@ sub get_name {
 
   # setup last accounts used for this customer/vendor
   if (!$form->{id}) {
-    for (qw(department_id department)) { delete $form->{$_} }
     
-    if ($form->{type} =~ /_(order|quotation)/) {
-      $query = qq|SELECT MAX(o.id),
-                  o.department_id, d.description AS department,
-		  o.warehouse_id, d.description AS warehouse
-		  FROM oe o
-		  LEFT JOIN department d ON (d.id = o.department_id)
-		  LEFT JOIN warehouse w ON (w.id = o.warehouse_id)
-		  WHERE o.$form->{vc}_id = $form->{"$form->{vc}_id"}
-		  GROUP BY o.department_id, department,
-		  o.warehouse_id, warehouse
-|;
-    } elsif ($form->{type} =~ /invoice/) {
+    # dummy query
+    $query = qq|SELECT trans_id
+                FROM acc_trans
+                WHERE trans_id = 0|;
+
+    if ($form->{type} =~ /invoice/) {
       $query = qq|SELECT c.accno, c.description,
-		  a.department_id, d.description AS department,
-		  a.warehouse_id, d.description AS warehouse,
-		  l.description AS translation
-		  FROM chart c
-		  JOIN acc_trans ac ON (ac.chart_id = c.id)
-		  JOIN $arap a ON (a.id = ac.trans_id)
-		  LEFT JOIN department d ON (d.id = a.department_id)
-		  LEFT JOIN warehouse w ON (w.id = a.warehouse_id)
-		  LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
-		  WHERE a.$form->{vc}_id = $form->{"$form->{vc}_id"}
-		  AND a.id IN (SELECT max(id) FROM $arap
-			       WHERE $form->{vc}_id = $form->{"$form->{vc}_id"})|;
-    } else {
+                  l.description AS translation
+                  FROM chart c
+                  JOIN acc_trans ac ON (ac.chart_id = c.id)
+                  JOIN $arap a ON (a.id = ac.trans_id)
+                  LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
+                  WHERE a.$form->{vc}_id = $form->{"$form->{vc}_id"}
+                  AND a.id IN (SELECT max(id) FROM $arap
+                  WHERE $form->{vc}_id = $form->{"$form->{vc}_id"})|;
+    } elsif ($form->{type} =~ /transaction/) {
       $query = qq|SELECT c.accno, c.description, c.link, c.category,
-		  ac.project_id, p.projectnumber, a.department_id,
-		  d.description AS department,
-		  l.description AS translation
-		  FROM chart c
-		  JOIN acc_trans ac ON (ac.chart_id = c.id)
-		  JOIN $arap a ON (a.id = ac.trans_id)
-		  LEFT JOIN project p ON (ac.project_id = p.id)
-		  LEFT JOIN department d ON (d.id = a.department_id)
-		  LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
-		  WHERE a.$form->{vc}_id = $form->{"$form->{vc}_id"}
-		  AND a.id IN (SELECT max(id) FROM $arap
-			       WHERE $form->{vc}_id = $form->{"$form->{vc}_id"})|;
+                  ac.project_id, p.projectnumber,
+                  l.description AS translation
+                  FROM chart c
+                  JOIN acc_trans ac ON (ac.chart_id = c.id)
+                  JOIN $arap a ON (a.id = ac.trans_id)
+                  LEFT JOIN project p ON (ac.project_id = p.id)
+                  LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
+                  WHERE a.$form->{vc}_id = $form->{"$form->{vc}_id"}
+                  AND a.id IN (SELECT max(id) FROM $arap
+                  WHERE $form->{vc}_id = $form->{"$form->{vc}_id"})|;
     }
 
     $sth = $dbh->prepare($query);
@@ -1183,18 +1263,12 @@ sub get_name {
 
     my $i = 0;
     while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
-      $form->{department} = $ref->{department};
-      $form->{department_id} = $ref->{department_id};
-
-      $form->{warehouse} = $ref->{warehouse};
-      $form->{warehouse_id} = $ref->{warehouse_id};
-      
       $ref->{description} = $ref->{translation} if $ref->{translation};
       if ($ref->{link} =~ /_amount/) {
-	$i++;
-	next if $form->{"amount_$i"};
-	$form->{"$form->{ARAP}_amount_$i"} = "$ref->{accno}--$ref->{description}" if $ref->{accno};
-	$form->{"projectnumber_$i"} = "$ref->{projectnumber}--$ref->{project_id}" if $ref->{project_id};
+        $i++;
+        next if $form->{"amount_$i"};
+        $form->{"$form->{ARAP}_amount_$i"} = "$ref->{accno}--$ref->{description}" if $ref->{accno};
+        $form->{"projectnumber_$i"} = "$ref->{projectnumber}--$ref->{project_id}" if $ref->{project_id};
       }
     }
     $sth->finish;
@@ -1215,6 +1289,8 @@ sub company_details {
   # connect to database
   $dbh = $form->dbconnect($myconfig) unless $dbh;
   
+  $form->{vc} =~ s/;//g;
+
   # get rest for the customer/vendor
   my $query = qq|SELECT ct.$form->{vc}number, ct.name, ad.address1, ad.address2,
                  ad.city, ad.state, ad.zipcode, ad.country,
@@ -1248,14 +1324,14 @@ sub company_details {
   for (qw(phone fax mobile)) { $form->{"$form->{vc}$_"} ||= $ref->{$_} }
   $sth->finish;
   
-  my ($null, $id) = split /--/, $form->{employee};
+  my (undef, $id) = split /--/, $form->{employee};
   $id *= 1;
   $query = qq|SELECT workphone, workfax, workmobile
               FROM employee
               WHERE id = $id|;
   ($form->{workphone}, $form->{workfax}, $form->{workmobile}) = $dbh->selectrow_array($query);
 
-  my @df = qw(weightunit cdt company companyemail companywebsite address tel fax businessnumber);
+  my @df = qw(weightunit cdt company companyemail companywebsite address tel fax businessnumber annualinterest latepaymentfee restockingcharge);
   my %defaults = $form->get_defaults($dbh, \@df);
   for (@df) { $form->{$_} = $defaults{$_} }
 
@@ -1312,6 +1388,9 @@ sub ship_to {
   # connect to database
   my $dbh = $form->dbconnect($myconfig);
   
+  $form->{vc} =~ s/;//g;
+  $form->{"$form->{vc}_id"} *= 1;
+  
   AA->company_details($myconfig, $form, $dbh);
   
   my $table = ($form->{vc} eq 'customer') ? 'ar' : 'ap';
@@ -1342,7 +1421,7 @@ sub ship_to {
 		 JOIN $table a ON (a.id = s.trans_id)
 	         WHERE a.$form->{vc}_id = $form->{"$form->{vc}_id"}|;
 
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     $query .= qq|
 		 EXCEPT
 		 SELECT
@@ -1387,10 +1466,10 @@ sub all_names {
   
   my $query;
   my $var;
-  my $null;
   
   my $where = "1 = 1";
 
+  $form->{vc} =~ s/;//g;
   my @sf = ("$form->{vc}number");
   push @sf, qw(name);
 
@@ -1416,8 +1495,8 @@ sub all_names {
     }
   }
 
-  if ($form->{currency}) {
-    $where .= " AND vc.curr = '$form->{currency}'";
+  if ($form->{curr}) {
+    $where .= " AND vc.curr = '$form->{curr}'";
   }
 
   for (qw(city state zipcode country)) {
@@ -1429,7 +1508,7 @@ sub all_names {
 
   for (qw(employee paymentmethod pricegroup business)) {
     if ($form->{$_}) {
-      ($null, $var) = split /--/, $form->{$_};
+      (undef, $var) = split /--/, $form->{$_};
       $where .= " AND vc.${_}_id = $var";
     }
   }
@@ -1437,18 +1516,18 @@ sub all_names {
   $query = qq|SELECT vc.id, vc.name, vc.$form->{vc}number, vc.startdate,
               vc.enddate,
               ad.city, ad.state, ad.zipcode, ad.country,
-	      e.name AS employee,
-	      b.description AS business,
-	      pg.pricegroup,
-	      pm.description AS paymentmethod,
-	      vc.curr
+              e.name AS employee,
+              b.description AS business,
+              pg.pricegroup,
+              pm.description AS paymentmethod,
+              vc.curr
               FROM $form->{vc} vc
-	      JOIN address ad ON (ad.trans_id = vc.id)
-	      LEFT JOIN employee e ON (e.id = vc.employee_id)
-	      LEFT JOIN paymentmethod pm ON (pm.id = vc.paymentmethod_id)
-	      LEFT JOIN business b ON (b.id = vc.business_id)
-	      LEFT JOIN pricegroup pg ON (pg.id = vc.pricegroup_id)
-	      WHERE $where|;
+              JOIN address ad ON (ad.trans_id = vc.id)
+              LEFT JOIN employee e ON (e.id = vc.employee_id)
+              LEFT JOIN paymentmethod pm ON (pm.id = vc.paymentmethod_id)
+              LEFT JOIN business b ON (b.id = vc.business_id)
+              LEFT JOIN pricegroup pg ON (pg.id = vc.pricegroup_id)
+              WHERE $where|;
 
   my @sf = (name);
   my %ordinal = $form->ordinal_order($dbh, $query);
@@ -1517,6 +1596,11 @@ sub consolidate {
 		 AND invoice = '$invoice'
 		 AND approved = '1'
 		 AND onhold = '0'|;
+
+  my @sf = (transdate);
+  my %ordinal = $form->ordinal_order($dbh, $query);
+  $query .= qq| ORDER BY | .$form->sort_order(\@sf, \%ordinal);
+
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
