@@ -47,9 +47,10 @@ sub paymentaccounts {
   my $query = qq|SELECT c.accno, c.description, c.link,
                  l.description AS translation
                  FROM chart c
-		 LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
-		 WHERE c.link LIKE '%$form->{ARAP}%'
-		 ORDER BY c.accno|;
+		             LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
+                 WHERE c.link LIKE '%$form->{ARAP}%'
+                 AND c.closed = '0'
+                 ORDER BY c.accno|;
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
@@ -61,13 +62,13 @@ sub paymentaccounts {
     $ref->{description} = $ref->{translation} if $ref->{translation};
     foreach my $item (split /:/, $ref->{link}) {
       if ($item eq $form->{ARAP}) {
-	push @{ $form->{PR}{$form->{ARAP}} }, $ref;
+        push @{ $form->{PR}{$form->{ARAP}} }, $ref;
       }
       if ($item eq "$form->{ARAP}_paid") {
-	push @{ $form->{PR}{"$form->{ARAP}_paid"} }, $ref;
+        push @{ $form->{PR}{"$form->{ARAP}_paid"} }, $ref;
       }
       if ($item eq "$form->{ARAP}_discount") {
-	push @{ $form->{PR}{"$form->{ARAP}_discount"} }, $ref;
+        push @{ $form->{PR}{"$form->{ARAP}_discount"} }, $ref;
       }
     }
   }
@@ -81,7 +82,7 @@ sub paymentaccounts {
   my %defaults = $form->get_defaults($dbh, \@{['closedto', "$form->{type}\_%"]});
   for (keys %defaults) { $form->{$_} = $defaults{$_} }
 
-  $form->{currencies} = $form->get_currencies($dbh, $myconfig);
+  $form->{currencies} = $form->get_currencies($myconfig, $dbh);
 
   if ($form->{payment} eq 'payments') {
     # get language codes
@@ -93,8 +94,8 @@ sub paymentaccounts {
   if ($form->{vc} eq 'vendor') {
     # get business types
     $query = qq|SELECT *
-		FROM business
-		ORDER BY 2|;
+                FROM business
+                ORDER BY rn|;
     $sth = $dbh->prepare($query);
     $sth->execute || $form->dberror($query);
 
@@ -105,8 +106,8 @@ sub paymentaccounts {
   }
 
   $query = qq|SELECT *
-	      FROM paymentmethod
-	      ORDER BY 2|;
+              FROM paymentmethod
+              ORDER BY rn|;
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
@@ -129,9 +130,9 @@ sub get_openvc {
 
   my $where = qq|a.amount != a.paid
                  AND a.approved = '1'
-		 AND a.onhold = '0'
-		 AND NOT a.id IN (SELECT id
-		                  FROM semaphore)|;
+                 AND a.onhold = '0'
+                 AND NOT a.id IN (SELECT id
+		             FROM semaphore)|;
 
   $form->{vc} =~ s/;//g;
   my $arap = ($form->{vc} eq 'customer') ? 'ar' : 'ap';
@@ -162,8 +163,8 @@ sub get_openvc {
     if ($form->{vc} eq 'vendor') {
       ($description, $id) = split /--/, $form->{business};
       if ($id) {
-	$where .= qq|
-		AND vc.business_id = $id|;
+        $where .= qq|
+        AND vc.business_id = $id|;
       }
     }
   }
@@ -195,17 +196,25 @@ sub get_openvc {
   # build selection list
   $query = qq|SELECT vc.*,
               ad.address1, ad.address2, ad.city, ad.state, ad.zipcode,
-	      ad.country, a.amount, a.paid,
- 	      a.exchangerate,
-	      l.description AS translation
-	      FROM $form->{vc} vc
-	      JOIN $arap a ON (a.$form->{vc}_id = vc.id)
-	      JOIN acc_trans ac ON (a.id = ac.trans_id)
-	      JOIN chart c ON (c.id = ac.chart_id)
-	      JOIN address ad ON (ad.trans_id = vc.id)
-	      LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
-	      WHERE $where
-	      ORDER BY vc.$sortorder|;
+              ad.country, a.amount, a.paid,
+              a.exchangerate,
+              l.description AS translation,
+              ch.accno AS $form->{ARAP},
+              ch.description AS $form->{ARAP}_description,
+              pa.accno AS $form->{ARAP}_paid,
+              pa.description AS $form->{ARAP}_paid_description,
+              pm.description AS paymentmethod
+              FROM $form->{vc} vc
+              JOIN $arap a ON (a.$form->{vc}_id = vc.id)
+              JOIN acc_trans ac ON (a.id = ac.trans_id)
+              JOIN chart c ON (c.id = ac.chart_id)
+              JOIN address ad ON (ad.trans_id = vc.id)
+              LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
+              LEFT JOIN chart ch ON (ch.id = vc.arap_accno_id)
+              LEFT JOIN chart pa ON (pa.id = vc.payment_accno_id)
+              LEFT JOIN paymentmethod pm ON (pm.id = vc.paymentmethod_id)
+              WHERE $where
+              ORDER BY vc.$sortorder|;
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
@@ -232,7 +241,7 @@ sub get_openvc {
     next if $vc{$ref->{id}};
     if ($form->{vc} eq 'vendor') {
       if ($ref->{threshold} > 0) {
-	next if $due{$ref->{id}} < $ref->{threshold};
+        next if $due{$ref->{id}} < $ref->{threshold};
       }
     }
 
@@ -415,9 +424,9 @@ sub get_openinvoices {
  
   # connect to database
   my $dbh = $form->dbconnect($myconfig);
-
-  $form->{vc} =~ s/;//g;
  
+  $form->{vc} =~ s/;//g;
+
   # remove locks
   $form->remove_locks($myconfig, $dbh, $form->{arap});
   
@@ -582,35 +591,22 @@ sub post_payment {
 	      ORDER BY t.validto DESC|;
   my $tth = $dbh->prepare($query) || $form->dberror($query);
   
-  my %defaults = $form->get_defaults($dbh, \@{['fx%_accno_id', 'cdt']});
+  my %defaults = $form->get_defaults($dbh, \@{['fxgainloss_accno_id', 'cdt']});
 
   my $ml = ($form->{ARAP} eq 'AR') ? 1 : -1;
 
   $form->{arap} = lc $form->{ARAP};
-
-  my $where;
   
-  if ($form->{ARAP} eq 'AR') {
-    $ml = 1;
-    $where = qq|
-		(c.link = 'AR'
-		OR c.link LIKE 'AR:%')
+  my $where = qq|
+		c.link = '$form->{ARAP}'
 		|;
-  } else {
-    $ml = -1;
-    $where = qq|
-                (c.link = 'AP'
-                OR c.link LIKE '%:AP'
-		OR c.link LIKE '%:AP:%')
-		|;
-  }
-
+ 
   # AR/AP default account
   $query = qq|SELECT c.id
               FROM chart c
               WHERE $where|;
   my ($arapdefault) = $dbh->selectrow_array($query);
-
+ 
   # AR/AP account
   $query = qq|SELECT DISTINCT c.id
               FROM chart c
@@ -651,8 +647,7 @@ sub post_payment {
 		WHERE ac.trans_id = ?
 		AND ac.vr_id = $form->{voucherid}
 		AND c.link LIKE '%$form->{ARAP}_paid%'
-		AND NOT (ac.chart_id = $defaults{fxgain_accno_id}
-		      OR ac.chart_id = $defaults{fxloss_accno_id})|;
+		AND NOT (ac.chart_id = $defaults{fxgainloss_accno_id})|;
     $sth = $dbh->prepare($query) || $form->dberror($query);
 
     # discount
@@ -824,10 +819,9 @@ sub post_payment {
       # gain/loss
       $amount = $form->round_amount(($form->round_amount($form->{"paid_$i"} * $trans{$form->{"id_$i"}}{exchangerate}, $form->{precision}) - $form->round_amount($form->{"paid_$i"} * $form->{exchangerate}, $form->{precision})) * $ml * -1, $form->{precision});
       if ($amount) {
-	my $accno_id = ($amount > 0) ? $defaults{fxgain_accno_id} : $defaults{fxloss_accno_id};
 	$query = qq|INSERT INTO acc_trans (trans_id, chart_id, transdate,
 		    amount, fx_transaction, approved, vr_id)
-		    VALUES ($form->{"id_$i"}, $accno_id,
+		    VALUES ($form->{"id_$i"}, $defaults{fxgainloss_accno_id},
 		    '$form->{datepaid}', $amount, '1', '$approved',
 		    $voucherid)|;
 	$dbh->do($query) || $form->dberror($query);
@@ -913,10 +907,9 @@ sub post_payment {
 	    $amount = $form->round_amount(($form->round_amount($cdt{$_} * $trans{$form->{"id_$i"}}{exchangerate}, $form->{precision}) - $form->round_amount($cdt{$_} * $form->{exchangerate}, $form->{precision})) * $ml * -1, $form->{precision});
 	    
 	    if ($amount) {
-	      my $accno_id = ($amount > 0) ? $defaults{fxgain_accno_id} : $defaults{fxloss_accno_id};
 	      $query = qq|INSERT INTO acc_trans (trans_id, chart_id, transdate,
 			  amount, fx_transaction, approved, vr_id, id)
-			  VALUES ($form->{"id_$i"}, $accno_id,
+			  VALUES ($form->{"id_$i"}, $defaults{fxgainloss_accno_id},
 			  '$form->{datepaid}', $amount, '1', '$approved',
 			  $voucherid, $form->{"id_$i"})|;
 	      $dbh->do($query) || $form->dberror($query);
@@ -1039,6 +1032,398 @@ sub invoice_ids {
 }
 
 
+sub payment_register {
+  my ($self, $myconfig, $form) = @_;
+
+  # connect to database
+  my $dbh = $form->dbconnect($myconfig);
+
+  my %defaults = $form->get_defaults($dbh, \@{['precision']});
+  $form->{precision} = $defaults{precision};
+
+  my $query = qq|SELECT c.id, c.accno, c.description,
+                 l.description AS translation
+                 FROM chart c
+		 LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
+		 WHERE c.charttype = 'A'
+                 AND c.link LIKE '%$form->{ARAP}_paid%'
+                 AND c.closed = '0'
+		 ORDER BY c.accno|;
+  my $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+
+  my @accno;
+  my $ref;
+  my $arap;
+  my $ml = 1;
+
+  if ($form->{ARAP} eq 'AP') {
+    $arap = 'ap';
+  } else {
+    $arap = 'ar';
+    $ml = -1;
+  }
+
+  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+    $ref->{description} = $ref->{translation} if $ref->{translation};
+    push @accno, $ref;
+  }
+  $sth->finish;
+
+  $query = qq|SELECT acc.amount * $ml AS amount, acc.transdate AS datepaid,
+              acc.source, acc.trans_id,
+              vc.name, vc.id AS $form->{vc}_id
+              FROM acc_trans acc
+              JOIN $arap a ON (a.id = acc.trans_id)
+              JOIN $form->{vc} vc ON (vc.id = a.$form->{vc}_id)|;
+
+  my $where = "WHERE acc.fx_transaction = '0'
+               AND acc.chart_id = ?";
+
+  unless ($form->{datepaidfrom} || $form->{datepaidto}) {
+    ($form->{datepaidfrom}, $form->{datepaidto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
+  }
+  if ($form->{datepaidfrom}) {
+    $where .= " AND acc.transdate >= '$form->{datepaidfrom}'";
+  }
+  if ($form->{datepaidto}) {
+    $where .= " AND acc.transdate <= '$form->{datepaidto}'";
+  }
+  if ($form->{"$form->{vc}_id"}) {
+    $where .= qq| AND vc.id = $form->{"$form->{vc}_id"}|;
+  }
+  $query .= qq| $where ORDER BY acc.source|;
+
+  $sth = $dbh->prepare($query);
+
+  # check if void
+  $query = qq|SELECT SUM(acc.amount) * $ml AS amount, vc.id
+              FROM acc_trans acc
+              JOIN $arap a ON (a.id = acc.trans_id)
+              JOIN $form->{vc} vc ON (vc.id = a.$form->{vc}_id)
+              WHERE acc.fx_transaction = '0'
+              AND acc.source = ?
+              AND acc.chart_id = ?
+              GROUP BY vc.id|;
+  my $ach = $dbh->prepare($query);
+
+  # printed
+  my $formname = ($form->{ARAP} eq 'AP') ? 'check' : 'payment';
+  $query = qq|SELECT printed
+              FROM status
+              WHERE formname LIKE '$formname'
+              AND printed
+              AND trans_id = ?|;
+  my $pth = $dbh->prepare($query);
+
+  my $accno;
+  ($accno) = split /--/, $form->{accno};
+  if ($accno) {
+    @accno = grep { $_->{accno} eq $accno } @accno;
+  }
+
+  my $samesource;
+  my %source;
+  my %inv;
+  my (undef, $option) = split /--/, $form->{option};
+
+  for (@accno) {
+    $sth->execute($_->{id});
+
+    while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+      if ($form->{checknumberfrom}) {
+        next if $ref->{source} < $form->{checknumberfrom};
+      }
+      if ($form->{checknumberto}) {
+        next if $ref->{source} > $form->{checknumberto};
+      }
+
+      if ($option eq 'all') { 
+        if ($ref->{amount}) {
+          $ach->execute($ref->{source}, $_->{id});
+          ($amount) = $ach->fetchrow_array;
+          $ach->finish;
+          if ($form->round_amount($amount,10) == 0) {
+            $ref->{amount} = 0;
+            next if $inv{$ref->{trans_id}};
+            $inv{$ref->{trans_id}} = 1;
+            $ref->{void} = 1;
+          }
+        }
+      } elsif ($option eq 'void') {
+        $ach->execute($ref->{source}, $_->{id});
+        ($amount) = $ach->fetchrow_array;
+        $ach->finish;
+        next if ($ref->{amount} > 0);
+        $ref->{amount} = 0;
+        next if $form->round_amount($amount,10);
+      } elsif ($option eq 'exclude') {
+        $ach->execute($ref->{source}, $_->{id});
+        ($amount) = $ach->fetchrow_array;
+        $ach->finish;
+        next if ($form->round_amount($amount,10) == 0);
+      } elsif ($option eq 'printed') {
+        $pth->execute($ref->{trans_id});
+        ($ok) = $pth->fetchrow_array;
+        $pth->finish;
+        next unless $ok;
+
+        $ach->execute($ref->{source}, $_->{id});
+        ($amount) = $ach->fetchrow_array;
+        $ach->finish;
+        next if ($form->round_amount($amount,10) == 0);
+
+      } elsif ($option eq 'notprinted') {
+        $pth->execute($ref->{trans_id});
+        ($ok) = $pth->fetchrow_array;
+        $pth->finish;
+        next if $ok;
+
+        $ach->execute($ref->{source}, $_->{id});
+        ($amount) = $ach->fetchrow_array;
+        $ach->finish;
+        next if ($form->round_amount($amount,10) == 0);
+
+      }
+
+      $ref->{datetonum} = $form->datetonum($myconfig, $ref->{datepaid});
+
+      $form->{"$_->{accno}"} = "$_->{accno}--$_->{description}";
+
+      if ($ref->{source} && $ref->{source} eq $samesource) {
+        $i = @{ $source{$_->{accno}} };
+        $source{$_->{accno}}[$i-1]->{amount} += $ref->{amount};
+        $source{$_->{accno}}[$i-1]->{trans_id} .= "\n$ref->{trans_id}";
+      } else {
+        push @{ $source{$_->{accno}} }, $ref;
+      }
+
+      $samesource = $ref->{source};
+    }
+    $sth->finish;
+  }
+
+  for (keys %source) {
+    for $ref (sort { sortsource($a, $b, $form) } @{ $source{$_} }) {
+      push @{ $form->{CHK}{$_} }, $ref;
+    }
+  }
+
+  $dbh->disconnect;
+
+}
+
+
+sub sortsource {
+  my ($a, $b, $form) = @_;
+
+  if ($form->{sort} eq 'datepaid') {
+    if ($form->{direction} eq 'DESC') {
+      return $b->{datetonum} <=> $a->{datetonum};
+    }
+    return $a->{datetonum} <=> $b->{datetonum};
+  }
+
+  if ($form->{sort} eq 'source') {
+    if ($form->{direction} eq 'DESC') {
+      return $b->{source} <=> $a->{source};
+    }
+    return $a->{source} <=> $b->{source};
+  }
+
+  if ($form->{direction} eq 'DESC') {
+    return $b->{$form->{sort}} cmp $a->{$form->{sort}};
+  }
+  return $a->{$form->{sort}} cmp $b->{$form->{sort}};
+
+}
+
+
+sub void_payments {
+  my ($self, $myconfig, $form) = @_;
+
+  # connect to database
+  my $dbh = $form->dbconnect_noauto($myconfig);
+
+  my $query = qq|SELECT SUM(acc.amount), MAX(acc.id), acc.project_id,
+                 acc.chart_id
+                 FROM acc_trans acc
+                 JOIN chart c ON (c.id = acc.chart_id)
+                 WHERE fx_transaction = '0'
+                 AND c.accno = ?
+                 AND acc.trans_id = ?
+                 AND acc.source = ?
+                 GROUP BY acc.project_id, acc.chart_id|;
+  my $sth = $dbh->prepare($query) || $form->dberror($query);
+
+  $query = qq|INSERT INTO acc_trans
+              (trans_id, chart_id, amount, source, project_id, id)
+              VALUES (?, ?, ?, ?, ?, ?)|;
+  my $ath = $dbh->prepare($query) || $form->dberror($query);
+
+  my $arap = lc $form->{ARAP};
+
+  for (1 .. $form->{rowcount}) {
+    if ($form->{"id_$_"}) {
+      for my $trans_id (split /\n/, $form->{"id_$_"}) {
+        $sth->execute($form->{"accno_$_"}, $trans_id, $form->{"source_$_"});
+        my ($amount, $id, $project_id, $chart_id) = $sth->fetchrow_array;
+        $sth->finish;
+
+        $amount *= -1;
+        $id++;
+        $ath->execute($trans_id, $chart_id, $amount, $form->{"source_$_"}, $project_id, $id);
+        $ath->finish;
+
+        $form->update_balance($dbh,
+                              $arap,
+                              "paid",
+                              qq|id = $trans_id|,
+                              $amount);
+      }
+    }
+  }
+
+  my $rc = $dbh->commit;
+  $dbh->disconnect;
+
+  $rc;
+
+}
+
+
+sub create_selects {
+  my ($self, $myconfig, $form) = @_;
+
+  my ($query, $sth);
+
+  my $dbh = $form->dbconnect($myconfig);
+
+  $form->get_peripherals($dbh);
+
+  $form->all_languages($myconfig, $dbh);
+
+  $query = qq|SELECT id FROM chart WHERE accno = '$form->{"accno_1"}'|;
+  ($chart_id) = $dbh->selectrow_array($query);
+
+  my %defaults = $form->get_defaults($dbh, \@{["chknumber_${chart_id}"]});
+  $form->{source} = $defaults{"chknumber_${chart_id}"} + 1;
+
+  $dbh->disconnect;
+
+}
+
+
+sub reissue_payment {
+  my ($self, $myconfig, $form, $i) = @_;
+
+  my $dbh = $form->dbconnect_noauto($myconfig);
+
+  my %defaults = $form->get_defaults($dbh, \@{['precision']});
+  $form->{precision} = $defaults{precision};
+
+  my $query;
+  my $sth;
+  my $tth;
+  my $ath;
+  my $zth;
+  my $ref;
+  my $trans_id;
+
+  my $rc = 0;
+  my $arap = lc $form->{ARAP};
+
+  # retrieve name, address
+  $query = qq|SELECT vc.*, ad.*, current_date AS datepaid
+              FROM $form->{vc} vc
+              JOIN address ad ON (ad.trans_id = vc.id)
+              WHERE vc.id = $form->{"$form->{vc}_id"}|;
+  $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+  $ref = $sth->fetchrow_hashref(NAME_lc);
+  for (qw(name address1 address2 city state zipcode country datepaid)) { $form->{$_} = $ref->{$_} }
+  $sth->finish;
+
+  # invoices
+  $query = qq|SELECT a.*, a.transdate AS invdate
+              FROM $arap a
+              WHERE a.id = ?|;
+  $sth = $dbh->prepare($query);
+
+  $query = qq|SELECT id
+              FROM chart
+              WHERE accno = '$form->{"accno_$i"}'|;
+  my ($chart_id) = $dbh->selectrow_array($query);
+
+  # update acc_trans for each trans_id
+  $query = qq|UPDATE acc_trans SET
+              source = '$form->{source}',
+              transdate = '$form->{datepaid}'
+              WHERE source = '$form->{"source_$i"}'
+              AND fx_transaction = '0'
+              AND trans_id = ?
+              AND chart_id = $chart_id|;
+  $tth = $dbh->prepare($query);
+
+  $query = qq|UPDATE $arap
+              SET datepaid = '$form->{datepaid}'
+              WHERE id = ?|;
+  $ath = $dbh->prepare($query);
+
+  # amount of check for each invoice
+  $query = qq|SELECT SUM(amount)
+              FROM acc_trans
+              WHERE fx_transaction = '0'
+              AND trans_id = ?
+              AND source = '$form->{"source_$i"}'
+              AND chart_id = $chart_id|;
+  $zth = $dbh->prepare($query);
+
+  $form->{amount} = 0;
+  for (qw(invnumber invdescription invdate due paid)) { @{ $form->{$_} } = () }
+
+  for $trans_id (split /\n/, $form->{"id_$i"}) {
+
+    $zth->execute($trans_id);
+    $paid = 0;
+    while ($amount = $zth->fetchrow_array) {
+      $form->{amount} += $amount;
+      $paid += $amount;
+    }
+    $zth->finish;
+
+    $sth->execute($trans_id);
+
+    while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+      $ref->{due} = $form->format_amount($myconfig, $ref->{amount}, $form->{precision});
+      $ref->{paid} = $form->format_amount($myconfig, $paid, $form->{precision});
+      for (qw(invnumber invdescription invdate due paid)) { push @{ $form->{$_} }, $ref->{$_} }
+    }
+    $sth->finish;
+
+    # update source, datepaid
+    $tth->execute($trans_id);
+    $tth->finish;
+
+    # update datepaid
+    $ath->execute($trans_id);
+    $ath->finish;
+  }
+
+  $form->{"source_$i"} = $form->{source};
+
+  my $chkno = ($form->{ARAP} eq 'AP') ? qq|chknumber_$form->{"accno_$i"}| : qq|receipt_$form->{"accno_$i"}|;
+  # record last check number
+  $form->update_defaults($myconfig, $chkno, $dbh, $form->{source});
+
+  my $rc = $dbh->commit;
+  $dbh->disconnect;
+
+  $rc;
+
+}
+
+
 
 1;
 
