@@ -20,6 +20,8 @@ sub delete_transaction {
   # connect to database
   my $dbh = $form->dbconnect_noauto($myconfig);
   
+  $form->{id} *= 1;
+  
   my %audittrail = ( tablename  => 'gl',
                      reference  => $form->{reference},
 		     formname   => 'transaction',
@@ -28,7 +30,7 @@ sub delete_transaction {
  
   $form->audittrail($dbh, "", \%audittrail);
 
-  if ($form->{batchid}) {
+  if ($form->{batchid} *= 1) {
     $query = qq|SELECT sum(amount)
 		FROM acc_trans
 		WHERE trans_id = $form->{id}
@@ -55,8 +57,8 @@ sub delete_transaction {
   
   my $id;
   for $id (qw(id apid)) {
-    for (qw(acc_trans dpt_trans yearend pay_trans status reference)) {
-      if ($form->{$id}) {
+    for (qw(acc_trans dpt_trans yearend pay_trans status)) {
+      if ($form->{$id} *= 1) {
 	$query = qq|DELETE FROM $_ WHERE trans_id = $form->{$id}|;
 	$dbh->do($query) || $form->dberror($query);
       }
@@ -67,7 +69,9 @@ sub delete_transaction {
     $query = qq|DELETE FROM $_ WHERE id = $form->{id}|;
     $dbh->do($query) || $form->dberror($query);
   }
-  
+
+  $form->delete_references($dbh);
+
   $form->remove_locks($myconfig, $dbh, 'gl');
 
   # commit and redirect
@@ -82,7 +86,6 @@ sub delete_transaction {
 sub post_transaction {
   my ($self, $myconfig, $form, $dbh) = @_;
   
-  my $null;
   my $project_id;
   my $department_id;
   my $i;
@@ -104,10 +107,10 @@ sub post_transaction {
   my %defaults = $form->get_defaults($dbh, \@{['precision']});
   $form->{precision} = $defaults{precision};
 
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     $keepcleared = 1;
     
-    if ($form->{batchid}) {
+    if ($form->{batchid} *= 1) {
       $query = qq|SELECT * FROM vr
 		  WHERE trans_id = $form->{id}|;
       $sth = $dbh->prepare($query) || $form->dberror($query);
@@ -141,13 +144,13 @@ sub post_transaction {
 
     if ($form->{id}) {
       # delete individual transactions
-      for (qw(acc_trans dpt_trans reference)) {
+      for (qw(acc_trans dpt_trans)) {
 	$query = qq|DELETE FROM $_ WHERE trans_id = $form->{id}|;
 	$dbh->do($query) || $form->dberror($query);
       }
     }
   }
-  
+
   if (!$form->{id}) {
    
     my $uid = localtime;
@@ -158,13 +161,13 @@ sub post_transaction {
 		                 WHERE login = '$form->{login}'),
 		'$approved')|;
     $dbh->do($query) || $form->dberror($query);
-    
+
     $query = qq|SELECT id FROM gl
                 WHERE reference = '$uid'|;
     ($form->{id}) = $dbh->selectrow_array($query);
   }
   
-  ($null, $department_id) = split /--/, $form->{department};
+  (undef, $department_id) = split /--/, $form->{department};
   $department_id *= 1;
 
   $form->{reference} = $form->update_defaults($myconfig, 'glnumber', $dbh) unless $form->{reference};
@@ -219,7 +222,7 @@ sub post_transaction {
     }
 
     # add the record
-    ($null, $project_id) = split /--/, $form->{"projectnumber_$i"};
+    (undef, $project_id) = split /--/, $form->{"projectnumber_$i"};
     $project_id ||= 'NULL';
     
     if ($keepcleared) {
@@ -246,26 +249,26 @@ sub post_transaction {
 
       if ($form->{currency} ne $form->{defaultcurrency}) {
 
-	$amount = $form->round_amount($amount * ($form->{exchangerate} - 1), $form->{precision});
+				$amount = $form->round_amount($amount * ($form->{exchangerate} - 1), $form->{precision});
 	
-	if ($amount) {
-	  $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, transdate,
-		      source, project_id, fx_transaction, memo, cleared, approved)
-		      VALUES
-		      ($form->{id}, (SELECT id
-				     FROM chart
-				     WHERE accno = '$accno'),
-		       $amount, '$form->{transdate}', |.
-		       $dbh->quote($form->{"source_$i"}) .qq|,
-		      $project_id, '1', |.$dbh->quote($form->{"memo_$i"}).qq|,
-		      $cleared, '$approved')|;
-	  $dbh->do($query) || $form->dberror($query);
-	}
+				if ($amount) {
+					$query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, transdate,
+								source, project_id, fx_transaction, memo, cleared, approved)
+								VALUES
+								($form->{id}, (SELECT id
+									 FROM chart
+									 WHERE accno = '$accno'),
+								 $amount, '$form->{transdate}', |.
+								 $dbh->quote($form->{"source_$i"}) .qq|,
+								$project_id, '1', |.$dbh->quote($form->{"memo_$i"}).qq|,
+								$cleared, '$approved')|;
+					$dbh->do($query) || $form->dberror($query);
+				}
       }
     }
   }
 
-  if ($form->{batchid}) {
+  if ($form->{batchid} *= 1) {
     # add voucher
     $form->{voucher}{transaction}{vouchernumber} = $form->update_defaults($myconfig, 'vouchernumber', $dbh) unless $form->{voucher}{transaction}{vouchernumber};
 
@@ -284,7 +287,7 @@ sub post_transaction {
   }
 
   # save reference documents
-  $form->save_reference($dbh);
+  $form->save_reference($dbh, 'gl');
     
   my %audittrail = ( tablename  => 'gl',
                      reference  => $form->{reference},
@@ -319,7 +322,6 @@ sub transactions {
   my $query;
   my $sth;
   my $var;
-  my $null;
   
   my %defaults = $form->get_defaults($dbh, \@{['precision', 'company']});
   for (keys %defaults) { $form->{$_} = $defaults{$_} }
@@ -351,7 +353,7 @@ sub transactions {
     $apwhere .= " AND lower(ct.vendornumber) LIKE '$var'";
   }
   if ($form->{department}) {
-    ($null, $var) = split /--/, $form->{department};
+    (undef, $var) = split /--/, $form->{department};
     $glwhere .= " AND g.department_id = $var";
     $arwhere .= " AND a.department_id = $var";
     $apwhere .= " AND a.department_id = $var";
@@ -425,8 +427,10 @@ sub transactions {
     $arwhere .= " AND lower(ac.memo) LIKE '$var'";
     $apwhere .= " AND lower(ac.memo) LIKE '$var'";
   }
-  
-  ($form->{datefrom}, $form->{dateto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
+
+  unless ($form->{datefrom} || $form->{dateto}) {
+    ($form->{datefrom}, $form->{dateto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
+  }
   
   if ($form->{datefrom}) {
     $glwhere .= " AND ac.transdate >= '$form->{datefrom}'";
@@ -468,6 +472,8 @@ sub transactions {
     $glwhere .= " AND c.category = '$form->{category}'";
     $arwhere .= " AND c.category = '$form->{category}'";
     $apwhere .= " AND c.category = '$form->{category}'";
+
+    delete $form->{l_contra};
   }
 
   if ($form->{accno} || $form->{gifi_accno}) {
@@ -543,7 +549,7 @@ sub transactions {
   
 
   my $false = ($myconfig->{dbdriver} =~ /Pg/) ? FALSE : q|'0'|;
-
+ 
   my $query = qq|SELECT g.id, 'gl' AS type, $false AS invoice, g.reference,
                  g.description, ac.transdate, ac.source,
 		 ac.amount, c.accno, c.gifi_accno, g.notes, c.link,
@@ -591,7 +597,7 @@ sub transactions {
 		 JOIN address ad ON (ad.trans_id = ct.id)
 		 LEFT JOIN department d ON (d.id = a.department_id)
 		 WHERE $apwhere|;
-
+ 
   my @sf = qw(id transdate reference accno);
   my %ordinal = $form->ordinal_order($dbh, $query);
   $query .= qq| ORDER BY | .$form->sort_order(\@sf, \%ordinal);
@@ -640,6 +646,7 @@ sub transactions {
     for (qw(address1 address2 city zipcode country)) { $ref->{address} .= "$ref->{$_} " }
 
     $trans{$ref->{id}}{$i} = {
+                 transdate => $ref->{transdate},
                       link => $ref->{link},
                       type => $ref->{type},
                      accno => $ref->{accno},
@@ -649,7 +656,7 @@ sub transactions {
                     amount => $ref->{debit} + $ref->{credit}
 		             };
     push @{ $form->{GL} }, $ref;
-    
+
     $i++;
     
   }
@@ -679,7 +686,6 @@ sub transactions {
       my %accno = ();
       my $aa = 0;
       my $j;
-      my %seen = ();
 
       for $i (reverse sort { $trans{$id}{$a}{amount} <=> $trans{$id}{$b}{amount} } keys %{$trans{$id}}) {
 
@@ -697,6 +703,7 @@ sub transactions {
 	  } else {
 	    push @accno, { accno => $trans{$id}{$i}{accno},
 		      gifi_accno => $trans{$id}{$i}{gifi_accno},
+                       transdate => $trans{$id}{$i}{transdate},
 			       i => $i };
 	  }
 	}
@@ -715,10 +722,11 @@ sub transactions {
 	for (@arap) {
 	  $i = 0;
 	  for $ref (@accno) {
-	    $form->{GL}[$_]{contra} .= "$ref->{accno} " unless $seen{$ref->{accno}};
-	    $seen{$ref->{accno}} = 1;
-	    $form->{GL}[$_]{gifi_contra} .= "$ref->{gifi_accno} " unless $seen{$ref->{gifi_accno}};
-	    $seen{$ref->{gifi_accno}} = 1;
+	    $form->{GL}[$_]{contra} .= "$ref->{accno} " unless $seen{"$ref->{accno}$ref->{transdate}"};
+	    $seen{"$ref->{accno}$ref->{transdate}"} = 1;
+
+	    $form->{GL}[$_]{gifi_contra} .= "$ref->{gifi_accno} " unless $seen{"$ref->{gifi_accno}$ref->{transdate}"};
+	    $seen{"$ref->{gifi_accno}$ref->{transdate}"} = 1;
 	  }
 	  $i++;
 	}
@@ -729,15 +737,12 @@ sub transactions {
       } else {
 	
 	%accno = %{$trans{$id}};
-	$j = 0;
-	
+
 	for $i (reverse sort { $trans{$id}{$a}{amount} <=> $trans{$id}{$b}{amount} } keys %{$trans{$id}}) {
 	  $found = 0;
 	  $amount = $trans{$id}{$i}{amount};
-	  $accno = $trans{$id}{$i}{accno};
-	  $gifi_accno = $trans{$id}{$i}{gifi_accno};
 	  $j = $i;
-	  
+
 	  if ($trans{$id}{$i}{debit}) {
 	    $amt = "debit";
 	    $rev = "credit";
@@ -745,12 +750,12 @@ sub transactions {
 	    $amt = "credit";
 	    $rev = "debit";
 	  }
-	  
-	  if ($trans{$id}{$i}{$amt}) {
+
+	  if ($amount) {
 	    for (keys %accno) {
-	      if ($accno{$_}{$rev} == $trans{$id}{$i}{$amt}) {
-		$form->{GL}[$_]{contra} = $trans{$id}{$i}{accno};
-		$form->{GL}[$_]{gifi_contra} = $trans{$id}{$i}{gifi_accno};
+	      if ($accno{$_}{$rev} == $amount) {
+		$form->{GL}[$i]{contra} = $accno{$_}{accno};
+		$form->{GL}[$i]{gifi_contra} = $accno{$_}{gifi_accno};
 		$found = 1;
 		last;
 	      }
@@ -758,25 +763,21 @@ sub transactions {
 	  }
 
 	  if (!$found) {
-	    delete $accno{$j};
-	    delete $trans{$id}{$j};
-	    
 	    if ($amount) {
-	      for $i (reverse sort { $a{amount} <=> $b{amount} } keys %accno) {
-		if ($accno{$i}{amount} <= $amount) {
-		  $form->{GL}[$i]{contra} = $accno;
-		  $form->{GL}[$i]{gifi_contra} = $gifi_accno;
-		  $amount = $form->round_amount($amount - $accno{$i}{amount}, 10);
-		  last if $amount < 0;
+	      for $i (reverse sort { $accno{$a}{amount} <=> $accno{$b}{amount} } keys %accno) {
+		if ($accno{$i}{$rev}) {
+
+                  # add contra to accno
+		  $form->{GL}[$j]{contra} .= "$accno{$i}{accno} ";
+		  $form->{GL}[$j]{gifi_contra} .= "$accno{$i}{gifi_accno} ";
+
+		  $amount = $form->round_amount($amount - $accno{$i}{$rev}, 10);
+                  last if $amount <= 0;
 
-		  $form->{GL}[$j]{contra} .= "$accno{$i}{accno} " unless $seen{$accno{$i}{accno}};
-		  $seen{$accno{$i}{accno}};
-		  $form->{GL}[$j]{gifi_contra} .= "$accno{$i}{gifi_accno} " unless $seen{$accno{$i}{gifi_accno}};
-		  $seen{$accno{$i}{gifi_accno}};
-		  delete $accno{$i};
-		  delete $trans{$id}{$i};
 		}
 	      }
+              $form->{GL}[$j]{contra} = join ' ', sort split / /, $form->{GL}[$j]{contra};
+              $form->{GL}[$j]{gifi_contra} = join ' ', sort split / /, $form->{GL}[$j]{gifi_contra};
 	    }
 	  }
 	}
@@ -800,12 +801,12 @@ sub transaction {
 
   $form->remove_locks($myconfig, $dbh, 'gl');
   
-  my %defaults = $form->get_defaults($dbh, \@{[qw(closedto revtrans precision referenceurl)]});
+  my %defaults = $form->get_defaults($dbh, \@{[qw(closedto revtrans precision referenceurl lock_%)]});
   for (keys %defaults) { $form->{$_} = $defaults{$_} }
 
-  $form->{currencies} = $form->get_currencies($dbh, $myconfig);
+  $form->{currencies} = $form->get_currencies($myconfig, $dbh);
   
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     $query = qq|SELECT g.*, 
                 d.description AS department,
 		br.id AS batchid, br.description AS batchdescription
@@ -861,7 +862,7 @@ sub transaction {
     # get recurring transaction
     $form->get_recurring($dbh);
 
-    $form->get_reference($dbh);
+    $form->all_references($dbh);
 
     $form->create_lock($myconfig, $dbh, $form->{id}, 'gl');
 
@@ -873,9 +874,10 @@ sub transaction {
   $query = qq|SELECT c.accno, c.description,
               l.description AS translation
               FROM chart c
-	      LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
-	      WHERE c.charttype = 'A'
-              ORDER by c.accno|;
+              LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
+              WHERE c.charttype = 'A'
+              AND c.closed = '0'
+              ORDER by 1|;
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
   
