@@ -26,8 +26,8 @@ sub get_employee {
   my $ref;
   my $notid = "";
   my $rne;
-  
-  my @df = qw(closedto revtrans company address tel fax businessnumber precision referenceurl);
+
+  my @df = qw(closedto revtrans company address tel fax businessnumber precision referenceurl lock_%);
   my %defaults = $form->get_defaults($dbh, \@df);
   for (keys %defaults) { $form->{$_} = $defaults{$_} }
 
@@ -42,12 +42,12 @@ sub get_employee {
     }
   }
 
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     $query = qq|SELECT e.*,
                 ad.id AS addressid, ad.address1, ad.address2, ad.city,
 		ad.state, ad.zipcode, ad.country,
-		bk.name AS bankname, bk.iban, bk.bic, bk.membernumber,
-                bk.clearingnumber,
+		bk.name AS bankname, bk.iban, bk.bic,
+                bk.membernumber, bk.clearingnumber,
 		ad1.address1 AS bankaddress1,
 		ad1.address2 AS bankaddress2,
 		ad1.city AS bankcity,
@@ -85,16 +85,16 @@ sub get_employee {
     # check if employee can be deleted, orphaned
     $form->{status} = "";
     $query = qq|SELECT count(*) FROM ap
-                WHERE vendor_id = $form->{id}|;
+		WHERE vendor_id = $form->{id}|;
     if (! $dbh->selectrow_array($query)) {
       $form->{status} = "orphaned";
     }
-                
+		
     if (! $form->{status}) {
       $query = qq|SELECT count(*) FROM jcitems
-                  WHERE employee_id = $form->{id}|;
+		  WHERE employee_id = $form->{id}|;
       if (! $dbh->selectrow_array($query)) {
-        $form->{status} = "orphaned";
+	$form->{status} = "orphaned";
       }
     }
    
@@ -176,6 +176,7 @@ sub get_employee {
       ($cd) = $dbh->selectrow_array($query);
     }
 
+    %deduct = ();
     if ($form->{trans_id}) {
       $query = qq|SELECT *
                   FROM pay_trans
@@ -205,13 +206,7 @@ sub get_employee {
       $sth->finish;
     }
     
-    $query = qq|SELECT a.rn
-		FROM acsrole a
-		JOIN employee e ON (e.acsrole_id = a.id)
-		WHERE e.id = $form->{id}|;
-    ($rne) = $dbh->selectrow_array($query);
-   
-    $form->get_reference($dbh);
+    $form->all_references($dbh);
 
     $form->create_lock($myconfig, $dbh, $form->{id}, 'hr') unless $nolock;
  
@@ -221,33 +216,7 @@ sub get_employee {
   
   }
 
-  my $login = $form->{login};
-  $login =~ s/\@.*//;
-  $query = qq|SELECT a.rn
-              FROM acsrole a
-	      JOIN employee e ON (e.acsrole_id = a.id)
-	      WHERE e.login = '$login'|;
-  my ($rnl) = $dbh->selectrow_array($query);
-
-  $rnl *= 1;
-  if ($rnl) {
-    $form->{admin} = ($rne) ? $rnl < $rne : 1;
-  }
-  $form->{admin} = 1 if $login eq 'admin';
-
-  # get acsrole
-  $query = qq|SELECT id, description
-              FROM acsrole
-	      WHERE rn > $rnl
-	      ORDER BY rn|;
-  $sth = $dbh->prepare($query);
-  $sth->execute || $form->dberror($query);
-
-  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
-    push @{ $form->{all_acsrole} }, $ref;
-  }
-  $sth->finish;
-
+  HR->isadmin($myconfig, $form, $dbh);
   
   # get wages
   $query = qq|SELECT *
@@ -298,6 +267,7 @@ sub get_employee {
 		WHERE c.charttype = 'A'
 		AND c.category = '$ae{$_}{category}'
 		$ae{$_}{link}
+                AND c.closed = '0'
 		ORDER BY c.accno|;
     $sth = $dbh->prepare($query);
     $sth->execute || $form->dberror($query);
@@ -327,7 +297,76 @@ sub get_employee {
 }
 
 
+sub isadmin {
+  my ($self, $myconfig, $form, $dbh) = @_;
 
+  my $disconnect = ($dbh) ? 0 : 1;;
+  
+  # connect to database
+  $dbh = $form->dbconnect($myconfig) unless $dbh;
+
+  $form->{id} *= 1;
+  my $rne;
+
+  my $query = qq|SELECT a.rn
+		 FROM acsrole a
+		 JOIN employee e ON (e.acsrole_id = a.id)
+		 WHERE e.id = $form->{id}|;
+  ($rne) = $dbh->selectrow_array($query);
+   
+  my $login = $form->{login};
+  $login =~ s/\@.*//;
+  $query = qq|SELECT a.rn
+              FROM acsrole a
+	      JOIN employee e ON (e.acsrole_id = a.id)
+	      WHERE e.login = '$login'|;
+  my ($rnl) = $dbh->selectrow_array($query);
+
+  $rnl *= 1;
+  if ($rnl) {
+    $form->{admin} = ($rne) ? $rnl < $rne : 1;
+  }
+  $form->{admin} = 1 if $login eq 'admin';
+
+  # get acsrole
+  $query = qq|SELECT id, description
+              FROM acsrole
+	      WHERE rn > $rnl
+	      ORDER BY rn|;
+  $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+
+  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+    push @{ $form->{all_acsrole} }, $ref;
+  }
+  $sth->finish;
+
+  $dbh->disconnect if $disconnect;
+
+}
+
+
+sub acsrole {
+  my ($self, $myconfig, $form) = @_;
+  
+  my $dbh = $form->dbconnect($myconfig);
+  
+  my $id;
+  (undef, $id) = split /--/, $form->{acsrole};
+  $id *= 1;
+  
+  my $query = qq|SELECT acs
+	         FROM acsrole
+	         WHERE id = $id|;
+  ($_) = $dbh->selectrow_array($query);
+
+  $dbh->disconnect;
+
+  $_;
+
+}
+
+ 
 sub save_employee {
   my ($self, $myconfig, $form) = @_;
 
@@ -339,19 +378,19 @@ sub save_employee {
   $form->remove_locks($myconfig, $dbh, 'hr');
 
   my $bank_address_id;
-  my $null;
   my $employeelogin;
   my $sales;
   my $acsrole_id;
   my $rn;
 
-  for (qw(paymentmethod acsrole)) { ($null, $form->{"${_}_id"}) = split /--/, $form->{$_} }
-
+  for (qw(paymentmethod acsrole)) { (undef, $form->{"${_}_id"}) = split /--/, $form->{$_} }
+  
   if ($form->{id} *= 1) {
     $query = qq|SELECT e.login, e.sales, e.acsrole_id, a.rn
-		FROM employee e
+                FROM employee e
                 LEFT JOIN acsrole a ON (e.acsrole_id = a.id)
-		WHERE e.id = $form->{id}|;
+                WHERE e.id = $form->{id}|;
+
     ($employeelogin, $sales, $acsrole_id, $rn) = $dbh->selectrow_array($query);
 
     my $login = $form->{login};
@@ -402,10 +441,6 @@ sub save_employee {
                 WHERE trans_id = $form->{id}|;
     $dbh->do($query) || $form->dberror($query);
     
-    $query = qq|DELETE FROM reference
-                WHERE trans_id = $form->{id}|;
-    $dbh->do($query) || $form->dberror($query);
-    
   } else {
     my $uid = localtime;
     $uid .= $$;
@@ -426,7 +461,6 @@ sub save_employee {
   $form->{employeenumber} = $form->update_defaults($myconfig, "employeenumber", $dbh) if ! $form->{employeenumber};
 
   for (qw(ap payment)) { ($form->{$_}) = split /--/, $form->{$_} }
-
   $form->{acsrole_id} ||= 'NULL';
   $form->{sales} *= 1;
   
@@ -583,7 +617,7 @@ sub save_employee {
   my $sth = $dbh->prepare($query) || $form->dberror($query);
 
   for ($i = 1; $i <= $form->{wage_rows}; $i++) {
-    ($null, $wage_id) = split /--/, $form->{"wage_$i"};
+    (undef, $wage_id) = split /--/, $form->{"wage_$i"};
     if ($wage_id) {
       $sth->execute($i,$wage_id) || $form->dberror($query);
     }
@@ -600,7 +634,7 @@ sub save_employee {
 
   for ($i = 1; $i <= $form->{deduction_rows}; $i++) {
     for (qw(exempt maximum)) { $form->{"${_}_$i"} = $form->parse_amount($myconfig, $form->{"${_}_$i"}) }
-    ($null, $deduction_id) = split /--/, $form->{"deduction_$i"};
+    (undef, $deduction_id) = split /--/, $form->{"deduction_$i"};
     if ($deduction_id) {
       $sth->execute($i, $deduction_id, $form->{"exempt_$i"}, $form->{"maximum_$i"}) || $form->dberror($query);
     }
@@ -619,8 +653,17 @@ sub save_employee {
     }
   }
   $sth->finish;
-
-  $form->save_reference($dbh);
+  
+  my %audittrail = ( tablename  => 'employee',
+                     reference => $form->{name},
+		     formname => '',
+		     action     => 'saved',
+		     id => $form->{id}
+		   );
+  
+  $form->audittrail($dbh, "", \%audittrail);
+ 
+  $form->save_reference($dbh, 'employee');
   
   my $rc = $dbh->commit;
   $dbh->disconnect;
@@ -638,6 +681,8 @@ sub delete_employee {
 
   my $query;
 
+  for (qw(id db)) { $form->{$_} =~ s/;//g }
+  
   $query = qq|SELECT address_id
               FROM bank
               WHERE id = $form->{id}|;
@@ -669,16 +714,19 @@ sub delete_employee {
               WHERE trans_id = $form->{id}|;
   $dbh->do($query) || $form->dberror($query);
   
-  $query = qq|DELETE FROM reference
-              WHERE trans_id = $form->{id}|;
-  $dbh->do($query) || $form->dberror($query);
- 
-  $query = qq|DELETE FROM audittrail
-	      WHERE employee_id = $form->{id}|;
-  $dbh->do($query) || $form->dberror($query);
-
+  $form->delete_references($dbh);
+  
   $form->remove_locks($myconfig, $dbh, 'hr');
- 
+
+  my %audittrail = ( tablename  => 'employee',
+                     reference => $login,
+		     formname => '',
+		     action     => 'deleted',
+		     id => $form->{id}
+		   );
+  
+  $form->audittrail($dbh, "", \%audittrail);
+  
   my $rc = $dbh->commit;
   $dbh->disconnect;
 
@@ -699,7 +747,7 @@ sub employees {
   my $where = "1 = 1";
 
   $form->{sort} ||= "name";
-
+  
   my $var;
   
   for (qw(name employeenumber notes)) {
@@ -708,7 +756,10 @@ sub employees {
       $where .= " AND lower(e.$_) LIKE '$var'";
     }
   }
- 
+  if ($form->{employeelogin} ne "") {
+    $var = $form->like(lc $form->{employeelogin});
+    $where .= " AND lower(e.login) LIKE '$var'";
+  }
   if ($form->{startdatefrom}) {
     $where .= " AND e.startdate >= '$form->{startdatefrom}'";
   }
@@ -730,17 +781,29 @@ sub employees {
   if ($form->{status} eq 'inactive') {
     $where .= qq| AND e.enddate <= current_date|;
   }
+  if ($form->{acsrole}) {
+    (undef, $var) = split /--/, $form->{acsrole};
+    $where .= qq| AND r.id = $var|;
+  }
 
-  my $query = qq|SELECT e.*, 
-                 ad.address1, ad.address2, ad.city, ad.state,
-		 ad.zipcode, ad.country,
-		 bk.iban, bk.bic,
-		 r.description AS acsrole
-                 FROM employee e
-		 LEFT JOIN address ad ON (ad.trans_id = e.id)
-		 LEFT JOIN bank bk ON (bk.id = e.id)
-		 LEFT JOIN acsrole r ON (r.id = e.acsrole_id)
-                 WHERE $where|;
+  my $payrolljoin = qq|LEFT JOIN employeewage ew ON (e.id = ew.employee_id)|;
+
+  if ($form->{status} eq 'payroll') {
+    $payrolljoin = qq|JOIN employeewage ew ON (e.id = ew.employee_id)|;
+  }
+
+  $query = qq|SELECT DISTINCT e.*, 
+              ad.address1, ad.address2, ad.city, ad.state,
+              ad.zipcode, ad.country,
+              bk.iban, bk.bic,
+              r.description AS acsrole,
+              ew.employee_id AS payroll
+              FROM employee e
+              LEFT JOIN address ad ON (ad.trans_id = e.id)
+              LEFT JOIN bank bk ON (bk.id = e.id)
+              LEFT JOIN acsrole r ON (r.id = e.acsrole_id)
+              $payrolljoin
+              WHERE $where|;
 
   my @sf = qw(name);
   my %ordinal = $form->ordinal_order($dbh, $query);
@@ -817,7 +880,7 @@ sub payroll_links {
   $form->remove_locks($myconfig, $dbh);
   
   my @var;
-  
+
   my @df = qw(closedto revtrans company precision namesbynumber referenceurl);
   my %defaults = $form->get_defaults($dbh, \@df);
   for (keys %defaults) { $form->{$_} = $defaults{$_} }
@@ -840,7 +903,7 @@ sub payroll_links {
 
   $form->{datepaid} = $form->{transdate} = $form->current_date($myconfig);
 
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     $query = qq|SELECT a.*, e.name AS employee, e.payperiod,
                 d.description AS department,
 		c1.accno AS ap_accno, c1.description AS ap_accno_description,
@@ -861,7 +924,14 @@ sub payroll_links {
     $sth->execute || $form->dberror($query);
     $ref = $sth->fetchrow_hashref(NAME_lc);
     delete $ref->{id};
-    
+
+    # projectnumber
+    $query = qq~SELECT p.projectnumber || '--' || p.id
+                FROM project p
+                JOIN acc_trans a ON (a.project_id = p.id)
+                WHERE a.trans_id = $form->{id}~;
+    ($form->{project}) = $dbh->selectrow_array($query);
+
     for (qw(ap payment)) {
       $ref->{$_} = qq|$ref->{"${_}_accno"}--$ref->{"${_}_accno_description"}|;
       $ref->{$_} = qq|$ref->{"${_}_accno"}--$ref->{"${_}_accno_description"}| if $ref->{"${_}_accno_translation"};
@@ -972,7 +1042,7 @@ sub payroll_links {
     $sth->finish;
     $form->{payrate_rows} = $i;
 
-    $form->get_reference($dbh);
+    $form->all_references($dbh);
 
     $form->create_lock($myconfig, $dbh, $form->{id}, 'hr');
     
@@ -982,6 +1052,7 @@ sub payroll_links {
 	      FROM chart c
 	      LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
 	      WHERE c.link = 'AP'
+              AND c.closed = '0'
 	      ORDER BY c.accno|;
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
@@ -996,6 +1067,7 @@ sub payroll_links {
 	      FROM chart c
 	      LEFT JOIN translation l ON (l.trans_id = c.id AND l.language_code = '$myconfig->{countrycode}')
 	      WHERE c.link LIKE '%AP_paid%'
+              AND c.closed = '0'
 	      ORDER BY c.accno|;
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
@@ -1086,7 +1158,7 @@ sub payroll_transactions {
 
   my $where = "1 = 1";
   my $acwhere = "1 = 1";
-
+  
   $form->{sort} ||= "employee";
 
   my $id;
@@ -1110,7 +1182,9 @@ sub payroll_transactions {
                 AND a.paymentmethod_id = $id|;
   }
 
-  ($form->{transdatefrom}, $form->{transdateto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
+  unless ($form->{transdatefrom} || $form->{transdateto}) {
+    ($form->{transdatefrom}, $form->{transdateto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
+  }
   
   $where .= " AND a.transdate >= '$form->{transdatefrom}'" if $form->{transdatefrom};
   $where .= " AND a.transdate <= '$form->{transdateto}'" if $form->{transdateto};
@@ -1158,7 +1232,7 @@ sub payroll_transactions {
 
     while ($dref = $wth->fetchrow_hashref(NAME_lc)) {
       $ref->{glid} = $dref->{glid};
-      if ($ref->{$dref->{id}} = $dref->{amount} * $dref->{qty}) {
+      if ($ref->{$dref->{id}} = $form->round_amount($dref->{amount} * $dref->{qty}, $form->{precision})) {
 	$form->{$dref->{id}} = 1;
 	$ok = 1;
       }
@@ -1200,7 +1274,6 @@ sub payroll_transactions {
 }
 
 
-
 sub payslip_details {
   my ($self, $myconfig, $form) = @_;
 
@@ -1209,13 +1282,14 @@ sub payslip_details {
   my $query;
   my $sth;
   my $ref;
+  my $id;
   
   my %defaults = $form->get_defaults($dbh, \@{['company', 'address', 'tel', 'fax', 'companyemail', 'companywebsite', 'businessnumber', 'precision', 'referenceurl']});
   for (keys %defaults) { $form->{$_} = $defaults{$_} }
 
   %defaults = $form->get_defaults($dbh, \@{['printer_%']});
   
-  my ($null, $id) = split /--/, $form->{employee};
+  (undef, $id) = split /--/, $form->{employee};
 
   if ($id *= 1) {
     $query = qq|SELECT e.*,
@@ -1250,7 +1324,7 @@ sub payslip_details {
     $sth->finish;
   }
 
-  ($null, $id) = split /--/, $form->{paymentmethod};
+  (undef, $id) = split /--/, $form->{paymentmethod};
   if ($id *= 1) {
     $query = qq|SELECT fee, roundchange
                 FROM paymentmethod
@@ -1319,7 +1393,6 @@ sub payslip_details {
 }
 
 
-
 sub post_transaction {
   my ($self, $myconfig, $form) = @_;
   
@@ -1377,7 +1450,7 @@ sub post_transaction {
 
     $ref = $sth->fetchrow_hashref(NAME_lc);
 
-    ($null, $user_id) = $form->get_employee($dbh);
+    (undef, $user_id) = $form->get_employee($dbh);
     my $vendornumber = $form->update_defaults($myconfig, 'vendornumber');
     
     $query = qq|INSERT INTO vendor (id, name, phone,
@@ -1388,8 +1461,8 @@ sub post_transaction {
                 VALUES (
                 $employee_id, |
                 .$dbh->quote($employee).qq|, '$ref->{workphone}',
-		'$ref->{workfax}', '$ref->{email}', '$ref->{notes}',
-		$vendornumber, $user_id, '$ap->{currency}',|
+		'$ref->{workfax}', '$ref->{email}', '$ref->{notes}', |
+		.$dbh->quote($vendornumber).qq|, $user_id, '$ap->{currency}',|
 		.$form->dbquote($ref->{startdate}, SQL_DATE).qq|, |
 		.$form->dbquote($ref->{enddate}, SQL_DATE).qq|, |
 		.$dbh->quote($ref->{apid}).qq|, |
@@ -1407,7 +1480,7 @@ sub post_transaction {
     $sth->finish;
   }
   
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     $query = qq|SELECT pt.glid, g.reference
 		FROM pay_trans pt
 		JOIN ap a ON (a.id = pt.trans_id)
@@ -1419,18 +1492,15 @@ sub post_transaction {
                 WHERE trans_id = $form->{id}|;
     $dbh->do($query) || $form->dberror($query);
   }
- 
-  $ap->{login} = $form->{login};
-  $ap->{id} = $form->{id};
-  $ap->{invnumber} = $form->{invnumber};
+
+  delete $form->{invnumber} if $form->{postasnew};
+  for (qw(login id invnumber department transdate description language_code)) { $ap->{$_} = $form->{$_} }
+
   $ap->{vendor_id} = $employee_id; 
   $ap->{defaultcurrency} = $ap->{currency};
-  $ap->{department} = $form->{department};
   $ap->{vc} = 'vendor';
-  $ap->{transdate} = $form->{transdate};
   $ap->{duedate} = $form->{transdate};
   $ap->{AP} = $form->{ap};
-  $ap->{description} = $form->{description};
   
   my $i = 0;
   my $j;
@@ -1455,8 +1525,8 @@ sub post_transaction {
 	($defer) = $cth->fetchrow_array;
 	$cth->finish;
 	
-	if ($amount = $form->round_amount($form->{"pay_$j"}, 10)) {
-	  push @employerpays, { employeraccno => $defer, employeeaccno => $accno, description => $form->{"wage_$j"}, amount => $amount };
+	if ($form->{"pay_$j"}) {
+	  push @employerpays, { employeraccno => $defer, employeeaccno => $accno, description => $form->{"wage_$j"}, amount => $form->{"pay_$j"} };
 	}
 	
       } else {
@@ -1471,7 +1541,6 @@ sub post_transaction {
       }
       $wth->finish;
 
-      
     }
   }
   
@@ -1507,7 +1576,7 @@ sub post_transaction {
       $amount = $form->parse_amount($myconfig, $form->{"deduct_$j"}) * $ref->{employerpays};
 
       if ($amount) {
-	push @employerpays, { employeraccno => $employeraccno, employeeaccno => $accno, description => $form->{"deduction_$j"}, amount => $amount };
+	push @employerpays, { employeraccno => $employeraccno, employeeaccno => $accno, description => $form->{"deduction_$j"}, amount => $form->format_amount($myconfig, $amount, $form->{precision}) };
       }
       
     }
@@ -1525,18 +1594,14 @@ sub post_transaction {
   $ap->{"memo_1"} = $form->{memo};
   $ap->{"paymentmethod_1"} = $form->{paymentmethod};
 
-  $i = 0;
-  for (1 .. $form->{reference_rows}) {
-    if ($form->{"referenceid_$_"}) {
-      $i++;
-      $ap->{"referenceid_$i"} = $form->{"referenceid_$_"};
-      $ap->{"referencedescription_$i"} = $form->{"referencedescription_$_"};
-      $gl->{"referenceid_$i"} = $form->{"referenceid_$_"};
-      $gl->{"referencedescription_$i"} = $form->{"referencedescription_$_"};
+  for $i (1 .. $form->{reference_rows}) {
+    for $item (qw(description id archive_id filename confidential)) {
+      $ap->{"reference${item}_$i"} = $form->{"reference${item}_$i"};
+      $gl->{"reference${item}_$i"} = $form->{"reference${item}_$i"};
     }
   }
-  $ap->{reference_rows} = $i;
-  $gl->{reference_rows} = $i;
+  $ap->{reference_rows} = $form->{reference_rows};
+  $gl->{reference_rows} = $form->{reference_rows};
   
   AA->post_transaction($myconfig, $ap, $dbh);
   
@@ -1544,7 +1609,7 @@ sub post_transaction {
   for $j (1 .. $form->{wage_rows}) {
     for (qw(qty amount)) { $form->{"${_}_$j"} = $form->parse_amount($myconfig, $form->{"${_}_$j"}) }
 
-    if ($form->round_amount($form->{"qty_$j"} * $form->{"amount_$j"},10)) {
+    if ($form->round_amount($form->{"qty_$j"} * $form->{"amount_$j"}, 10)) {
       $pth->execute($ap->{id}, $form->{"wage_id_$j"}, $form->{"qty_$j"}, $form->{"amount_$j"});
       $pth->finish;
     }
@@ -1571,15 +1636,17 @@ sub post_transaction {
     $i = 1;
     for (@employerpays) {
 
-      if ($_->{amount}) {
+      $amount = $form->parse_amount($myconfig, $_->{amount});
+
+      if ($amount) {
 	$gl->{"accno_$i"} = $_->{employeeaccno} || $form->{expense};
 	$gl->{"memo_$i"} = $_->{description};
 	$gl->{"projectnumber_$i"} = $form->{projectnumber};
 
-	if ($_->{amount} < 0) {
-	  $gl->{"debit_$i"} = $form->format_amount($myconfig, $_->{amount}, $form->{precision});
+	if ($amount < 0) {
+	  $gl->{"debit_$i"} = $form->format_amount($myconfig, $amount, $form->{precision});
 	} else {
-	  $gl->{"credit_$i"} = $form->format_amount($myconfig, $_->{amount} * -1, $form->{precision});
+	  $gl->{"credit_$i"} = $form->format_amount($myconfig, $amount * -1, $form->{precision});
 	}
 	$i++;
 	
@@ -1587,10 +1654,10 @@ sub post_transaction {
 	$gl->{"memo_$i"} = $_->{description};
 	$gl->{"projectnumber_$i"} = $form->{projectnumber};
 
-	if ($_->{amount} < 0) {
-	  $gl->{"credit_$i"} = $form->format_amount($myconfig, $_->{amount}, $form->{precision});
+	if ($amount < 0) {
+	  $gl->{"credit_$i"} = $form->format_amount($myconfig, $amount, $form->{precision});
 	} else {
-	  $gl->{"debit_$i"} = $form->format_amount($myconfig, $_->{amount} * -1, $form->{precision});
+	  $gl->{"debit_$i"} = $form->format_amount($myconfig, $amount * -1, $form->{precision});
 	}
 	  
 	$i++;
@@ -1631,7 +1698,7 @@ sub get_deduction {
   my %defaults = $form->get_defaults($dbh, \@{['precision', 'company']});
   for (keys %defaults) { $form->{$_} = $defaults{$_} }
 
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     $query = qq|SELECT d.*,
 		 c1.accno AS employee_accno,
 		 c1.description AS employee_accno_description,
@@ -1771,12 +1838,13 @@ sub save_deduction {
   # connect to database
   my $dbh = $form->dbconnect_noauto($myconfig);
 
-  my $null;
+  $form->{id} *= 1;
+  
   my $deduction_id;
   my $query;
   my $sth;
 
-  ($null, $form->{basedon}) = split /--/, $form->{basedon};
+  (undef, $form->{basedon}) = split /--/, $form->{basedon};
   
   if (! $form->{id}) {
     my $uid = localtime;
@@ -1846,7 +1914,7 @@ sub save_deduction {
   $sth = $dbh->prepare($query) || $form->dberror($query);
 
   for ($i = 1; $i <= $form->{deduct_rows}; $i++) {
-    ($null, $deduction_id) = split /--/, $form->{"deduct_$i"};
+    (undef, $deduction_id) = split /--/, $form->{"deduct_$i"};
     if ($deduction_id) {
       $form->{"percent_$i"} = $form->parse_amount($myconfig, $form->{"percent_$i"});
       $form->{"percent_$i"} /= 100;
@@ -1870,6 +1938,8 @@ sub delete_deduction {
   # connect to database
   my $dbh = $form->dbconnect_noauto($myconfig);
 
+  $form->{id} *= 1;
+  
   # delete deduction
   my $query = qq|DELETE FROM deduction
 	         WHERE id = $form->{id}|;
@@ -1904,7 +1974,7 @@ sub get_wage {
   my %defaults = $form->get_defaults($dbh, \@{['precision', 'company']});
   for (keys %defaults) { $form->{$_} = $defaults{$_} }
 
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     $query = qq|SELECT w.*,
 		 c.accno, c.description AS accno_description,
 		 l.description AS accno_translation,
@@ -2044,6 +2114,8 @@ sub delete_wage {
   # connect to database
   my $dbh = $form->dbconnect_noauto($myconfig);
 
+  $form->{id} *= 1;
+  
   # delete deduction
   my $query = qq|DELETE FROM wage
 	         WHERE id = $form->{id}|;
