@@ -14,7 +14,7 @@
 
 package JC;
 
-use SL::IS;
+use SL::PM;
 
 
 sub retrieve_card {
@@ -35,19 +35,19 @@ sub retrieve_card {
  
   $form->remove_locks($myconfig, $dbh, 'jcitems');
   
-  if ($form->{id}) {
+  if ($form->{id} *= 1) {
     # retrieve timecard/storescard
     $query = qq|SELECT j.*, to_char(j.checkedin, 'HH24:MI:SS') AS checkedina,
                 to_char(j.checkedout, 'HH24:MI:SS') AS checkedouta,
-		to_char(j.checkedin, '$dateformat') AS transdate,
-		e.name AS employee, p.partnumber,
-		pr.projectnumber, pr.description AS projectdescription,
-		pr.production, pr.completed, pr.parts_id AS project,
-		pr.customer_id
+                to_char(j.checkedin, '$dateformat') AS transdate,
+                e.name AS employee, p.partnumber, p.id AS parts_id,
+                pr.projectnumber, pr.description AS projectdescription,
+                pr.production, pr.completed, pr.parts_id AS project,
+                pr.customer_id
                 FROM jcitems j
-		JOIN employee e ON (e.id = j.employee_id)
-		JOIN parts p ON (p.id = j.parts_id)
-		JOIN project pr ON (pr.id = j.project_id)
+                LEFT JOIN employee e ON (e.id = j.employee_id)
+                JOIN parts p ON (p.id = j.parts_id)
+                JOIN project pr ON (pr.id = j.project_id)
                 WHERE j.id = $form->{id}|;
     $sth = $dbh->prepare($query);
     $sth->execute || $form->dberror($query);
@@ -64,8 +64,8 @@ sub retrieve_card {
 
     $query = qq|SELECT s.printed, s.spoolfile, s.formname
                 FROM status s
-		WHERE s.formname = '$form->{type}'
-		AND s.trans_id = $form->{id}|;
+                WHERE s.formname = '$form->{type}'
+                AND s.trans_id = $form->{id}|;
     $sth = $dbh->prepare($query);
     $sth->execute || $form->dberror($query);
 
@@ -77,12 +77,11 @@ sub retrieve_card {
     for (qw(printed queued)) { $form->{$_} =~ s/ +$//g }
 
     if ($form->{customer_id}) {
-      my $pmh = price_matrix_query($dbh, $form, $form->{project_id}, $form->{customer_id});
+      $form->exchangerate_defaults($dbh, $myconfig, $form);
+      my $pmh = PM->price_matrix_query($dbh, $form);
       %ref = ();
       $ref->{id} = $form->{parts_id};
-      
-      IS::exchangerate_defaults($dbh, $myconfig, $form);
-      IS::price_matrix($pmh, $ref, $form->datetonum($myconfig, $form->{transdate}), 4, $form, $myconfig);
+      PM->price_matrix($pmh, $ref, $form->datetonum($myconfig, $form->{transdate}), 4, $form, $myconfig);
     }
 
   }
@@ -92,6 +91,8 @@ sub retrieve_card {
   JC->jcitems_links($myconfig, $form, $dbh);
 
   $form->all_languages($myconfig, $dbh);
+  
+  $form->all_references($dbh, $form->{type});
 
   $dbh->disconnect;
 
@@ -115,19 +116,19 @@ sub jcitems_links {
 
   my $query;
 
-  if ($form->{project_id}) {
+  if ($form->{project_id} *= 1) {
     $query = qq|SELECT parts_id
                 FROM project
 	        WHERE id = $form->{project_id}|;
     if ($dbh->selectrow_array($query)) {
       $form->{project} = 'job';
       if (! exists $form->{orphaned}) {
-	$query = qq|SELECT id
+        $query = qq|SELECT id
 		    FROM project
 		    WHERE parts_id > 0
 		    AND production > completed
 		    AND id = $form->{project_id}|;
-	($form->{orphaned}) = $dbh->selectrow_array($query);
+        ($form->{orphaned}) = $dbh->selectrow_array($query);
       }
     } else {
       $form->{orphaned} = 1;
@@ -191,7 +192,7 @@ sub jcitems_links {
   $sth->finish;
 
   $form->reports($myconfig, $dbh, $form->{login});
-  
+
   $dbh->disconnect if $disconnect;
   
 }
@@ -202,14 +203,15 @@ sub retrieve_item {
   
   my $dbh = $form->dbconnect($myconfig);
   
-  my ($null, $project_id) = split /--/, $form->{projectnumber};
+  my $project_id;
+  (undef, $project_id) = split /--/, $form->{projectnumber};
   $project_id *= 1;
   
   my $query = qq|SELECT customer_id
                  FROM project
 		 WHERE id = $project_id|;
-  ($customer_id) = $dbh->selectrow_array($query);
-  $customer_id *= 1;
+  ($form->{customer_id}) = $dbh->selectrow_array($query);
+  $form->{customer_id} *= 1;
   
   my $var;
   my $where;
@@ -224,24 +226,22 @@ sub retrieve_item {
       $where .= " AND p.inventory_accno_id > 0
                   AND p.income_accno_id > 0";
     } else {
-      $where .= " AND p.income_accno_id IS NULL";
+      $where .= " AND p.income_accno_id IS NULL
+                  AND p.inventory_accno_id > 0";
     }
     
     $query = qq|SELECT p.id, p.partnumber, p.description,
-                p.lastcost AS sellprice,
+                p.sellprice,
                 p.unit, t.description AS translation
                 FROM parts p
 		LEFT JOIN translation t ON (t.trans_id = p.id AND t.language_code = '$form->{language_code}')
 	        WHERE p.obsolete = '0'
 		$where|;
   }
-  
+
   if ($form->{project} eq 'project') {
-    if ($form->{type} eq 'storescard') {
-      $where .= " AND p.inventory_accno_id > 0";
-    } else {
-      $where .= " AND p.inventory_accno_id IS NULL";
-    }
+    $where .= " AND p.inventory_accno_id IS NULL
+                AND p.income_accno_id > 0";
     
     $query = qq|SELECT p.id, p.partnumber, p.description,
                 p.sellprice,
@@ -257,12 +257,12 @@ sub retrieve_item {
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
-  my $pmh = price_matrix_query($dbh, $form, $project_id, $customer_id);
-  IS::exchangerate_defaults($dbh, $myconfig, $form);
+  $form->exchangerate_defaults($dbh, $myconfig, $form);
+  my $pmh = PM->price_matrix_query($dbh, $form);
 
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     $ref->{description} = $ref->{translation} if $ref->{translation};
-    IS::price_matrix($pmh, $ref, $form->datetonum($myconfig, $form->{transdate}), 4, $form, $myconfig);
+    PM->price_matrix($pmh, $ref, $form->datetonum($myconfig, $form->{transdate}), 4, $form, $myconfig);
     $ref->{parts_id} = $ref->{id};
     delete $ref->{id};
     push @{ $form->{item_list} }, $ref;
@@ -274,12 +274,14 @@ sub retrieve_item {
 }
 
 
-sub delete_timecard {
+sub delete {
   my ($self, $myconfig, $form) = @_;
 
   # connect to database
   my $dbh = $form->dbconnect_noauto($myconfig);
- 
+  
+  $form->{id} *= 1;
+  
   my %audittrail = ( tablename  => 'jcitems',
                      reference  => $form->{id},
 		     formname   => $form->{type},
@@ -314,6 +316,8 @@ sub delete_timecard {
 	      AND trans_id = $form->{id}|;
   $dbh->do($query) || $form->dberror($query);
 
+  $form->delete_references($dbh);
+  
   $form->remove_locks($myconfig, $dbh, 'jcitems');
 
   my $rc = $dbh->commit;
@@ -341,11 +345,10 @@ sub jcitems {
   
   my $query;
   my $where = "1 = 1";
-  my $null;
   my $var;
   
   if ($form->{projectnumber}) {
-    ($null, $var) = split /--/, $form->{projectnumber};
+    (undef, $var) = split /--/, $form->{projectnumber};
     $where .= " AND j.project_id = $var";
 
     $query = qq|SELECT parts_id
@@ -371,7 +374,7 @@ sub jcitems {
     }
   }
   if ($form->{employee}) {
-    ($null, $var) = split /--/, $form->{employee};
+    (undef, $var) = split /--/, $form->{employee};
     $where .= " AND j.employee_id = $var";
   }
   if ($form->{description}) {
@@ -390,11 +393,13 @@ sub jcitems {
     }
   }
   
-  ($form->{startdatefrom}, $form->{startdateto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
+  unless ($form->{startdatefrom} || $form->{startdateto}) {
+    ($form->{startdatefrom}, $form->{startdateto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
+  }
   
   $where .= " AND j.checkedin >= '$form->{startdatefrom}'" if $form->{startdatefrom};
   $where .= " AND j.checkedout < date '$form->{startdateto}' + 1" if $form->{startdateto};
- 
+
   my $dateformat = $myconfig->{dateformat};
   $dateformat =~ s/yy$/yyyy/;
   $dateformat =~ s/yyyyyy/yyyy/;
@@ -411,7 +416,7 @@ sub jcitems {
   if ($form->{project} eq 'project') {
     $where .= " AND pr.parts_id IS NULL";
   }
-
+  
   $query = qq|SELECT j.id, j.description, j.qty, j.allocated,
 	      to_char(j.checkedin, 'HH24:MI') AS checkedin,
 	      to_char(j.checkedout, 'HH24:MI') AS checkedout,
@@ -421,13 +426,13 @@ sub jcitems {
 	      p.partnumber,
 	      pr.projectnumber, pr.description AS projectdescription,
 	      e.employeenumber, e.name AS employee,
-	      to_char(j.checkedin, 'WW') AS workweek, pr.parts_id,
+	      to_char(j.checkedin, 'IW') AS workweek, pr.parts_id,
 	      j.sellprice, p.inventory_accno_id, p.income_accno_id,
 	      j.notes
 	      FROM jcitems j
 	      JOIN parts p ON (p.id = j.parts_id)
 	      JOIN project pr ON (pr.id = j.project_id)
-	      JOIN employee e ON (e.id = j.employee_id)
+	      LEFT JOIN employee e ON (e.id = j.employee_id)
 	      WHERE $where|;
 
   my @sf = qw(transdate projectnumber);
@@ -457,7 +462,7 @@ sub jcitems {
     
     $ref->{transdate} = $ref->{transdatea};
     delete $ref->{transdatea};
-    
+
     push @{ $form->{transactions} }, $ref;
   }
   $sth->finish;
@@ -478,15 +483,17 @@ sub save {
 
   my $query;
   my $sth;
+  my $project_id;
   
-  my ($null, $project_id) = split /--/, $form->{projectnumber};
-
-  if ($form->{id}) {
+  (undef, $project_id) = split /--/, $form->{projectnumber};
+  $project_id *= 1;
+  
+  if ($form->{id} *= 1) {
     # check if it was a job
     $query = qq|SELECT pr.parts_id, pr.production - pr.completed
-		FROM project pr
-		JOIN jcitems j ON (j.project_id = pr.id)
-		WHERE j.id = $form->{id}|;
+                FROM project pr
+                JOIN jcitems j ON (j.project_id = pr.id)
+                WHERE j.id = $form->{id}|;
     my ($job_id, $qty) = $dbh->selectrow_array($query);
 
     if ($job_id && $qty == 0) {
@@ -497,13 +504,13 @@ sub save {
     # check if new one belongs to a job
     if ($project_id) {
       $query = qq|SELECT pr.parts_id, pr.production - pr.completed
-		  FROM project pr
-		  WHERE pr.id = $project_id|;
+                  FROM project pr
+                  WHERE pr.id = $project_id|;
       my ($job_id, $qty) = $dbh->selectrow_array($query);
 
       if ($job_id && $qty == 0) {
-	$dbh->disconnect;
-	return -2;
+        $dbh->disconnect;
+        return -2;
       }
     }
     
@@ -531,29 +538,32 @@ sub save {
     $outdate = $form->add_date($myconfig, $form->{transdate}, 1, 'days');
   }
 
-  ($null, $form->{employee_id}) = split /--/, $form->{employee};
+  (undef, $form->{employee_id}) = split /--/, $form->{employee};
   unless ($form->{employee_id}) {
     ($form->{employee}, $form->{employee_id}) = $form->get_employee($dbh);
   } 
 
   $query = qq|UPDATE jcitems SET
               project_id = $project_id,
-	      parts_id = $form->{parts_id},
-	      description = |.$dbh->quote($form->{description}).qq|,
-	      qty = $form->{qty},
-	      allocated = $form->{allocated},
-	      sellprice = $form->{sellprice},
-	      fxsellprice = $form->{sellprice},
-	      serialnumber = |.$dbh->quote($form->{serialnumber}).qq|,
-	      checkedin = timestamp '$form->{transdate} $form->{inhour}:$form->{inmin}:$form->{insec}',
-	      checkedout = timestamp '$outdate $form->{outhour}:$form->{outmin}:$form->{outsec}',
-	      employee_id = $form->{employee_id},
-	      notes = |.$dbh->quote($form->{notes}).qq|
-	      WHERE id = $form->{id}|;
+              parts_id = $form->{parts_id},
+              description = |.$dbh->quote($form->{description}).qq|,
+              qty = $form->{qty},
+              allocated = $form->{allocated},
+              sellprice = $form->{sellprice},
+              fxsellprice = $form->{sellprice},
+              serialnumber = |.$dbh->quote($form->{serialnumber}).qq|,
+              checkedin = timestamp '$form->{transdate} $form->{inhour}:$form->{inmin}:$form->{insec}',
+              checkedout = timestamp '$outdate $form->{outhour}:$form->{outmin}:$form->{outsec}',
+              employee_id = $form->{employee_id},
+              notes = |.$dbh->quote($form->{notes}).qq|
+              WHERE id = $form->{id}|;
   $dbh->do($query) || $form->dberror($query);
 
   # save printed, queued
   $form->save_status($dbh);
+  
+  # save references
+  $form->save_reference($dbh, $form->{type});
 
   my %audittrail = ( tablename  => 'jcitems',
                      reference  => $form->{id},
@@ -562,7 +572,7 @@ sub save {
 		     id         => $form->{id} );
 
   $form->audittrail($dbh, "", \%audittrail);
-  
+
   $form->remove_locks($myconfig, $dbh, 'jcitems');
 
   my $rc = $dbh->commit;
@@ -572,51 +582,6 @@ sub save {
 }
 
 
-sub price_matrix_query {
-  my ($dbh, $form, $project_id, $customer_id) = @_;
-  
-  my $curr = substr($form->get_currencies($dbh, $myconfig),0,3);
-  
-  my $query = qq|SELECT p.id AS parts_id, 0 AS customer_id, 0 AS pricegroup_id,
-              0 AS pricebreak, p.sellprice, NULL AS validfrom, NULL AS validto,
-	      '$curr' AS curr, '' AS pricegroup
-              FROM parts p
-	      WHERE p.id = ?
-
-	      UNION
-  
-              SELECT p.*, g.pricegroup
-              FROM partscustomer p
-	      LEFT JOIN pricegroup g ON (g.id = p.pricegroup_id)
-	      WHERE p.parts_id = ?
-	      AND p.customer_id = $customer_id
-	      
-	      UNION
-
-	      SELECT p.*, g.pricegroup 
-	      FROM partscustomer p 
-	      LEFT JOIN pricegroup g ON (g.id = p.pricegroup_id)
-	      JOIN customer c ON (c.pricegroup_id = g.id)
-	      WHERE p.parts_id = ?
-	      AND c.id = $customer_id
-	      
-	      UNION
-
-	      SELECT p.*, '' AS pricegroup
-	      FROM partscustomer p
-	      WHERE p.customer_id = 0
-	      AND p.pricegroup_id = 0
-	      AND p.parts_id = ?
-
-	      ORDER BY 2 DESC, 3 DESC, 4
-	      
-	      |;
-
-  $dbh->prepare($query) || $form->dberror($query);
-
-}
-
-
 sub company_defaults {
   my ($self, $myconfig, $form) = @_;
 
