@@ -1,6 +1,6 @@
 #=====================================================================
-# SQL-Ledger ERP
-# Copyright (c) 2006
+# SQL-Ledger
+# Copyright (c) DWS Systems Inc.
 #
 #  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.com
@@ -12,12 +12,16 @@
 
 use SL::AA;
 
+require "$form->{path}/sr.pl";
+require "$form->{path}/cm.pl";
+require "$form->{path}/js.pl";
+
 # any custom scripts for this one   
-if (-f "$form->{path}/custom_arap.pl") {
-    eval { require "$form->{path}/custom_arap.pl"; };
+if (-f "$form->{path}/custom/arap.pl") {
+    eval { require "$form->{path}/custom/arap.pl"; };
 }
-if (-f "$form->{path}/$form->{login}_arap.pl") {
-    eval { require "$form->{path}/$form->{login}_arap.pl"; };
+if (-f "$form->{path}/custom/$form->{login}/arap.pl") {
+    eval { require "$form->{path}/custom/$form->{login}/arap.pl"; };
 }
  
 
@@ -37,13 +41,18 @@ sub check_name {
       # this is needed for is, ir and oe
       for (split / /, $form->{taxaccounts}) { delete $form->{"${_}_rate"} }
 
+      for (qw(city state country)) { delete $form->{$_} }
+      
       # for credit calculations
       $form->{oldinvtotal} = 0;
       $form->{oldtotalpaid} = 0;
       $form->{calctax} = 1;
-      
+
       $form->{"${name}_id"} = $new_id;
       AA->get_name(\%myconfig, \%$form);
+      
+      $form->{"paymentmethod_$form->{paidaccounts}"} = $form->{payment_method};
+      $form->{"$form->{ARAP}_paid_$form->{paidaccounts}"} = $form->{payment_accno};
 
       $form->{$name} = $form->{"old$name"} = "$new_name--$new_id";
       $form->{currency} =~ s/ //g;
@@ -94,7 +103,10 @@ sub check_name {
 	$form->{"old${name}number"} = $form->{"${name}number"};
 
 	AA->get_name(\%myconfig, \%$form);
-	
+
+	$form->{"paymentmethod_$form->{paidaccounts}"} = $form->{payment_method};
+	$form->{"$form->{ARAP}_paid_$form->{paidaccounts}"} = $form->{payment_accno};
+
 	$form->{currency} =~ s/ //g;
 	# put employee together if there is a new employee_id
 	$form->{employee} = "$form->{employee}--$form->{employee_id}" if $form->{employee_id};
@@ -137,6 +149,8 @@ sub select_name {
   $column_data{name} = qq|<th class=listheading>|.$locale->text($label).qq|</th>|;
   $column_data{address} = qq|<th class=listheading colspan=5>|.$locale->text('Address').qq|</th>|;
   
+  $form->helpref("list_names", $myconfig{countrycode});
+  
   # list items with radio button on a form
   $form->header;
 
@@ -149,7 +163,7 @@ sub select_name {
 
 <table width=100%>
   <tr>
-    <th class=listtop>$title</th>
+    <th class=listtop>$form->{helpref}$title</a></th>
   </tr>
   <tr space=5></tr>
   <tr>
@@ -248,11 +262,8 @@ sub name_selected {
   
   for (qw(ndx lastndx nextsub)) { delete $form->{$_} }
 
-  $arap_accno = $form->{$form->{ARAP}};
-  
   AA->get_name(\%myconfig, \%$form);
 
-  $form->{$form->{ARAP}} ||= $arap_accno;
   $form->{"old$form->{ARAP}"} = $form->{$form->{ARAP}};
 
   # put employee together if there is a new employee_id
@@ -260,6 +271,9 @@ sub name_selected {
   $form->{cashdiscount} *= 100;
   $form->{cashdiscount} = 0 if $form->{type} =~ /(debit|credit)_/;
   for (qw(terms discountterms)) { $form->{$_} = "" if ! $form->{$_} }
+  
+  $form->{"paymentmethod_$form->{paidaccounts}"} = $form->{payment_method};
+  $form->{"$form->{ARAP}_paid_$form->{paidaccounts}"} = $form->{payment_accno};
 
   &rebuild_formnames;
 
@@ -292,9 +306,9 @@ sub rebuild_formnames {
 sub rebuild_vc {
   my ($vc, $ARAP, $transdate, $job) = @_;
 
-  ($null, $form->{employee_id}) = split /--/, $form->{employee};
+  (undef, $form->{employee_id}) = split /--/, $form->{employee};
   $form->all_vc(\%myconfig, $vc, $ARAP, undef, $transdate, $job);
-  $form->{"select$vc"} = "";
+  $form->{"select$vc"} = ($form->{generate}) ? "\n" : "";
   for (@{ $form->{"all_$vc"} }) { $form->{"select$vc"} .= qq|$_->{name}--$_->{id}\n| }
   $form->{"select$vc"} = $form->escape($form->{"select$vc"},1);
   
@@ -310,6 +324,34 @@ sub rebuild_vc {
 }
 
 
+sub rebuild_departments {
+
+  $form->all_departments(\%myconfig, undef, $form->{vc});
+
+  ($subset) = split /--/, $form->{department};
+  $subset =~ s/:.*//g;
+
+  $form->{selectdepartment} = "\n" if @{ $form->{all_department} };
+  $form->{olddepartment} = $form->{department};
+
+  for (@{ $form->{all_department} }) { 
+    if ($subset) {
+      if ($_->{description} =~ /:/) {
+        if ($_->{description} =~ /${subset}:/) {
+          $form->{selectdepartment} .= qq|$_->{description}--$_->{id}\n|;
+        }
+      } else {
+        $form->{selectdepartment} .= qq|$_->{description}--$_->{id}\n|;
+      }
+    } else {
+      if ($_->{description} !~ /:/) {
+        $form->{selectdepartment} .= qq|$_->{description}--$_->{id}\n|;
+      }
+    }
+  }
+
+}
+
 
 sub add_transaction {
 
@@ -367,6 +409,8 @@ sub select_project {
   $column_data{projectnumber} = qq|<th>|.$locale->text('Number').qq|</th>|;
   $column_data{description} = qq|<th>|.$locale->text('Description').qq|</th>|;
   
+  $form->helpref("select_project", $myconfig{countrycode});
+  
   # list items with radio button on a form
   $form->header;
 
@@ -381,7 +425,7 @@ sub select_project {
 
 <table width=100%>
   <tr>
-    <th class=listtop>$title</th>
+    <th class=listtop>$form->{helpref}$title</a></th>
   </tr>
   <tr space=5></tr>
   <tr>
@@ -481,6 +525,13 @@ sub project_selected {
 sub post_as_new {
 
   for (qw(id printed emailed queued)) { delete $form->{$_} }
+  $form->{postasnew} = 1;
+  if ($form->{lock_sinumber}) {
+    delete $form->{invnumber};
+  }
+  if ($form->{lock_glnumber}) {
+    delete $form->{reference};
+  }
   &post;
 
 }
@@ -489,6 +540,9 @@ sub post_as_new {
 sub print_and_post_as_new {
 
   for (qw(id printed emailed queued)) { delete $form->{$_} }
+  if ($form->{lock_sinumber}) {
+    delete $form->{invnumber};
+  }
   &print_and_post;
 
 }
@@ -525,6 +579,7 @@ sub repost {
   delete $form->{action};
   $form->{repost} = 1;
 
+  
   $form->header;
 
   print qq|
@@ -555,7 +610,13 @@ sub schedule {
   ($form->{recurringreference}, $form->{recurringdescription}, $form->{recurringstartdate}, $form->{recurringrepeat}, $form->{recurringunit}, $form->{recurringhowmany}, $form->{recurringpayment}, $form->{recurringprint}, $form->{recurringemail}, $form->{recurringmessage}) = split /,/, $form->{recurring};
 
   for (qw(reference description message)) { $form->{"recurring$_"} = $form->quote($form->unescape($form->{"recurring$_"})) }
-  
+
+  $type = $form->{type} || "general_ledger";
+  if (exists $form->{ARAP}) {
+    $type = lc $form->{ARAP} . "_$form->{type}";
+  }
+  $form->helpref("recurring_$type", $myconfig{countrycode});
+
   $form->{recurringstartdate} ||= $form->{transdate};
   $recurringpayment = "checked" if $form->{recurringpayment};
 
@@ -572,7 +633,7 @@ sub schedule {
     $nextdate = qq|
 	      <tr>
 		<th align=right nowrap>|.$locale->text('Next Date').qq|</th>
-		<td><input name=recurringnextdate size=11 class=date title="$myconfig{'dateformat'}" value=$form->{recurringnextdate}></td>
+		<td><input name=recurringnextdate size=11 class=date title="$myconfig{'dateformat'}" value=$form->{recurringnextdate}>|.&js_calendar("main", "recurringnextdate").qq|</td>
 	      </tr>
 |;
   }
@@ -583,8 +644,10 @@ sub schedule {
   }
   for (qw(check receipt)) { delete $formname{$_} }
 
-  $selectformat = qq|html--html
-postscript--|.$locale->text('Postscript').qq|
+  $selectformat = qq|html--|.$locale->text('html').qq|
+xml--|.$locale->text('XML').qq|
+txt--|.$locale->text('Text').qq|
+ps--|.$locale->text('Postscript').qq|
 pdf--|.$locale->text('PDF');
 
   if ($form->{type} !~ /transaction/ && %formname) {
@@ -644,10 +707,11 @@ pdf--|.$locale->text('PDF');
 
   }
 
-  if (%printer && $latex && %formname) {
+  if ($form->{selectprinter} && $latex && %formname) {
     $selectprinter = "";
-    for (sort keys %printer) { $selectprinter .= qq|<option value="$_">$_\n| }
-    
+    for (split /\n/, $form->unescape($form->{selectprinter})) { $selectprinter .= qq|
+          <option value="$_">$_| }
+	  
     # formname:format:printer
     @p = split /:/, $form->{recurringprint};
 
@@ -736,7 +800,7 @@ pdf--|.$locale->text('PDF');
 	      </tr>
 	      <tr>
 		<th align=right nowrap>|.$locale->text('For').qq|</th>
-		<td><input name=recurringhowmany size=3 value=$form->{recurringhowmany}></td>
+		<td><input name=recurringhowmany class="inputright" size=3 value=$form->{recurringhowmany}></td>
 		<th align=left nowrap>|.$locale->text('time(s)').qq|</th>
 	      </tr>
 	    </table>
@@ -753,14 +817,16 @@ pdf--|.$locale->text('PDF');
   
   $form->header;
 
+  &calendar;
+  
   print qq|
 <body>
 
-<form method=post action=$form->{script}>
+<form method="post" name="main" action="$form->{script}">
 
 <table width=100%>
   <tr class=listtop>
-    <th class=listtop>$title</th>
+    <th class=listtop>$form->{helpref}$title</a></th>
   </tr>
   <tr space=5></tr>
   <tr>
@@ -780,7 +846,7 @@ pdf--|.$locale->text('PDF');
 
 	      <tr>
 		<th align=right nowrap>|.$locale->text('Startdate').qq|</th>
-		<td><input name=recurringstartdate size=11 class=date title="$myconfig{'dateformat'}" value=$form->{recurringstartdate}></td>
+		<td><input name=recurringstartdate size=11 class=date title="$myconfig{'dateformat'}" value=$form->{recurringstartdate}>|.&js_calendar("main", "recurringstartdate").qq|</td>
 	      </tr>
 	      $nextdate
 	    </table>
@@ -824,12 +890,12 @@ pdf--|.$locale->text('PDF');
              'Delete Schedule' => { ndx => 16, key => 'D', value => $locale->text('Delete Schedule') },
 	    );
   
-  $form->print_button(\%button, 'Save Schedule');
-  
-  if ($form->{recurring}) {
-    $form->print_button(\%button, 'Delete Schedule');
+  unless ($form->{recurring}) {
+    delete $button{'Delete Schedule'};
   }
 
+  $form->print_button(\%button);
+
   # delete variables
   for (qw(action recurring)) { delete $form->{$_} }
   for (qw(reference description startdate nextdate enddate repeat unit howmany payment print email message)) { delete $form->{"recurring$_"} }
@@ -859,8 +925,8 @@ sub save_schedule {
 
   @p = ();
   for $item (split /\r?\n/, $form->unescape($form->{selectformname})) {
-    (@a) = split /--/, $item;
-    push @p, $a[0];
+    (@f) = split /--/, $item;
+    push @p, $f[0];
   }
 
   $recurringemail = "";
@@ -931,7 +997,7 @@ sub reprint {
   for (keys %temp) { $form->{$_} = $temp{$_} }
   
   $form->{rowcount}++;
-  $form->{paidaccounts}++ ;
+  $form->{paidaccounts}++;
 
   delete $form->{paid};
 
@@ -1009,14 +1075,15 @@ sub preview {
   $form->{media} = "screen";
 
   &print;
-
+  
 }
 
+
 sub new_number {
 
   $invnumber = "invnumber";
   $numberfld = ($form->{vc} eq 'customer') ? "sinumber" : "vinumber";
-
+  
   if ($form->{type} =~ /order/) {
     $invnumber = "ordnumber";
     $numberfld = ($form->{vc} eq 'customer') ? "sonumber" : "ponumber";
@@ -1026,11 +1093,15 @@ sub new_number {
   } elsif ($form->{script} eq 'gl.pl') {
     $invnumber = "reference";
     $numberfld = "glnumber";
+  } elsif ($form->{script} eq 'hr.pl') {
+    $numberfld = $invnumber = "employeenumber";
+    HR->isadmin(\%myconfig, \%$form);
   }
 
   $form->{"$invnumber"} = $form->update_defaults(\%myconfig, $numberfld);
 
   &update;
-
+ 
 }
 
+
