@@ -1,6 +1,6 @@
 #=====================================================================
-# SQL-Ledger ERP
-# Copyright (c) 2006
+# SQL-Ledger
+# Copyright (c) DWS Systems Inc.
 #
 #  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.com
@@ -20,7 +20,7 @@ $form = new Form;
 
 $locale = new Locale $language, "admin";
 
-$form->{charset} = $locale->{charset};
+$form->{charset} = $charset;
 
 eval { require DBI; };
 $form->error($locale->text('DBI not installed!')) if ($@);
@@ -33,16 +33,15 @@ $form->{"root login"} = 1;
 require "$form->{path}/pw.pl";
 
 # customization
-if (-f "$form->{path}/custom_$form->{script}") {
-  eval { require "$form->{path}/custom_$form->{script}"; };
+if (-f "$form->{path}/custom/$form->{script}") {
+  eval { require "$form->{path}/custom/$form->{script}"; };
   $form->error($@) if ($@);
 }
 
-
 if ($form->{action}) {
 
   &check_password unless $form->{action} eq $locale->text('logout');
-  
+
   &{ $locale->findsub($form->{action}) };
     
 } else {
@@ -74,7 +73,7 @@ sub adminlogin {
 <script language="javascript" type="text/javascript">
 <!--
 function sf(){
-  document.forms[0].password.focus();
+  document.main.password.focus();
 }
 // End -->
 </script>
@@ -86,7 +85,7 @@ function sf(){
 <a href="http://www.sql-ledger.com"><img src=$images/sql-ledger.gif border=0 target=_blank></a>
 <h1 class=login>|.$locale->text('Version').qq| $form->{version}<p>|.$locale->text('Administration').qq|</h1>
 
-<form method=post action="$form->{script}">
+<form method=post name=main action="$form->{script}">
 
 <table>
   <tr>
@@ -119,7 +118,7 @@ sub create_config {
   if ($form->{password}) {
     my $t = time + $form->{timeout};
     srand( time() ^ ($$ + ($$ << 15)) );
-    $key = "root login$form->{password}$t";
+    $key = "root$form->{password}$t";
 
     my $i = 0;
     my $l = length $key;
@@ -138,10 +137,10 @@ sub create_config {
   }
 
   open(CONF, ">$userspath/root login.conf") or $form->error("root login.conf : $!");
-  print CONF qq|# configuration file for root login
+  print CONF qq|# configuration file for root
 
 \%rootconfig = (
-sessionkey => '$form->{sessionkey}'
+  sessionkey => '$form->{sessionkey}'
 );\n\n|;
 
   close CONF;
@@ -168,7 +167,7 @@ sub logout {
 sub edit {
 
   $form->{title} = "SQL-Ledger ".$locale->text('Administration');
-  
+
   if (-f "$userspath/$form->{dbname}.LCK") {
     open(FH, "$userspath/$form->{dbname}.LCK") or $form->error("$userspath/$form->{dbname}.LCK : $!");
     $form->{lock} = <FH>;
@@ -183,24 +182,24 @@ sub edit {
 
 sub form_footer {
 
-  $nologin = qq|
-<input type=submit class=submit name=action value="|.$locale->text('Lock Dataset').qq|">|;
+  %button = ('Delete' => { ndx => 2, key => 'D', value => $locale->text('Delete') },
+             'Change Password' => { ndx => 4, key => 'C', value => $locale->text('Change Password') },
+             'Change Host' => { ndx => 5, key => 'H', value => $locale->text('Change Host') },
+            );
 
   if (-f "$userspath/$form->{dbname}.LCK") {
-    $nologin = qq|
-<input type=submit class=submit name=action value="|.$locale->text('Unlock Dataset').qq|">|;
+    $button{'Unlock Dataset'} = { ndx => 1, key => 'U', value => $locale->text('Unlock Dataset') };
+  } else {
+    $button{'Lock Dataset'} = { ndx => 1, key => 'L', value => $locale->text('Lock Dataset') };
   }
 
-  $delete = qq|<input type=submit class=submit name=action value="|.$locale->text('Delete').qq|">|;
-  
   $form->{callback} = "$form->{script}?action=list_datasets&path=$form->{path}";
 
-  $form->hide_form(qw(company dbname path callback));
-  
-  print qq|
+  $form->hide_form(qw(company dbname dbhost dbdriver dbuser path callback));
 
-$nologin
-$delete
+  $form->print_button(\%button);
+
+  print qq|
 
 </form>
 
@@ -219,48 +218,46 @@ sub list_datasets {
 # type=submit $locale->text('Sybase')
 
   open(FH, "$memberfile") or $form->error("$memberfile : $!");
-  my @member = <FH>;
-  close(FH);
-  
-  $nologin = qq|
-<input type=submit class=submit name=action value="|.$locale->text('Lock System').qq|">|;
 
-  if (-f "$userspath/nologin.LCK") {
-    $nologin = qq|
-<input type=submit class=submit name=action value="|.$locale->text('Unlock System').qq|">|;
-  }
+  my %member;
+  my $member;
 
-  $login = "";
-
-  while (@member) {
-    $_ = shift @member;
-    
-    if (/^\[.*\]/) {
-      %temp = ();
-      do {
-	if (/^(company|dbname|dbdriver|dbhost|dbuser)=/) {
-	  chop ($var = $&);
-	  ($null, $temp{$var}) = split /=/, $_, 2;
-	}
-	$_ = shift @member;
-      } until /^\s+$/;
-
-      chop $temp{dbname};
-      for (keys %temp) { $member{$temp{dbname}}{$_} = $temp{$_} }
-      $member{$temp{dbname}}{locked} = "x" if -f "$userspath/$member{$temp{dbname}}{dbname}.LCK";
+  while (<FH>) {
+    if (/^\[(.*)\]/) {
+      $member = $+;
+      if ($member =~ /^admin\@/) {
+        $member = substr($member,6);
+        $new = 1;
+      } else {
+        $new = 0;
+      }
+    }
+    if ($new) {
+      if (/^(company|dbname|dbdriver|dbhost|dbuser|templates)=/) {
+        $var = $1;
+        (undef, $member{$member}{$var}) = split /=/, $_, 2;
+        $member{$member}{$var} =~ s/(\r\n|\n)//;
+      }
     }
   }
   
-  delete $member{""};
+  close(FH);
+
+  delete $member{"root login"};
+
+  for (keys %member) {
+    $member{$_}{locked} = "x" if -f "$userspath/$member{$_}{dbname}.LCK";
+  }
 
   $column_data{company} = qq|<th>|.$locale->text('Company').qq|</th>|;
   $column_data{dbdriver} = qq|<th>|.$locale->text('Driver').qq|</th>|;
   $column_data{dbhost} = qq|<th>|.$locale->text('Host').qq|</th>|;
   $column_data{dbuser} = qq|<th>|.$locale->text('User').qq|</th>|;
   $column_data{dbname} = qq|<th>|.$locale->text('Dataset').qq|</th>|;
+  $column_data{templates} = qq|<th>|.$locale->text('Templates').qq|</th>|;
   $column_data{locked} = qq|<th width=1%>|.$locale->text('Locked').qq|</th>|;
 
-  @column_index = qw(dbname company locked dbdriver dbuser dbhost);
+  @column_index = qw(dbname company templates locked dbdriver dbuser dbhost);
   
   $dbdriver ||= "Pg";
   $dbdriver{$dbdriver} = "checked";
@@ -297,16 +294,12 @@ sub list_datasets {
 |;
 
   foreach $key (sort keys %member) {
-    chomp $member{$key}{company};
-    $href = "$script?action=edit&dbname=$key&path=$form->{path}&company=$member{$key}{company}&locked=$member{$key}{locked}";
-    $href =~ s/ /%20/g;
-    
+    $href = "$script?action=edit&dbname=$key&path=$form->{path}&locked=$member{$key}{locked}&dbhost=$member{$key}{dbhost}&dbdriver=$member{$key}{dbdriver}&dbuser=$member{$key}{dbuser}";
+    $href .= "&company=".$form->escape($member{$key}{company},1);
+
     $member{$key}{dbname} = $member{$key}{dbuser} if ($member{$key}{dbdriver} eq 'Oracle');
 
-    $column_data{company} = qq|<td>$member{$key}{company}</td>|;
-    $column_data{dbdriver} = qq|<td>$member{$key}{dbdriver}</td>|;
-    $column_data{dbhost} = qq|<td>$member{$key}{dbhost}</td>|;
-    $column_data{dbuser} = qq|<td>$member{$key}{dbuser}</td>|;
+    for (qw(company dbdriver dbhost dbuser templates)) { $column_data{$_} = qq|<td>$member{$key}{$_}</td>| }
     $column_data{dbname} = qq|<td><a href=$href>$member{$key}{dbname}</a></td>|;
     $column_data{locked} = qq|<td align=center>$member{$key}{locked}</td>|;
     
@@ -332,12 +325,23 @@ sub list_datasets {
 <input type=hidden name=path value=$form->{path}>
 
 $dbdrivers
-
 <p>
-<input type=submit class=submit name=action value="|.$locale->text('Add Dataset').qq|">
-<input type=submit class=submit name=action value="|.$locale->text('Change Password').qq|">
-$nologin
-<input type=submit class=submit name=action value="|.$locale->text('Logout').qq|">
+|;
+
+  %button = ('Add Dataset' => { ndx => 1, key => 'A', value => $locale->text('Add Dataset') },
+             'Change Password' => { ndx => 2, key => 'C', value => $locale->text('Change Password') },
+             'Logout' => { ndx => 4, key => 'X', value => $locale->text('Logout') }
+            );
+
+  if (-f "$userspath/nologin.LCK") {
+    $button{'Unlock System'} = { ndx => 3, key => 'U', value => $locale->text('Unlock System') };
+  } else {
+    $button{'Lock System'} = { ndx => 3, key => 'L', value => $locale->text('Lock System') };
+  }
+
+  $form->print_button(\%button);
+  
+  print qq|
 
 </form>
 
@@ -355,6 +359,8 @@ sub form_header {
 
   $form->header;
 
+  $focus = "lock";
+
   if ($form->{locked}) {
     $locked = qq|
           <td>$form->{lock}</td>|;
@@ -364,9 +370,9 @@ sub form_header {
   }
 
   print qq|
-<body class=admin>
+<body class=admin onload="document.main.${focus}.focus()" />
 
-<form method=post action=$form->{script}>
+<form name=main method=post action=$form->{script}>
 
 <table>
   <tr class=listheading><th>$form->{title}</th></tr>
@@ -455,31 +461,30 @@ sub do_delete {
   }
   
   # get variables for dbname
-  while (@db) {
-    $_ = shift @db;
+  while ($_ = shift @db) {
     
     if (/^\[(.*)\]/) {
       $user = $+;
-      %temp = ();
-      do {
-	chop;
-	($var, $value) = split /=/, $_, 2;
-	if ($value) {
-	  $temp{$var} = $value;
-	}
-	$_ = shift @db;
-      } until /^\s/;
+      next;
+    }
 
-      for (keys %temp) {
-	$db{$temp{dbname}}{$_} = $temp{$_};
-	$member{$temp{dbname}}{$user}{$_} = $temp{$_};
-      }
+    chop;
+    ($var, $value) = split /=/, $_, 2;
+    $temp{$user}{$var} = $value;
+
+  }
+
+  for $user (keys %temp) {
+    $dbname = $temp{$user}{dbname};
+    for (keys %{$temp{$user}}) {
+      $db{$dbname}{$_} = $temp{$user}{$_};
+      $member{$dbname}{$user}{$_} = $temp{$user}{$_};
     }
   }
-  
+
   $form->{dbdriver} = $db{$form->{dbname}}{dbdriver};
   &dbdriver_defaults;
-  for (qw(dbconnect dbuser dbhost dbport)) { $form->{$_} = $db{$form->{dbname}}{$_} }
+  for (qw(dbconnect dbuser dbhost dbport templates)) { $form->{$_} = $db{$form->{dbname}}{$_} }
   $form->{dbpasswd} = unpack 'u', $db{$form->{dbname}}{dbpasswd};
 
   # delete dataset
@@ -501,35 +506,49 @@ sub do_delete {
   
   for $db (sort keys %member) {
     for $user (sort keys %{ $member{$db} }) {
-      print FH "\[$user\]\n";
-      for $var (sort keys %{ $member{$db}{$user} }) {
-	print FH "${var}=$member{$db}{$user}{$var}\n";
+      if ($user) {
+        print FH "\[$user\]\n";
+        for $var (sort keys %{ $member{$db}{$user} }) {
+          print FH "${var}=$member{$db}{$user}{$var}\n" if $member{$db}{$user}{$var};
+        }
+        print FH "\n";
       }
-      print FH "\n";
     }
   }
   close(FH);
   
-  unlink "${memberfile}.LCK";
-  unlink "$userspath/$form->{dbname}.LCK";
+  # delete spool and template directory if it is not shared
+  # compare dbname and templates
+  for (qw(dbname spool)) {
 
-  # delete spool and template directory
-  for $dir ("$templates/$form->{dbname}", "$spool/$form->{dbname}") {
-    if (-d "$dir") {
+    if ($_ eq "dbname") {
+      $dir = "$templates/$form->{dbname}";
+      $form->{templates} ||= $form->{dbname};
+      next if ($form->{dbname} ne $form->{templates});
+    }
+
+    if ($_ eq "spool") {
+      $dir = "$spool/$form->{dbname}";
+    }
+
+    if (-d $dir) {
       opendir DIR, $dir;
       @all = grep !/^\./, readdir DIR;
       closedir DIR;
       for $subdir (@all) {
-	if (-d "$dir/$subdir") {
-	  unlink <$dir/$subdir/*>;
-	  rmdir "$dir/$subdir";
-	}
+        if (-d "$dir/$subdir") {
+          unlink <$dir/$subdir/*>;
+          rmdir "$dir/$subdir";
+        }
       }
       unlink <$dir/*>;
       rmdir "$dir";
     }
   }
   
+  unlink "${memberfile}.LCK";
+  unlink "$userspath/$form->{dbname}.LCK";
+
   $form->redirect($locale->text('Dataset deleted!'));
 
 }
@@ -539,12 +558,18 @@ sub change_password {
 
   $form->{title} = $locale->text('Change Password');
 
+  if ($form->{dbname}) {
+    $form->{title} = $locale->text('Change Password for Dataset')." $form->{dbname}";
+  }
+
+  $focus = "new_password";
+
   $form->header;
 
   print qq|
-<body class=admin>
+<body class=admin onload="document.main.${focus}.focus()" />
 
-<form method=post action=$form->{script}>
+<form method=post name=main action=$form->{script}>
 
 <table>
   <tr>
@@ -576,7 +601,8 @@ sub change_password {
 <input type=submit class=submit name=action value="|.$locale->text('Continue').qq|">|;
 
   $form->{nextsub} = "do_change_password";
-  $form->hide_form(qw(path nextsub));
+
+  $form->hide_form(qw(path nextsub dbname));
 
   print qq|
 </form>
@@ -590,26 +616,107 @@ sub change_password {
 
 sub do_change_password {
 
-  $form->error($locale->text('Passwords do not match!')) if $form->{new_password} ne $form->{confirm_password};
+  if ($form->{new_password}) {
+    $form->error('Password may not contain ? or &') if $form->{new_password} =~ /\?|\&/;
+    if ($form->{new_password} ne $form->{confirm_password}) {
+      $form->error($locale->text('Password does not match!'));
+    }
+  }
 
-  $root->{password} = $form->{new_password};
+  if ($form->{dbname}) {
+    # get connection details from members file
+    $admin = new User $memberfile, "admin\@$form->{dbname}";
+
+    for (qw(dbconnect dbuser dbpasswd dbhost dbdriver)) { $form->{$_} = $admin->{$_} }
+    $form->{dbpasswd} = unpack 'u', $form->{dbpasswd};
+
+    open(FH, ">${memberfile}.LCK") or $form->error("${memberfile}.LCK : $!");
+    close(FH);
   
-  if (! -f $memberfile) {
-    open(FH, ">$memberfile") or $form->error("$memberfile : $!");
+    if (! open(FH, "+<$memberfile")) {
+      unlink "${memberfile}.LCK";
+      $form->error("$memberfile : $!");
+    }
+    @member = <FH>;
+
+    while ($_ = shift @member) {
+      if (/^\[.*\]/) {
+        chop;
+        $member = $_;
+        $member =~ s/(^\[|\]$)//g;
+        $_ = shift @member;
+        do {
+          chop;
+          ($var, $val) = split /=/, $_, 2;
+          $member{$member}{$var} = $val;
+          $_ = shift @member;
+        } until /^\s+$/;
+      }
+    }
+    
+    # change password
+    User->dbpassword(\%$form);
+
+    # change passwords in members file
+    $form->{dbpasswd} = pack 'u', $form->{new_password};
+    chomp $form->{dbpasswd};
+
+    seek(FH, 0, 0);
+    truncate(FH, 0);
+
     print FH qq|# SQL-Ledger members
 
+[root login]\n|;
+
+    for (sort keys %{$member{"root login"}}) {
+      print FH qq|$_=$member{"root login"}{$_}\n|;
+    }
+    print FH "\n";
+    delete $member{"root login"};
+
+    for (keys %member) {
+      if ($member{$_}{dbdriver} eq $form->{dbdriver}) {
+        if ($member{$_}{dbuser} eq $form->{dbuser}) {
+          if ($member{$_}{dbhost} eq $form->{dbhost}) {
+            $member{$_}{dbpasswd} = $form->{dbpasswd};
+          }
+        }
+      }
+    }
+
+    for $member (sort keys %member) {
+      print FH qq|[$member]\n|;
+      for (sort keys %{$member{$member}}) {
+        print FH qq|$_=$member{$member}{$_}\n|;
+      }
+      print FH "\n";
+    }
+    close(FH);
+
+    unlink "${memberfile}.LCK";
+
+  } else {
+
+    $root->{password} = $form->{new_password};
+    
+    if (! -f $memberfile) {
+      open(FH, ">$memberfile") or $form->error("$memberfile : $!");
+      print FH qq|# SQL-Ledger members
+
 [root login]
 |;
-    close FH;
+      close FH;
+    }
+   
+    $root->{'root login'} = 1;
+    $root->{login} = "root login";
+    $root->save_member($memberfile);
+
+    $form->{password} = $form->{new_password};
+    &create_config;
+    
   }
- 
-  $root->{'root login'} = 1;
-  $root->{login} = "root login";
-  $root->save_member($memberfile);
 
-  $form->{password} = $form->{new_password};
-  &create_config;
-  
   $form->{callback} = "$form->{script}?action=list_datasets&path=$form->{path}&password=$form->{password}";
 
   $form->redirect($locale->text('Password changed!'));
@@ -617,6 +724,137 @@ sub do_change_password {
 }
 
 
+sub change_host {
+
+  $form->{title} = $locale->text('Change Host for Dataset')." $form->{dbname}";
+
+  $focus = "new_host";
+
+  $form->header;
+
+  print qq|
+<body class=admin onload="document.main.${focus}.focus()" />
+
+<form method=post name=main action=$form->{script}>
+
+<table>
+  <tr>
+  <tr class=listheading>
+    <th>$form->{title}</th>
+  </tr>
+  <tr size=5></tr>
+  <tr>
+    <td>
+      <table width=100%>
+	<tr>
+	  <th align=right>|.$locale->text('Host').qq|</th>
+	  <td><input name=new_host size=40 value=$form->{dbhost}></td>
+	</tr>
+      </table>
+    </td>
+  </tr>
+  <tr>
+    <td>
+      <hr size=3 noshade>
+    </td>
+  </tr>
+</table>
+
+<input type=submit class=submit name=action value="|.$locale->text('Continue').qq|">|;
+
+  $form->{nextsub} = "do_change_host";
+
+  $form->hide_form(qw(path nextsub dbname dbhost dbdriver));
+
+  print qq|
+</form>
+
+</body>
+</html>
+|;
+
+}
+
+
+sub do_change_host {
+
+  open(FH, ">${memberfile}.LCK") or $form->error("${memberfile}.LCK : $!");
+  close(FH);
+
+  if (! open(FH, "+<$memberfile")) {
+    unlink "${memberfile}.LCK";
+    $form->error("$memberfile : $!");
+  }
+  @member = <FH>;
+
+  while ($_ = shift @member) {
+    if (/^\[.*\]/) {
+      chop;
+      $member = $_;
+      $member =~ s/(^\[|\]$)//g;
+      $_ = shift @member;
+      do {
+        chop;
+        ($var, $val) = split /=/, $_, 2;
+        $member{$member}{$var} = $val;
+        $_ = shift @member;
+      } until /^\s+$/;
+    }
+  }
+
+  seek(FH, 0, 0);
+  truncate(FH, 0);
+
+  print FH qq|# SQL-Ledger members
+
+[root login]\n|;
+
+  for (sort keys %{$member{"root login"}}) {
+    print FH qq|$_=$member{"root login"}{$_}\n|;
+  }
+  print FH "\n";
+  delete $member{"root login"};
+
+  for (keys %member) {
+    if ($member{$_}{dbdriver} eq $form->{dbdriver}) {
+      if ($member{$_}{dbname} eq $form->{dbname}) {
+        if ($form->{new_host} ne $member{$_}{dbhost}) {
+          $member{$_}{dbhost} = $form->{new_host};
+          if ($form->{dbdriver} =~ /(Pg|Sybase)/) {
+            $member{$_}{dbconnect} = "dbi:$form->{dbdriver}:dbname=$form->{dbname}";
+          }
+          if ($form->{dbdriver} eq 'Oracle') {
+            $form->{dbconnect} = "dbi:Oracle:sid=$member{$_}{sid}";
+          }
+
+          if ($form->{new_host}) {
+            $member{$_}{dbconnect} .= ";host=$form->{new_host}";
+          } else {
+            delete $member{$_}{dbhost};
+          }
+        }
+      }
+    }
+  }
+
+  for $member (sort keys %member) {
+    print FH qq|[$member]\n|;
+    for (sort keys %{$member{$member}}) {
+      print FH qq|$_=$member{$member}{$_}\n|;
+    }
+    print FH "\n";
+  }
+  close(FH);
+
+  unlink "${memberfile}.LCK";
+
+  $form->{callback} = "$form->{script}?action=list_datasets&path=$form->{path}";
+
+  $form->redirect($locale->text('Lockfile removed!'));
+
+}
+
+
 sub check_password {
 
   $root = new User "$memberfile", "root login";
@@ -625,7 +863,7 @@ sub check_password {
   eval { require "$userspath/${rootname}.conf"; };
 
   if ($root->{password}) {
-      
+
     if ($form->{password}) {
       $form->{callback} .= "&password=$form->{password}" if $form->{callback};
       if ($root->{password} ne crypt $form->{password}, 'ro') {
@@ -637,7 +875,7 @@ sub check_password {
       &create_config;
       
     } else {
-      
+
       if ($ENV{HTTP_USER_AGENT}) {
 	$ENV{HTTP_COOKIE} =~ s/;\s*/;/g;
 	@cookies = split /;/, $ENV{HTTP_COOKIE};
@@ -646,8 +884,8 @@ sub check_password {
 	  ($name,$value) = split /=/, $_, 2;
 	  $cookie{$name} = $value;
 	}
-	
-	$cookie = ($form->{path} eq 'bin/lynx') ? $cookie{login} : $cookie{"SL-root login"};
+
+	$cookie = ($form->{path} eq 'bin/lynx') ? $cookie{login} : $cookie{"SL-root"};
 
         if ($cookie) {
 	  $form->{sessioncookie} = $cookie;
@@ -665,16 +903,15 @@ sub check_password {
 	    $s .= $ndx{$_};
 	  }
 
-	  $l = length 'root login';
+	  $l = 4;
 	  $login = substr($s, 0, $l);
 	  $time = substr($s, -10);
 	  $password = substr($s, $l, (length $s) - ($l + 10));
-	  
-	  if ((time > $time) || ($login ne 'root login') || ($root->{password} ne crypt $password, 'ro')) {
+
+	  if ((time > $time) || ($login ne 'root') || ($root->{password} ne crypt $password, 'ro')) {
 	    &getpassword;
 	    exit;
 	  }
-
 	} else {
 	  &getpassword;
 	  exit;
@@ -733,12 +970,14 @@ sub dbselect_source {
   
   $form->{callback} = "$form->{script}?action=list_datasets&path=$form->{path}";
   
+  $focus = "dbhost";
+
   $form->header;
 
   print qq|
-<body class=admin>
+<body class=admin onLoad="document.main.${focus}.focus()" />
 
-<form method=post action=$form->{script}>
+<form name=main method=post action=$form->{script}>
 
 <table>
   <tr class=listheading>
@@ -792,82 +1031,63 @@ sub yes { &{ $form->{nextsub} } }
 
 
 sub create_dataset {
-  
+
   @dbsources = sort User->dbsources(\%$form);
 
-  opendir SQLDIR, "sql/." or $form->error($!);
+  opendir SQLDIR, "sql" or $form->error($!);
   foreach $item (sort grep /-chart\.sql/, readdir SQLDIR) {
     next if ($item eq 'Default-chart.sql');
     $item =~ s/-chart\.sql//;
-    push @charts, qq|<input name=chart class=radio type=radio value="$item">$item|;
+    $selectchart .= "$item\n";
   }
+
+  $selectchart = "Default\n$selectchart" if (-f 'sql/Default-chart.sql');
   closedir SQLDIR;
+
+  $form->{name} ||= $locale->text('Admin');
   
-  # is there a template basedir
-  if (! -d "$templates") {
-    $form->error($locale->text('Directory').": $templates ".$locale->text('does not exist'));
+  $templatedir = "doc/templates";
+
+  if (-d "$templatedir") {
+    opendir TEMPLATEDIR, "$templatedir" or $form->error("$templatedir : $!");
+
+    @all = grep !/^\./, readdir TEMPLATEDIR;
+    closedir TEMPLATEDIR;
+
+    for (@all) { push @alldir, $_ if -d "$templatedir/$_" }
+
+    @alldir = sort grep !/Default/, @alldir;
+    unshift @alldir, 'Default' if -d "$templatedir/Default";
+
+    for (@alldir) { $selecttemplates .= qq|$_\n| }
+
   }
 
-  opendir TEMPLATEDIR, "$templates/." or $form->error("$templates : $!");
-  @all = grep !/^\.\.?$/, readdir TEMPLATEDIR;
-  closedir TEMPLATEDIR;
+  $selectencoding = User->encoding($form->{dbdriver});
+  $form->{charset} = $charset;
 
-  @allhtml = sort grep /\.html/, @all;
+  # get subdirectories from templates directory
+  if (-d $templates) {
+    opendir TEMPLATEDIR, "$templates" or $form->error("$templates : $!");
+    @d = grep !/^\./, readdir TEMPLATEDIR;
+    closedir TEMPLATEDIR;
+    for (@d) { push @dir, $_ if -d "$templates/$_" }
 
-  @allhtml = reverse grep !/Default/, @allhtml;
-  push @allhtml, 'Default';
-  @allhtml = reverse @allhtml;
+    if (@dir) {
+      $selectusetemplates = "\n";
+      for (sort @dir) { $selectusetemplates .= "$_\n" }
+      chop $selectusetemplates;
 
-  for (sort @alldir) { $selectusetemplates .= qq|$_\n| }
-
-  $lastitem = $allhtml[0];
-  $lastitem =~ s/-.*//g;
-  $selectmastertemplates = qq|$lastitem\n|;
-  for (@allhtml) {
-    $_ =~ s/-.*//g;
-
-    if ($_ ne $lastitem) {
-      $selectmastertemplates .= qq|$_\n|;
-      $lastitem = $_;
+      $usetemplates = qq|
+        <tr>
+          <th align=right nowrap>|.$locale->text('Use Templates').qq|</th>
+          <td><select name=usetemplates>|.$form->select_option($selectusetemplates).qq|</select></td>
+        </tr>
+|;
     }
+
   }
 
-  # add Default at beginning
-  unshift @charts, qq|<input name=chart class=radio type=radio value="Default" checked>Default|;
-
-  $selectencoding{Pg} = qq|<option>
-  <option value=SQL_ASCII>ASCII
-  <option value=UTF8>Unicode (UTF-8)
-  <option value=EUC_CN>Chinese EUC
-  <option value=EUC_JP>Japanese EUC
-  <option value=EUC_KR>Korean EUC
-  <option value=JOHAB>Korean (Hangul)
-  <option value=EUC_TW>Taiwan EUC
-  <option value=KOI8>KOI8-R(U)
-  <option value=LATIN1>ISO 8859-1/ECMA 94 (Latin alphabet no. 1)
-  <option value=LATIN2>ISO 8859-2/ECMA 94 (Latin alphabet no. 2)
-  <option value=LATIN3>ISO 8859-3/ECMA 94 (Latin alphabet no. 3)
-  <option value=LATIN4>ISO 8859-4/ECMA 94 (Latin alphabet no. 4)
-  <option value=LATIN5>ISO 8859-9/ECMA 128 (Latin alphabet no. 5)
-  <option value=LATIN6>ISO 8859-10/ECMA 144 (Latin alphabet no. 6)
-  <option value=LATIN7>ISO 8859-13 (Latin alphabet no. 7)
-  <option value=LATIN8>ISO 8859-14 (Latin alphabet no. 8)
-  <option value=LATIN9>ISO 8859-15 (Latin alphabet no. 9)
-  <option value=LATIN10>ISO 8859-16/ASRO SR 14111 (Latin alphabet no. 10)
-  <option value=ISO_8859_5>ISO 8859-5/ECMA 113 (Latin/Cyrillic)
-  <option value=ISO_8859_6>ISO 8859-6/ECMA 114 (Latin/Arabic)
-  <option value=ISO_8859_7>ISO 8859-7/ECMA 118 (Latin/Greek)
-  <option value=ISO_8859_8>ISO 8859-8/ECMA 121 (Latin/Hebrew)
-  <option value=MULE_INTERNAL>Mule internal type
-  <option value=ALT>Windows CP866 (Cyrillic)
-  <option value=WIN1250>Windows CP1250 (Central Euope)
-  <option value=WIN>Windows CP1251 (Cyrillic)
-  <option value=WIN1252>Windows CP1252 (Western European)
-  <option value=WIN1256>Windows CP1256 (Arabic)
-  <option value=TCVN>Windows CP1258 (Vietnamese)
-  <option value=WIN874>Windows CP874 (Thai)
-  |;
-  
   $form->{title} = "SQL-Ledger / ".$locale->text('Create Dataset');
   
   $form->header;
@@ -885,65 +1105,61 @@ sub create_dataset {
   <tr>
     <td>
       <table>
-	<tr>
-	  <th align=right nowrap>|.$locale->text('Existing Datasets').qq|</th>
-	  <td>
-      |;
+        <tr>
+          <th align=right nowrap>|.$locale->text('Existing Datasets').qq|</th>
+          <td>
+|;
 
 	  for (@dbsources) { print "[&nbsp;$_&nbsp;] " }
 	  
 	  print qq|
-	  </td>
-	</tr>
-	<tr>
-	  <th align=right nowrap>|.$locale->text('Dataset').qq|</th>
-	  <td><input name=db></td>
-	</tr>
-	<tr>
-	  <th align=right nowrap>|.$locale->text('Company').qq|</th>
-	  <td><input name=company size=25></td>
-	</tr>
-	<tr>
-	  <th align=right nowrap>|.$locale->text('Templates').qq|</th>
-	  <td><select name=mastertemplates>|.$form->select_option($selectmastertemplates).qq|</select></td>
-	</tr>
+          </td>
+        </tr>
+        <tr>
+          <th align=right nowrap>|.$locale->text('Dataset').qq|</th>
+          <td><input name=db></td>
+        </tr>
+        <tr>
+          <th align=right nowrap>|.$locale->text('Company').qq|</th>
+          <td><input name=company size=35></td>
+        </tr>
+        <tr>
+          <th align=right nowrap>|.$locale->text('Administrator').qq|</th>
+          <td><input name=name size=35 value="$form->{name}"></td>
+        </tr>
+        <tr>
+          <th align=right nowrap>|.$locale->text('E-mail').qq|</th>
+          <td><input name=email size=35></td>
+        </tr>
+        <tr>
+          <th align=right nowrap>|.$locale->text('Password').qq|</th>
+          <td><input name=adminpassword type=password size=25></td>
+        </tr>
+        <tr>
+          <th align=right nowrap>|.$locale->text('Templates').qq|</th>
+          <td><select name=mastertemplates>|.$form->select_option($selecttemplates).qq|</select></td>
+        </tr>
+        $usetemplates
 |;
 
-  if ($selectencoding{$form->{dbdriver}}) {
+  if ($selectencoding) {
     print qq|
-	<tr>
-	  <th align=right nowrap>|.$locale->text('Multibyte Encoding').qq|</th>
-	  <td><select name=encoding>$selectencoding{$form->{dbdriver}}</select></td>
-	</tr>
+        <tr>
+          <th align=right nowrap>|.$locale->text('Multibyte Encoding').qq|</th>
+          <td><select name=encoding>|.$form->select_option($selectencoding, $form->{charset},1,1).qq|</select></td>
+        </tr>
 |;
   }
 
   print qq|
  
-	<tr>
-	  <th align=right nowrap>|.$locale->text('Create Chart of Accounts').qq|</th>
-	  <td>
-	    <table>
+        <tr>
+          <th align=right nowrap>|.$locale->text('Create Chart of Accounts').qq|</th>
+          <td><select name=chart>|.$form->select_option($selectchart).qq|</select></td>
+        </tr>
 |;
 
-  while (@charts) {
-    print qq|
-	      <tr>
-|;
-
-    for (0 .. 2) { print "<td>$charts[$_]</td>\n" }
-
-    print qq|
-	      </tr>
-|;
-
-    splice @charts, 0, 3;
-  }
-
   print qq|
-	    </table>
-	  </td>
-	</tr>
       </table>
     </td>
   </tr>
@@ -975,56 +1191,76 @@ sub dbcreate {
   
   $form->error("$memberfile : ".$locale->text('locked!')) if (-f ${memberfile}.LCK);
 
+  # check if dbname is already in use
+  open(FH, "$memberfile") or $form->error("$memberfile : $!");
+  @member = <FH>;
+  close(FH);
+
+  while ($_ = shift @member) {
+    if (/^\[(.*)\]/) {
+      $user = $+;
+      $user =~ s/.*\@//;
+    }
+
+    if (/^dbname=/) {
+      chop;
+      (undef, $dbname) = split /=/, $_, 2;
+       $member{$user} = $dbname;
+    }
+  }
+
+  $form->error($locale->text('Duplicate Dataset!')) if $form->{db} eq $member{$form->{db}};
+
   open(FH, ">${memberfile}.LCK") or $form->error("${memberfile}.LCK : $!");
   close(FH);
  
-  # is there a basedir
+  $templatedir = "doc/templates";
+
+  umask(002);
+
   if (! -d "$templates") {
-    $form->error($locale->text('Directory').": $templates ".$locale->text('does not exist'));
+    mkdir "$templates", oct("771") or $form->error("$templates : $!");
   }
 
-  $form->{company} = $form->{db} unless $form->{company};
+  $form->{company} ||= $form->{db};
   
-  User->dbcreate(\%$form);
-
   # create user template directory and copy master files
-  $form->{templates} = "$templates/$form->{db}";
-  
-  if (! -d "$form->{templates}") {
-    
-    umask(002);
-    
-    if (mkdir "$form->{templates}", oct("771")) {
-      
-      umask(007);
-      
-      # copy templates to the directory
-      opendir TEMPLATEDIR, "$templates/." or $form->error("$templates : $!");
-      @templates = grep /$form->{mastertemplates}-/, readdir TEMPLATEDIR;
-      closedir TEMPLATEDIR;
+  $form->{templates} = ($form->{usetemplates}) ? "$form->{usetemplates}" : "$form->{db}";
 
-      foreach $file (@templates) {
-	open(TEMP, "$templates/$file") or $form->error("$templates/$file : $!");
-	
-	$file =~ s/$form->{mastertemplates}-//;
-	open(NEW, ">$form->{templates}/$file") or $form->error("$form->{templates}/$file : $!");
-	  
-	for (<TEMP>) {
-	  print NEW $_;
-	}
-	close(TEMP);
-	close(NEW);
-      }
-    } else {
-      $form->error("$form->{templates} : $!");
+  # get templates from master
+  opendir TEMPLATEDIR, "$templatedir/$form->{mastertemplates}" or $form->error("$templates : $!");
+  @templates = grep !/^\./, readdir TEMPLATEDIR;
+  closedir TEMPLATEDIR;
+
+  if (! -d "$templates/$form->{templates}") {
+    mkdir "$templates/$form->{templates}", oct("771") or $form->error("$templates/$form->{templates} : $!");
+  }
+
+  foreach $file (@templates) {
+    if (! -f "$templates/$form->{templates}/$file") {
+      open(TEMP, "$templatedir/$form->{mastertemplates}/$file") or $form->error("$templatedir/$form->{mastertemplates}/$file : $!");
+      open(NEW, ">$templates/$form->{templates}/$file") or $form->error("$templates/$form->{templates}/$file : $!");
+      
+      for (<TEMP>) { print NEW $_ }
+      close(NEW);
+      close(TEMP);
     }
   }
-  
-  if (! -d "$spool/$form->{db}") {
-    if (! (mkdir "$spool/$form->{db}", oct("771")) ) {
-      $form->error("$spool/$form->{db} : $!");
+
+  # copy logo files
+  for $ext (qw(eps png)) {
+    if (! -f "$templates/$form->{templates}/logo.$ext") {
+      open(TEMP, "$userspath/sql-ledger.$ext");
+      open(NEW, ">$templates/$form->{templates}/logo.$ext");
+      for (<TEMP>) { print NEW $_ }
+      close(NEW);
+      close(TEMP);
     }
   }
+
+  if (! -d "$spool/$form->{db}") {
+    mkdir "$spool/$form->{db}", oct("771") or $form->error("$spool/$form->{db} : $!");
+  }
   
   # add admin to members file
   if (! open(FH, ">>$memberfile")) {
@@ -1032,18 +1268,32 @@ sub dbcreate {
     $form->error("$memberfile : $!");
   }
 
-  $form->{dbname} = $form->{db};
-  $form->{login} = "admin\@$form->{dbname}";
-  $form->{stylesheet} = "sql-ledger.css";
+  # create dataset
+  User->dbcreate(\%$form);
 
-  print FH "\n\[$form->{login}\]\n";
+  $form->{charset} = $form->{encoding};
+  $form->{dbname} = $form->{db};
+  $form->{login} = "admin\@$form->{db}";
+  $form->{stylesheet} = "sql-ledger.css";
+  $form->{dateformat} = "mm-dd-yy";
+  $form->{numberformat} = "1,000.00";
+  $form->{dboptions} = "set DateStyle to 'POSTGRES, US'";
+  $form->{dboptions} .= ';set client_encoding to \''.$form->{encoding}."'" if $form->{encoding};
+  $form->{vclimit} = "1000";
+
+  print FH "\[$form->{login}\]\n";
   
   if ($form->{dbpasswd}) {
     $form->{dbpasswd} = pack 'u', $form->{dbpasswd};
     chomp $form->{dbpasswd};
   }
+
+  if ($form->{adminpassword}) {
+    srand( time() ^ ($$ + ($$ << 15)) );
+    $form->{password} = crypt $form->{adminpassword}, 'ad';
+  }
   
-  for (qw(company dbconnect dbdriver dbhost dbname dboptions dbpasswd dbport dbuser stylesheet)) {
+  for (sort (qw(company name email dbconnect dbdriver dbhost dbname dboptions dbpasswd dbport dbuser stylesheet password dateformat numberformat charset vclimit templates))) {
     print FH "$_=$form->{$_}\n" if $form->{$_};
   }
   print FH "\n";
@@ -1051,6 +1301,8 @@ sub dbcreate {
 
   unlink "${memberfile}.LCK";
  
+  delete $form->{password};
+
   &list_datasets;
 
 }
@@ -1079,20 +1331,22 @@ sub lock_dataset { &do_lock_system }
 
 sub lock_system {
   
-   $form->{title} = "SQL-Ledger / ";
-   
-   if ($form->{dbname}) {
-     $form->{title} .= $locale->text('Lock Dataset')." / ".$form->{dbname};
-   } else {
-     $form->{title} .= $locale->text('Lock System');
-   }
+  $form->{title} = "SQL-Ledger / ";
   
+  if ($form->{dbname}) {
+    $form->{title} .= $locale->text('Lock Dataset')." / ".$form->{dbname};
+  } else {
+    $form->{title} .= $locale->text('Lock System');
+  }
+
+  $focus = "lock";
+
   $form->header;
 
   print qq|
-<body class=admin>
+<body class=admin onLoad="document.main.${focus}.focus()" />
 
-<form method=post action=$form->{script}>
+<form name=main method=post action=$form->{script}>
 
 <table>
   <tr class=listheading>
