@@ -1,6 +1,6 @@
 #=====================================================================
-# SQL-Ledger ERP
-# Copyright (c) 2007
+# SQL-Ledger
+# Copyright (c) DWS Systems Inc.
 #
 #  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.com
@@ -14,9 +14,9 @@
 
 use SL::IM;
 use SL::CP;
-use SL::JS;
 
 require "$form->{path}/sr.pl";
+require "$form->{path}/js.pl";
 
 1;
 # end of main
@@ -27,14 +27,15 @@ sub import {
   %title = ( sales_invoice => 'Sales Invoices',
              sales_order => 'Sales Orders',
              purchase_order => 'Purchase Orders',
-	     payment => 'Payments',
-	     customer => 'Customers',
-	     vendor => 'Vendors',
-	     part => 'Parts',
-	     service => 'Services',
-	     labor => 'Labor/Overhead',
-	     partsgroup => 'Groups',
-	     coa => 'Chart of Accounts'
+             payment => 'Payments',
+             customer => 'Customers',
+             vendor => 'Vendors',
+             part => 'Parts',
+             service => 'Services',
+             labor => 'Labor/Overhead',
+             partsgroup => 'Groups',
+             coa => 'Chart of Accounts',
+             gl => 'General Ledger'
 	   );
 
 # $locale->text('Import Sales Invoices')
@@ -73,28 +74,46 @@ sub import {
       for (@{ $form->{all_paymentaccount} }) { $selectpaymentaccount .= qq|$_->{accno}--$_->{description}\n| }
       $paymentaccount = qq|
          <tr>
-	  <th align="right">|.$locale->text('Account').qq|</th>
-	  <td>
-	    <select name=paymentaccount>|.$form->select_option($selectpaymentaccount)
-	    .qq|</select>
-	  </td>
-	</tr>
-	<tr>
-	  <th align="right" nowrap>|.$locale->text('Currency').qq|</th>
-	  <td><select name=currency>|
-	  .$form->select_option($form->{selectcurrency}, $form->{currency})
-	  .qq|</select></td>
-	</tr>
+          <th align="right">|.$locale->text('Account').qq|</th>
+          <td>
+            <select name=paymentaccount>|.$form->select_option($selectpaymentaccount)
+            .qq|</select>
+          </td>
+        </tr>
+        <tr>
+          <th align="right" nowrap>|.$locale->text('Currency').qq|</th>
+          <td><select name=currency>|
+          .$form->select_option($form->{selectcurrency}, $form->{currency})
+          .qq|</select></td>
+        </tr>
 |;
     }
     $v11 = qq|
         <tr>
 	        <td><input name=filetype type=radio class=radio value=v11>&nbsp;|.$locale->text('.v11').qq|</td>
-	</tr>
+        </tr>
 |;
   }
+
+  $form->all_languages(\%myconfig);
+
+  if (@{ $form->{all_language} }) {
+    $form->{mapfile} = $myconfig{countrycode};
+    $form->{selectmapfile} = "\n";
+    for (@{ $form->{all_language} }) { $form->{selectmapfile} .= qq|$_->{code}--$_->{description}\n| }
+
+    $mapfile = qq|
+        <tr>
+          <th align="right">|.$locale->text('Mapfile').qq|</th>
+          <td colspan=3>
+            <select name=mapfile>|.$form->select_option($form->{selectmapfile}, $form->{mapfile}, undef, 1).qq|</select>
+          </td>
+        </tr>
+|;
+
+  }
   
-print qq|
+  print qq|
 <body>
 
 <form enctype="multipart/form-data" method=post action=$form->{script}>
@@ -109,34 +128,32 @@ print qq|
       <table>
         $paymentaccount
         <tr>
-	  <th align="right">|.$locale->text('File to Import').qq|</th>
-	  <td>
-	    <input name=data size=60 type=file>
-	  </td>
-	</tr>
-	<tr>
-	  <th align="right">|.$locale->text('Type of File').qq|</th>
-	  <td>
-	    <table>
-	      <tr>
-	        <td><input name=filetype type=radio class=radio value=csv checked>&nbsp;|.$locale->text('csv').qq|</td>
-		<td>|.$locale->text('Delimiter').qq|</td>
-		<td><input name=delimiter size=2 value="$form->{delimiter}"></td>
-		<td><input name=tabdelimited type=checkbox class=checkbox>&nbsp;|.$locale->text('Tab delimited').qq|</td>
-		<td><input name=stringsquoted type=checkbox class=checkbox checked>&nbsp;|.$locale->text('Strings quoted').qq|</td>
-	      </tr>
-		$v11
+	        <th align="right">|.$locale->text('File to Import').qq|</th>
+          <td>
+            <input name=data size=60 type=file>
+          </td>
+        </tr>
+        <tr>
+          <th align="right">|.$locale->text('Type of File').qq|</th>
+          <td>
+            <table>
+              <tr>
+                <td><input name=filetype type=radio class=radio value=csv checked>&nbsp;|.$locale->text('csv').qq|</td>
+                <td>|.$locale->text('Delimiter').qq|</td>
+                <td><input name=delimiter size=2 value="$form->{delimiter}"></td>
+                <td><input name=tabdelimited type=checkbox class=checkbox>&nbsp;|.$locale->text('Tab delimited').qq|</td>
+                <td><input name=stringsquoted type=checkbox class=checkbox checked>&nbsp;|.$locale->text('Strings quoted').qq|</td>
+              </tr>
+              $v11
 <!--
-	      <tr>
-	        <td><input name=filetype type=radio class=radio value=xml>&nbsp;|.$locale->text('xml').qq|</td>
-	      </tr>
+              <tr>
+                <td><input name=filetype type=radio class=radio value=xml>&nbsp;|.$locale->text('xml').qq|</td>
+              </tr>
 -->
-	      <tr>
-		<td><input name=mapfile type=checkbox value=1>&nbsp;|.$locale->text('Mapfile').qq|</td>
-	      </tr>
-	    </table>
-	  </td>
-	</tr>
+            </table>
+          </td>
+        </tr>
+        $mapfile
       </table>
     </td>
   </tr>
@@ -192,12 +209,11 @@ sub export {
     for (@{ $form->{all_report} }) { $form->{selectreportform} .= qq|$_->{reportdescription}--$_->{reportid}\n| }
     
     $reportform = qq|
-       <tr>
-	<th align="right">|.$locale->text('Report').qq|</th>
-	<td>
-	  <select name=report onChange="ChangeReport();">|.$form->select_option($form->{selectreportform}, undef, 1)
-	  .qq|</select>
-	</td>
+      <tr>
+        <th align="right">|.$locale->text('Report').qq|</th>
+        <td>
+        <select name=report onChange="ChangeReport();">|.$form->select_option($form->{selectreportform}, undef, 1).qq|</select>
+        </td>
       </tr>
 |;
   }
@@ -272,16 +288,16 @@ sub export {
       $form->{paymentaccount} = @{ $form->{all_paymentaccount} }[0]->{accno}.qq|--|.@{ $form->{all_paymentaccount} }[0]->{description};
 
       if (@{ $form->{all_paymentmethod} }) {
-	$form->{selectpaymentmethod} = "\n";
-	for (@{ $form->{all_paymentmethod} }) { $form->{selectpaymentmethod} .= qq|$_->{description}--$_->{id}\n| }
+        $form->{selectpaymentmethod} = "\n";
+          for (@{ $form->{all_paymentmethod} }) { $form->{selectpaymentmethod} .= qq|$_->{description}--$_->{id}\n| }
       }
 
       $paymentaccount = qq|
-         <tr>
-	  <th align="right">|.$locale->text('Account').qq|</th>
-	  <td>
-	    <select name=paymentaccount>|.$form->select_option($form->{selectpaymentaccount})
-	    .qq|</select>
+        <tr>
+          <th align="right">|.$locale->text('Account').qq|</th>
+        <td>
+          <select name=paymentaccount>|.$form->select_option($form->{selectpaymentaccount})
+	        .qq|</select>
 	  </td>
 	  <td>$includeinreport{accountnumber}->{html}</td>
 |;
@@ -330,12 +346,14 @@ sub export {
   }
 
 
-  JS->change_report(\%$form, \@input, \@checked, \%radio);
+  &change_report(\%$form, \@input, \@checked, \%radio);
+
+  &calendar;
   
   print qq|
 <body>
 
-<form method=post action=$form->{script}>
+<form method="post" name="main" action="$form->{script}">
 
 <table width=100%>
   <tr>
@@ -347,23 +365,23 @@ sub export {
       <table>
         $reportform
         $paymentaccount
-	$currency
-	$paymentmethod
-	<tr>
-	  <th align="right">|.$locale->text('Date Prepared').qq|</th>
-	  <td>
-	    <input name=dateprepared value="$form->{dateprepared}" title="$myconfig{dateformat}">
-	  </td>
-	  <td>$includeinreport{dateprepared}->{html}</td>
-	</tr>
+        $currency
+        $paymentmethod
         <tr>
-	  <th align="right">|.$locale->text('Date Format').qq|</th>
-	  <td>
-	    <select name=dateformat>|
-	    .$form->select_option($selectdateformat, $form->{dateformat})
-	    .qq|
-	  </td>
-	</tr>
+          <th align="right">|.$locale->text('Date Prepared').qq|</th>
+          <td>
+            <input name=dateprepared size=11 value="$form->{dateprepared}" title="$myconfig{dateformat}">|.&js_calendar("main", "dateprepared").qq|
+          </td>
+          <td>$includeinreport{dateprepared}->{html}</td>
+        </tr>
+        <tr>
+          <th align="right">|.$locale->text('Date Format').qq|</th>
+          <td>
+            <select name=dateformat>|
+            .$form->select_option($selectdateformat, $form->{dateformat})
+            .qq|
+          </td>
+        </tr>
       </table>
     </td>
   </tr>
@@ -392,10 +410,10 @@ sub export {
 	  <td>
 	    <table>
 	      <tr>
-		<td><input name=linefeed type=radio class=radio value="UNIX" $form->{UNIX}>&nbsp;|.$locale->text('UNIX').qq|</td>
-		<td><input name=linefeed type=radio class=radio value="MAC" $form->{CR}>&nbsp;|.$locale->text('MAC').qq|</td>
-		<td><input name=linefeed type=radio class=radio value="DOS" $form->{DOS}>&nbsp;|.$locale->text('DOS').qq|</td>
-		<td><input name=linefeed type=radio class=radio value="noLF" $form->{noLF}>&nbsp;|.$locale->text('None').qq|</td>
+          <td><input name=linefeed type=radio class=radio value="UNIX" $form->{UNIX}>&nbsp;|.$locale->text('UNIX').qq|</td>
+          <td><input name=linefeed type=radio class=radio value="MAC" $form->{CR}>&nbsp;|.$locale->text('MAC').qq|</td>
+          <td><input name=linefeed type=radio class=radio value="DOS" $form->{DOS}>&nbsp;|.$locale->text('DOS').qq|</td>
+          <td><input name=linefeed type=radio class=radio value="noLF" $form->{noLF}>&nbsp;|.$locale->text('None').qq|</td>
 	      </tr>
 	    </td>
 	  </table>
@@ -494,9 +512,13 @@ sub im_sales_invoice {
   $column_data{duedate} = $locale->text('Due Date');
   $column_data{employee} = $locale->text('Salesperson');
 
-  $form->helpref("import_$form->{type}", $myconfig{countrycode});
+  $column_data{ndx} = qq|<input name="allbox" type=checkbox class=checkbox value="1" checked onChange="CheckAll();">|;
   
+  $form->helpref("import_$form->{type}", $myconfig{countrycode});
+
   $form->header;
+  
+  &check_all(qw(allbox ndx_));
  
   print qq|
 <body>
@@ -542,11 +564,10 @@ sub im_sales_invoice {
       $column_data{totalqty} = qq|<td align=right>|.$form->format_amount(\%myconfig, $form->{"totalqty_$i"}).qq|</td>|;
 
       $column_data{runningnumber} = qq|<td align=right>$k</td>|;
-      
-      if ($form->{"missing_$i"}) {
-	$column_data{ndx} = qq|<td>&nbsp;</td>|;
+      if ($form->{"missingpart_$i"} || $form->{"missing$form->{vc}_$i"}) {
+        $column_data{ndx} = qq|<td>&nbsp;</td>|;
       } else {
-	$column_data{ndx} = qq|<td><input name="ndx_$i" type=checkbox class=checkbox checked></td>|;
+        $column_data{ndx} = qq|<td><input name="ndx_$i" type=checkbox class=checkbox checked></td>|;
       }
 
       for (@column_index) { print $column_data{$_} }
@@ -581,9 +602,9 @@ sub im_sales_invoice {
     <tr>
       <td>|;
       $form->info($locale->text('The following customers are not on file:')."\n\n");
-      for (split /\n/, $form->{missingcustomer}) {
-	$form->info("$_\n");
-      }
+    for (split /\n/, $form->{missingcustomer}) {
+	    $form->info("$_\n");
+    }
     print qq|
       </td>
     </tr>
@@ -679,10 +700,14 @@ sub im_order {
   $column_data{total} = $locale->text('Total');
   $column_data{curr} = $locale->text('Curr');
 
+  $column_data{ndx} = qq|<input name="allbox" type=checkbox class=checkbox value="1" checked onChange="CheckAll();">|;
+
   $form->helpref("import_$form->{type}", $myconfig{countrycode});
   
   $form->header;
  
+  &check_all(qw(allbox ndx_));
+
   print qq|
 <body>
 
@@ -727,10 +752,10 @@ sub im_order {
 
       $column_data{runningnumber} = qq|<td align=right>$k</td>|;
       
-      if ($form->{"missing_$i"}) {
-	$column_data{ndx} = qq|<td>&nbsp;</td>|;
+      if ($form->{"missingpart_$i"} || $form->{"missing$form->{vc}_$i"}) {
+        $column_data{ndx} = qq|<td>&nbsp;</td>|;
       } else {
-	$column_data{ndx} = qq|<td><input name="ndx_$i" type=checkbox class=checkbox checked></td>|;
+        $column_data{ndx} = qq|<td><input name="ndx_$i" type=checkbox class=checkbox checked></td>|;
       }
 
       for (@column_index) { print $column_data{$_} }
@@ -768,7 +793,7 @@ sub im_order {
       <td>|;
       $form->info("$msg\n\n");
       for (split /\n/, $form->{"missing$form->{vc}"}) {
-	$form->info("$_\n");
+        $form->info("$_\n");
       }
     print qq|
       </td>
@@ -776,7 +801,6 @@ sub im_order {
 |;
   }
 
-
   if ($form->{missingpart}) {
     print qq|
     <tr>
@@ -837,12 +861,16 @@ sub im_coa {
   $column_data{gifi_accno} = $locale->text('GIFI');
   $column_data{contra} = $locale->text('C');
   
+  $column_data{ndx} = qq|<input name="allbox" type=checkbox class=checkbox value="1" checked onChange="CheckAll();">|;
+  
   IM->prepare_import_data(\%myconfig, \%$form);
  
   $form->helpref("import_$form->{type}", $myconfig{countrycode});
   
   $form->header;
  
+  &check_all(qw(allbox ndx_));
+  
   print qq|
 <body>
 
@@ -927,6 +955,163 @@ sub import_chart_of_accounts {
 }
 
 
+sub im_gl {
+
+  &import_file;
+
+  $form->{callback} = "$form->{script}?action=import";
+  for (qw(type login path)) { $form->{callback} .= "&$_=$form->{$_}" }
+
+  &xrefhdr;
+  
+  @column_index = (ndx);
+
+  for (sort { $form->{$form->{type}}{$a}{ndx} <=> $form->{$form->{type}}{$b}{ndx} } keys %{ $form->{$form->{type}} }) {
+    push @column_index, $_;
+  }
+
+  $form->{reportcode} = "import_$form->{type}";
+
+  $column_data{reference} = $locale->text('Reference');
+  $column_data{description} = $locale->text('Description');
+  $column_data{transdate} = $locale->text('Date');
+  $column_data{accno} = $locale->text('Account');
+  $column_data{debit} = $locale->text('Debit');
+  $column_data{credit} = $locale->text('Credit');
+  $column_data{memo} = $locale->text('Memo');
+  $column_data{notes} = $locale->text('Notes');
+  $column_data{department} = $locale->text('Department');
+  $column_data{approved} = $locale->text('A');
+  $column_data{curr} = $locale->text('Curr');
+  $column_data{exchangerate} = $locale->text('Rate');
+  $column_data{source} = $locale->text('Source');
+  $column_data{project} = $locale->text('Project');
+  
+  $column_data{ndx} = qq|<input name="allbox" type=checkbox class=checkbox value="1" checked onChange="CheckAll();">|;
+  
+  IM->prepare_import_data(\%myconfig, \%$form);
+
+  $sameitem = "";
+
+  for $i (1 .. $form->{rowcount}) {
+		unless ($form->{"reference_$i"}) {
+      $form->error($locale->text('Reference missing!'));
+			exit;
+		}
+		
+    if ($form->{"reference_$i"} ne $sameitem) {
+      $ndx{$form->{"reference_$i"}} = 1;
+      $debit{$form->{"reference_$i"}} = $form->{"debit_$i"};
+      $credit{$form->{"reference_$i"}} = $form->{"credit_$i"};
+    } else {
+      $debit{$form->{"reference_$i"}} += $form->{"debit_$i"};
+      $credit{$form->{"reference_$i"}} += $form->{"credit_$i"};
+    }
+    $sameitem = $form->{"reference_$i"};
+  }
+
+  $form->helpref("import_$form->{type}", $myconfig{countrycode});
+  
+  $form->header;
+ 
+  &check_all(qw(allbox ndx_));
+  
+  print qq|
+<body>
+
+<form method=post action=$form->{script}>
+
+<table width=100%>
+  <tr>
+    <th class=listtop>$form->{helpref}$form->{title}</a></th>
+  </tr>
+  <tr height="5"></tr>
+  <tr>
+    <td>
+      <table width=100%>
+        <tr class=listheading>
+|;
+
+  for (@column_index) { print "\n<th>$column_data{$_}</th>" }
+
+  print qq|
+        </tr>
+|;
+
+  @c = (qw(ndx reference transdate description notes department approved curr exchangerate));
+
+  $sameitem = "";
+
+  for $i (1 .. $form->{rowcount}) {
+
+    $j++; $j %= 2;
+ 
+    print qq|
+        <tr class=listrow$j>
+|;
+
+    for (@column_index) { $column_data{$_} = qq|<td>$form->{"${_}_$i"}</td>| }
+
+    $column_data{ndx} = qq|<td><input name="ndx_$i" type=checkbox class=checkbox checked></td>| if $debit{$form->{"reference_$i"}} == $credit{$form->{"reference_$i"}};
+    for (qw(debit credit)) { $column_data{$_} = qq|<td align=right>|.$form->format_amount(\%myconfig, $form->{"${_}_$i"}, $form->{precision}).qq|</td>| }
+    $column_data{approved} = ($form->{"approved_$i"}) ? qq|<td>*</td>| : qq|<td>&nbsp;</td>|;
+
+    if ($form->{"reference_$i"} eq $sameitem) {
+      for (@c) { $column_data{$_} = qq|<td>&nbsp;</td>| }
+    }
+    $sameitem = $form->{"reference_$i"};
+
+    $form->hide_form("reference_$i");
+    
+    for (@column_index) { print $column_data{$_} }
+
+    print qq|
+	</tr>
+|;
+
+  }
+
+  print qq|
+      </table>
+    </td>
+  </tr>
+  <tr>
+    <td><hr size=3 noshade></td>
+  </tr>
+
+</table>
+|;
+
+  $form->hide_form(qw(delimiter tabdelimited mapfile stringsquoted type rowcount login path callback));
+
+  print qq|
+<input name=action class=submit type=submit value="|.$locale->text('Import General Ledger').qq|">
+</form>
+
+</body>
+</html>
+|;
+
+}
+
+
+sub import_general_ledger {
+
+  $form->{reportcode} = "import_$form->{type}";
+  IM->import_gl(\%myconfig, \%$form);
+
+  if ($form->{added}) {
+    $form->info($locale->text('Added').":\n$form->{added}\n");
+  }
+  if ($form->{failed}) {
+    $form->info($locale->text('Failed').":\n$form->{failed}\n");
+  }
+
+  $form->info;
+
+}
+
+
 sub xmlorder {
 
   @data = split /\n/, $form->{data};
@@ -958,13 +1143,6 @@ sub xmlorder {
     }
   }
   
-#  for (keys %data) {
-#    warn "$_";
-#    for $item (@{$data{$_}}) {
-#      warn $item;
-#    }
-#  }
-
 }
 
 
@@ -1082,7 +1260,7 @@ sub xrefhdr {
   $i = 1;
 
   if ($form->{mapfile}) {
-    open(FH, "$templates/$myconfig{dbname}/import.map") or $form->error($!);
+    open(FH, "$templates/$myconfig{templates}/$form->{mapfile}/import.map") or $form->error($!);
 
     while (<FH>) {
       next if /^(#|;|\s)/;
@@ -1094,16 +1272,17 @@ sub xrefhdr {
       last if $xrefhdr && $_ =~ /^\[/;
 
       if (/^\[$form->{type}\]/) {
-	$xrefhdr = 1;
-	next;
+        $xrefhdr = 1;
+        next;
       }
 
       if ($xrefhdr) {
-	($key, $value) = split /=/, $_;
-	@f = split /,/, $value;
-	$form->{$form->{type}}{$f[0]} = { field => $key, length => $f[1], ndx => $i++ };
+        ($key, $value) = split /=/, $_;
+        @f = split /,/, $value;
+        $form->{$form->{type}}{$f[0]} = { field => $key, length => $f[1], ndx => $i++ };
       }
     }
+    delete $form->{$form->{type}}{''};
     close FH;
     
   } else {
@@ -1115,8 +1294,8 @@ sub xrefhdr {
       $form->{delimiter} = '\t';
     } else {
       if ($form->{stringsquoted}) {
-	$str =~ s/(^"|"$)//g;
-	$str =~ s/"$form->{delimiter}"/$form->{delimiter}/g;
+        $str =~ s/(^"|"$)//g;
+        $str =~ s/"$form->{delimiter}"/$form->{delimiter}/g;
       }
     }
       
@@ -1182,7 +1361,7 @@ sub import_sales_invoices {
 	} else {
 	  $form->info($locale->text('added')."\n");
 	}
-	
+				
       } else {
 	$form->error($locale->text('Posting failed!'));
       }
@@ -1237,20 +1416,20 @@ sub import_orders {
       # save order
       $form->info("${m}. ".$locale->text('Saving Order ...'));
       if (IM->import_order(\%myconfig, \%$newform, \@{ $ndx{$k} })) {
-	$form->{precision} = $newform->{precision};
+        $form->{precision} = $newform->{precision};
 
-	$form->info(qq| $newform->{ordnumber}, $newform->{description}, $newform->{"$form->{vc}number"}, $newform->{name}, $newform->{city}, |);
-	$form->info($form->format_amount(\%myconfig, $newform->{ordtotal}, $newform->{precision}));
-	$form->info(" ... ");
-	
-	if ($newform->{updated}) {
-	  $form->info($locale->text('updated')."\n");
-	} else {
-	  $form->info($locale->text('added')."\n");
-	}
-	
+        $form->info(qq| $newform->{ordnumber}, $newform->{description}, $newform->{"$form->{vc}number"}, $newform->{name}, $newform->{city}, |);
+        $form->info($form->format_amount(\%myconfig, $newform->{ordtotal}, $newform->{precision}));
+        $form->info(" ... ");
+				
+        if ($newform->{updated}) {
+          $form->info($locale->text('updated')."\n");
+        } else {
+          $form->info($locale->text('added')."\n");
+        }
+        
       } else {
-	$form->error($locale->text('Save failed!'));
+        $form->error($locale->text('Save failed!'));
       }
 
       $total += $newform->{ordtotal};
@@ -1337,11 +1516,15 @@ sub im_csv_payment {
   $column_data{dcn} = $locale->text('DCN');
   $column_data{amount} = $locale->text('Paid');
   $column_data{exchangerate} = $locale->text('Exch');
-
+  
+  $column_data{ndx} = qq|<input name="allbox" type=checkbox class=checkbox value="1" checked onChange="CheckAll();">|;
+  
   $form->helpref("import_$form->{type}", $myconfig{countrycode});
 
   $form->header;
- 
+  
+  &check_all(qw(allbox ndx_));
+
   print qq|
 <body>
 
@@ -1585,10 +1768,17 @@ sub im_vc {
   $column_data{bankzipcode} = $locale->text('Zipcode');
   $column_data{bankcountry} = $locale->text('Country');
 
+  $column_data{clearingnumber} = $locale->text('Clearing Number');
+  $column_data{membernumber} = $locale->text('Member Number');
+  
+  $column_data{ndx} = qq|<input name="allbox" type=checkbox class=checkbox value="1" checked onChange="CheckAll();">|;
+
   $form->helpref("import_$form->{type}", $myconfig{countrycode});
   
   $form->header;
  
+  &check_all(qw(allbox ndx_));
+
   print qq|
 <body>
 
@@ -1663,7 +1853,7 @@ sub im_vc {
     %button = ('Import Vendors' => { ndx => 1, key => 'I', value => $locale->text('Import Vendors') });
   }
 
-  for (sort { $button{$a}->{ndx} <=> $button{$b}->{ndx} } keys %button) { $form->print_button(\%button, $_) }
+  $form->print_button(\%button);
   
   print qq|
 </form>
@@ -1709,7 +1899,7 @@ sub im_item {
   for (qw(type login path)) { $form->{callback} .= "&$_=$form->{$_}" }
 
   &xrefhdr;
-    
+   
   @column_index = qw(runningnumber ndx);
 
   for (sort { $form->{$form->{type}}{$a}{ndx} <=> $form->{$form->{type}}{$b}{ndx} } keys %{ $form->{$form->{type}} }) {
@@ -1749,11 +1939,15 @@ sub im_item {
   $column_data{vendorpartnumber} = $locale->text('Part Number');
   $column_data{leadtime} = $locale->text('Leadtime');
   $column_data{vendorcurr} = $locale->text('Curr');
+  
+  $column_data{ndx} = qq|<input name="allbox" type=checkbox class=checkbox value="1" checked onChange="CheckAll();">|;
 
   $form->helpref("import_$form->{type}", $myconfig{countrycode});
   
   $form->header;
  
+  &check_all(qw(allbox ndx_));
+  
   print qq|
 <body>
 
@@ -1820,7 +2014,9 @@ sub im_item {
 	     'labor' => { ndx => 1, key => 'I', value => $locale->text('Import Labor/Overhead') }
             );
 
-  $form->print_button(\%button, $form->{type});
+  for (qw(part service labor)) { delete $button{$_} unless $_ eq $form->{type} }
+
+  $form->print_button(\%button);
   
   print qq|
 </form>
@@ -1834,7 +2030,7 @@ sub im_item {
 
 sub import_parts { &import_items }
 sub import_services { &import_items }
-sub import_labor { &import_items }
+sub import_labor_overhead { &import_items }
 
 
 sub import_items {
@@ -1873,15 +2069,18 @@ sub im_partsgroup {
   }
 
   $column_data{runningnumber} = "&nbsp;";
-  $column_data{ndx} = "&nbsp;";
   $column_data{partsgroup} = $locale->text('Group');
   $column_data{code} = $locale->text('Code');
   $column_data{pos} = $locale->text('POS');
   
+  $column_data{ndx} = qq|<input name="allbox" type=checkbox class=checkbox value="1" checked onChange="CheckAll();">|;
+
   $form->helpref("import_$form->{type}", $myconfig{countrycode});
 
   $form->header;
  
+  &check_all(qw(allbox ndx_));
+
   print qq|
 <body>
 
@@ -2116,7 +2315,7 @@ sub ex_payment {
   
   $form->header;
 
-  JS->check_all(qw(allbox ndx_));
+  &check_all(qw(allbox ndx_));
 
   print qq|
   
@@ -2453,7 +2652,7 @@ sub ex_payment {
 
   delete $button{'Export Payments'} if ! $form->{rowcount};
 
-  for (sort { $button{$a}->{ndx} <=> $button{$b}->{ndx} } keys %button) { $form->print_button(\%button, $_) }
+  $form->print_button(\%button);
 
   if ($form->{menubar}) {
     require "$form->{path}/menu.pl";
@@ -2686,7 +2885,7 @@ sub export_payments {
   binmode(OUT);
 
   print qq|Content-Type: application/file;
-Content-Disposition: attachment; filename="$form->{filename}.$form->{filetype}"\n\n|;
+Content-Disposition: attachment; filename=$form->{filename}.$form->{filetype}\n\n|;
 
 
   if ($form->{filetype} eq 'csv') {
