@@ -1,6 +1,6 @@
 #=====================================================================
-# SQL-Ledger ERP
-# Copyright (c) 2006
+# SQL-Ledger
+# Copyright (c) DWS Systems Inc.
 #
 #  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.com
@@ -14,6 +14,7 @@
 $menufile = "menu.ini";
 use SL::Menu;
 
+require "$form->{path}/js.pl";
 
 1;
 # end of main
@@ -21,58 +22,6 @@ use SL::Menu;
 
 sub display {
   
-#$form->{callback} = "am.pl?login=$form->{login}&action=$form->{main}&path=$form->{path}&password=$form->{password}&jsmenu=1";
-#$form->redirect;
-#exit;
-
- 
-$form->{frames} = 1;
-#warn $userspath;
-#exit;
-
-  if ($form->{js}) {
-    if ($form->{frames}) {
-      &display_frame;
-    } else {
-      
-      my $menu = new Menu "$menufile";
-      $menu->add_file("custom_$menufile") if -f "custom_$menufile";
-      $menu->add_file("$form->{login}_$menufile") if -f "$form->{login}_$menufile";
-
-      $form->{stagger} = "\t";
-#      $form->{jsmenu} = qq|var MENU_ITEMS = [\n|;
-      $form->{mlmenu} = qq|<div class="mlmenu horizontal blackwhite">\n|;
-
-#      &jsmenu(\%$menu);
-      &mlmenu(\%$menu);
-
-#      $form->{jsmenu} .= qq|];|;
-      $form->{mlmenu} .= qq|</ul>\n</div>\n|;
-
-      # save file
-#      if (! open(FH, ">$userspath/$form->{login}_menu_items.js")) {
-#	&display_frame;
-#      }
-      
-#$form->debug;
-$form->debug("$userspath/mlmenu.html");
-#exit;
-      # display entry screen
-      $form->{callback} = "am.pl?login=$form->{login}&action=$form->{main}&path=$form->{path}&password=$form->{password}&jsmenu=1&mlmenu=";
-      $form->{callback} .= $form->escape($form->{mlmenu},1);
-      
-      $form->redirect;
-
-    }
-  } else {
-    &display_frame;
-  }
-  
-}
-
-
-sub display_frame {
-
   $menuwidth = ($ENV{HTTP_USER_AGENT} =~ /links/i) ? "240" : "155";
   $menuwidth = $myconfig{menuwidth} if $myconfig{menuwidth};
 
@@ -94,13 +43,12 @@ sub display_frame {
 }
 
 
-
 sub acc_menu {
 
   my $menu = new Menu "$menufile";
-  $menu->add_file("custom_$menufile") if -f "custom_$menufile";
-  $menu->add_file("$form->{login}_$menufile") if -f "$form->{login}_$menufile";
-  
+  $menu->add_file("$form->{path}/custom/$menufile") if -f "$form->{path}/custom/$menufile";
+  $menu->add_file("$form->{path}/custom/$form->{login}/$menufile") if -f "$form->{path}/custom/$form->{login}/$menufile";
+
   $form->{title} = $locale->text('Accounting Menu');
 
   $form->header;
@@ -127,9 +75,11 @@ function ChangeClass(menu, newClass) {
 document.onselectstart = new Function("return false");
 </script>
 
-<body class=menu>
+<body class=menu />
 
 <img src=$images/sql-ledger.gif width=80 border=0>
+
+<br>$myconfig{name}
 |;
 
   if ($form->{js}) {
@@ -372,87 +322,6 @@ sub jsmenu {
 }
 
 
-sub mlmenu {
-  my ($menu, $level) = @_;
-  
-  # build menu_{login}.html for user
-  my @menuorder = $menu->access_control(\%myconfig, $level);
-
-  while (@menuorder){
-    $item = shift @menuorder;
-    $label = $item;
-    $label =~ s/.*--//g;
-    $label = $locale->text($label);
-
-    if ($menu->{$item}{submenu}) {
-
-      $form->{items} = 1;
-      
-      $form->{stagger} .= "\t";
-
-      # remove same level items
-      map { shift @menuorder } grep /^$item/, @menuorder;
-
-      &mlmenu($menu, $item);
-      
-      $form->{mlmenu} =~ s/<\/li>$/\n$form->{stagger}<ul>/;
-      chop $form->{stagger};
-      $form->{mlmenu} .= qq|$form->{stagger}</ul>\n|;
-
-    } else {
-
-      if ($menu->{$item}{module}) {
-	$form->{items} = 1;
-	
-	if ($level eq "") {
-
-	  $menu->{$item}{jsmenu} = 1;
-	  $str = $menu->menuitem(\%myconfig, \%$form, $item, $level);
-
-          $form->{mlmenu} .= $form->{stagger};
-	  $form->{mlmenu} .= qq|<li>$str$label</a>\n$form->{stagger}<ul>\n|;
-	  
-	  # remove same level items
-	  map { shift @menuorder } grep /^$item/, @menuorder;
-
-          &mlmenu($menu, $item);
-
-	  $form->{mlmenu} .= qq|$form->{stagger}</ul>\n|;
-	  
-	} else {
-	
-	  $menu->{$item}{jsmenu} = 1;
-	  $str = $menu->menuitem(\%myconfig, \%$form, $item, $level);
-          $form->{mlmenu} .= $form->{stagger};
-	  $form->{mlmenu} .= qq|<li>$str</a>$label</li>\n|;
-
-	}
-
-      } else {
-
-        $form->{mlmenu} .= $form->{stagger};
-	$form->{mlmenu} .= qq|<ul><li>$label</li>\n|;
-	$form->{stagger} .= "\t";
-        
-	&mlmenu($menu, $item);
-
-	chop $form->{stagger};
-        if ($form->{items}) {
-	  $form->{mlmenu} .= qq|$form->{stagger}</ul>\n|;
-	} else {
-	  $form->{mlmenu} .= qq|</ul>\n|;
-	}
-	$form->{items} = 0;
-      }
-
-    }
-
-  }
-
-}
-
-
-
 sub menubar {
 
   1;
