@@ -1,6 +1,7 @@
 -- French Canadian GIFI
 -- Cline des Ligneris
 -- checked DS. March 3, 2004
+-- updated DS Nov 14, 2014
 --
 INSERT INTO gifi (accno,description) VALUES ('1000','Encaisse et dpts');
 INSERT INTO gifi (accno,description) VALUES ('1001','Encaisse');
@@ -49,6 +50,11 @@ INSERT INTO gifi (accno,description) VALUES ('1300','Sommes exigibles d\'actionn
 INSERT INTO gifi (accno,description) VALUES ('1301','Sommes exigibles d\'actionnaire(s) (particuliers)');
 INSERT INTO gifi (accno,description) VALUES ('1302','Sommes exigibles d\'actionnaire(s) (socits)');
 INSERT INTO gifi (accno,description) VALUES ('1303','Sommes exigibles d\'administrateur(s)');
+INSERT INTO gifi (accno,description) VALUES ('1310','Sommes exigibles d\'associ(s)/commandit(s)');
+INSERT INTO gifi (accno,description) VALUES ('1311','Sommes exigibles des commanditaires');
+INSERT INTO gifi (accno,description) VALUES ('1312','Sommes exigibles d\'associs qui sont des socits de personnes');
+INSERT INTO gifi (accno,description) VALUES ('1313','Sommes exigibles des commandits');
+INSERT INTO gifi (accno,description) VALUES ('1314','Sommes exigibles d\'associs dtermins qui ne sont pas des commanditaires');
 INSERT INTO gifi (accno,description) VALUES ('1360','Placements dans une(des) coentreprise(s)/ socit(s) de personnes');
 INSERT INTO gifi (accno,description) VALUES ('1380','Sommes exigibles de coentreprise(s)/socit(s) de personnes');
 INSERT INTO gifi (accno,description) VALUES ('1400','Sommes exigibles des personnes apparentes/placements dans des personnes apparentes');
@@ -194,6 +200,11 @@ INSERT INTO gifi (accno,description) VALUES ('2182','Sommes exigibles d\'actionn
 INSERT INTO gifi (accno,description) VALUES ('2183','Sommes exigibles d\'administrateur(s)');
 INSERT INTO gifi (accno,description) VALUES ('2190','Sommes exigibles des membres');
 INSERT INTO gifi (accno,description) VALUES ('2200','Placements dans une(des) coentreprise(s)/socit(s) de personnes');
+INSERT INTO gifi (accno,description) VALUES ('2210','Sommes exigibles d\'associ(s)/commandit(s)');
+INSERT INTO gifi (accno,description) VALUES ('2211','Sommes exigibles des commanditaires');
+INSERT INTO gifi (accno,description) VALUES ('2212','Sommes exigibles d\'associs qui sont des socits de personnes');
+INSERT INTO gifi (accno,description) VALUES ('2213','Sommes exigibles des commandits');
+INSERT INTO gifi (accno,description) VALUES ('2214','Sommes exigibles d\'associs dtermins qui ne sont pas des commanditaires');
 INSERT INTO gifi (accno,description) VALUES ('2220','Sommes exigibles de coentreprise(s)/socit(s)de personnes');
 INSERT INTO gifi (accno,description) VALUES ('2240','Sommes exigibles des personnes apparentes/');
 INSERT INTO gifi (accno,description) VALUES ('2241','Sommes exigibles de/placements dans des personnes canadiennes apparentes');
@@ -255,11 +266,17 @@ INSERT INTO gifi (accno,description) VALUES ('2703','lment de passif pour les ti
 INSERT INTO gifi (accno,description) VALUES ('2704','Titres sur l\'or et sur l\'argent');
 INSERT INTO gifi (accno,description) VALUES ('2705','Chques et autres items en circulation');
 INSERT INTO gifi (accno,description) VALUES ('2706','Billets garantis');
+INSERT INTO gifi (accno,description) VALUES ('2707','Prts sur carte de crdit');
 INSERT INTO gifi (accno,description) VALUES ('2770','Revenus reports');
 INSERT INTO gifi (accno,description) VALUES ('2780','Sommes dues  un(des) actionnaire(s)/administrateur(s)');
 INSERT INTO gifi (accno,description) VALUES ('2781','Sommes dues  un(des) actionnaire(s) (particuliers)');
 INSERT INTO gifi (accno,description) VALUES ('2782','Sommes dues  un(des) actionnaire(s) (socits)');
 INSERT INTO gifi (accno,description) VALUES ('2783','Sommes dues  un(des) administrateur(s)');
+INSERT INTO gifi (accno,description) VALUES ('2790','Sommes payables aux associ(s)/commandit(s)');
+INSERT INTO gifi (accno,description) VALUES ('2791','Sommes payables aux commanditaires');
+INSERT INTO gifi (accno,description) VALUES ('2792','Sommes payables aux associs qui sont des socits de personnes');
+INSERT INTO gifi (accno,description) VALUES ('2793','Sommes payables aux commandits');
+INSERT INTO gifi (accno,description) VALUES ('2794','Sommes payables aux associs dtermins qui ne sont pas des commanditaires');
 INSERT INTO gifi (accno,description) VALUES ('2840','Sommes dues  une(des) coentreprise(s)/ socit(s) de personnes');
 INSERT INTO gifi (accno,description) VALUES ('2860','Sommes dues  des personnes apparentes');
 INSERT INTO gifi (accno,description) VALUES ('2861','Billets  demande dus  des personnes apparentes');
@@ -298,6 +315,11 @@ INSERT INTO gifi (accno,description) VALUES ('3262','Sommes dues  un(des) action
 INSERT INTO gifi (accno,description) VALUES ('3263','Sommes dues  un(des) administrateur(s)');
 INSERT INTO gifi (accno,description) VALUES ('3270','Sommes dues  des membres');
 INSERT INTO gifi (accno,description) VALUES ('3280','Sommes dues  une(des) coentreprise(s)/socit(s) de personnes');
+INSERT INTO gifi (accno,description) VALUES ('3291','Sommes payables aux associ(s)/commandit(s)');
+INSERT INTO gifi (accno,description) VALUES ('3292','Sommes payables aux commanditaires');
+INSERT INTO gifi (accno,description) VALUES ('3293','Sommes payables aux associs qui sont des socits de personnes');
+INSERT INTO gifi (accno,description) VALUES ('3294','Sommes payables aux commandits');
+INSERT INTO gifi (accno,description) VALUES ('3295','Sommes payables aux associs dtermins qui ne sont pas des commanditaires');
 INSERT INTO gifi (accno,description) VALUES ('3300','Sommes dues  des personnes apparentes');
 INSERT INTO gifi (accno,description) VALUES ('3301','Sommes dues  des personnes canadiennes apparentes');
 INSERT INTO gifi (accno,description) VALUES ('3302','Sommes dues  des personnes trangres apparentes');
@@ -321,10 +343,18 @@ INSERT INTO gifi (accno,description) VALUES ('3541','Surplus d\'apport');
 INSERT INTO gifi (accno,description) VALUES ('3542','Surplus d\'expertise');
 INSERT INTO gifi (accno,description) VALUES ('3543','Rserve gnrale');
 INSERT INTO gifi (accno,description) VALUES ('3570','Compte du sige social');
+INSERT INTO gifi (accno,description) VALUES ('3580','Cumul des autres lments du rsultat tendu - Ce poste est  l\'usage des socits et des socits de personnes qui utilisent aussi bien les normes comptables d\'avant la conversion (Partie V du manuel de comptabilit de l\'ICCA) ou les Normes internationales d\'information financire (IFRS) (Partie I du manuel de comptabilit de l\'ICCA) lorsqu\'elles prparent leurs tats financiers. Ce poste est constitu du solde cumul de toutes les composantes des autres lments du rsultat tendu, soit les produits, charges, gains et pertes qui sont comptabiliss dans le rsultat tendu, mais exclus du rsultat net.');
+INSERT INTO gifi (accno,description) VALUES ('3590','Capital des commandits (cot de la participation dans la socit de personnes) (poste  l\'usage des socits de personnes)');
+INSERT INTO gifi (accno,description) VALUES ('3591','Capital des commanditaires (cot de la participation dans la socit de personnes) (poste  l\'usage des socits de personnes)');
+INSERT INTO gifi (accno,description) VALUES ('3592','Apport de capital de l\'exercice (poste  l\'usage des socits de personnes)');
 INSERT INTO gifi (accno,description) VALUES ('3600','Bnfices non rpartis/dficit');
 INSERT INTO gifi (accno,description) VALUES ('3620','Total des capitaux propres');
+INSERT INTO gifi (accno,description) VALUES ('3630','Total du capital des associs (poste  l\'usage des socits de personnes) - Ce poste reprsente le total de tous les montants de capital des associs et doit tre report (lisez  Contrle de validit des postes )');
 INSERT INTO gifi (accno,description) VALUES ('3640','Total du passif et des capitaux propres');
+INSERT INTO gifi (accno,description) VALUES ('3650','Total du passif et du capital des associs (poste  l\'usage des socits de personnes)');
+INSERT INTO gifi (accno,description) VALUES ('3660','Bnfices non rpartis/dficit - dbut de l\'exercice');
 INSERT INTO gifi (accno,description) VALUES ('3680','Revenu net/perte nette');
+INSERT INTO gifi (accno,description) VALUES ('3690','Apport de capital au dbut de l\'exercice (poste  l'usage des socits de personnes)');
 INSERT INTO gifi (accno,description) VALUES ('3700','Dividendes dclars');
 INSERT INTO gifi (accno,description) VALUES ('3701','Dividendes en espces');
 INSERT INTO gifi (accno,description) VALUES ('3702','Ristournes');
@@ -335,7 +365,15 @@ INSERT INTO gifi (accno,description) VALUES ('3742','Rserves spciales');
 INSERT INTO gifi (accno,description) VALUES ('3743','Redressements relatifs aux devises');
 INSERT INTO gifi (accno,description) VALUES ('3744','lments inhabituels de revenus');
 INSERT INTO gifi (accno,description) VALUES ('3745','Transferts interfonds');
+INSERT INTO gifi (accno,description) VALUES ('3750','Retraits (poste  l\'usage des socits de personnes)');
 INSERT INTO gifi (accno,description) VALUES ('3849','Bnfices non rpartis/dficit  fin de l\'exercice');
+INSERT INTO gifi (accno,description) VALUES ('7000','carts de rvaluation - variations au poste de l\'excdent de rvaluation en raison de la rvaluation d\'immobilisations corporelles et des immobilisations incorporelles');
+INSERT INTO gifi (accno,description) VALUES ('7002','Gains/pertes au titre de prestations dfinies - carts actuariels sur l\'obligation au titre de prestations dfinies comptabiliss dans la priode au cours de laquelle ils sont survenus.');
+INSERT INTO gifi (accno,description) VALUES ('7004','Gains/pertes rsultant de la conversion des tats financiers d\'un tablissement  l\'tranger - gains et pertes rsultant de la conversion des tats financiers d\'un tablissement  l\'tranger');
+INSERT INTO gifi (accno,description) VALUES ('7006','Gains/pertes se rapportant aux instruments de capitaux propres - variations ultrieures de la juste valeur marchande d\'un placement dans un instrument de capitaux propres qui n\'est pas dtenu  des fins de transaction.');
+INSERT INTO gifi (accno,description) VALUES ('7008','Partie relle des gains/pertes sur instruments de couverture de flux de trsorerie - partie relle des gains et des pertes sur les instruments de couverture de flux de trsorerie.');
+INSERT INTO gifi (accno,description) VALUES ('7010','Impt sur le revenu concernant les autres lments du rsultat tendu - impt concernant chaque autre lment du rsultat tendu s\'il est rapport avant impt');
+INSERT INTO gifi (accno,description) VALUES ('7020','Autres lments divers du rsultat tendu - gains et pertes concernant d\'autres lments du rsultat tendu non compris aux lignes 7000  7008');
 INSERT INTO gifi (accno,description) VALUES ('8000','Ventes commerciales de biens et services');
 INSERT INTO gifi (accno,description) VALUES ('8020','Ventes de biens et de services  des personnes apparentes');
 INSERT INTO gifi (accno,description) VALUES ('8030','Ventes entre divisions');
@@ -460,8 +498,9 @@ INSERT INTO gifi (accno,description) VALUES ('8520','Publicit et promotion');
 INSERT INTO gifi (accno,description) VALUES ('8521','Publicit');
 INSERT INTO gifi (accno,description) VALUES ('8522','Dons');
 INSERT INTO gifi (accno,description) VALUES ('8523','Repas et frais de reprsentation');
-INSERT INTO gifi (accno,description) VALUES ('8524','Promotion');
-INSERT INTO gifi (accno,description) VALUES ('8570','Amortissement de biens incorporels');
+INSERT INTO gifi (accno,description) VALUES ('8524','Promotion - dmonstrations/prsentations, kiosques, prsentoirs, dpliants, chantillons et ateliers (donns)');
+INSERT INTO gifi (accno,description) VALUES ('8570','Amortissement de biens incorporels - amortissement de biens incorporels tels que les charges reportes, l\'achalandage (avant 2002), les brevets, les franchises, les droits d\'auteur, les marques de commerce, les frais de constitution et les frais de recherche et dveloppement');
+INSERT INTO gifi (accno,description) VALUES ('8571','cart d\'acquisition (achalandage) - pour les annes d\'imposition 2002 et suivantes');
 INSERT INTO gifi (accno,description) VALUES ('8590','Crances irrcouvrables');
 INSERT INTO gifi (accno,description) VALUES ('8610','Pertes sur prts');
 INSERT INTO gifi (accno,description) VALUES ('8611','Provision pour les pertes sur prts');
@@ -554,12 +593,14 @@ INSERT INTO gifi (accno,description) VALUES ('9150','Dpenses lies  l\'informatiq
 INSERT INTO gifi (accno,description) VALUES ('9151','Amliorations et modernisations');
 INSERT INTO gifi (accno,description) VALUES ('9152','Internet');
 INSERT INTO gifi (accno,description) VALUES ('9180','Taxes foncires');
+INSERT INTO gifi (accno,description) VALUES ('9201','Runions et congrs - participation  des colloques');
 INSERT INTO gifi (accno,description) VALUES ('9200','Frais de dplacement');
 INSERT INTO gifi (accno,description) VALUES ('9220','Services publics');
 INSERT INTO gifi (accno,description) VALUES ('9221','lectricit');
 INSERT INTO gifi (accno,description) VALUES ('9222','Eau');
 INSERT INTO gifi (accno,description) VALUES ('9223','Chauffage');
-INSERT INTO gifi (accno,description) VALUES ('9224','Frais de carburant');
+INSERT INTO gifi (accno,description) VALUES ('9224','Frais de carburant - charbon, diesel, combustibles, gaz naturel, essence et gaz propane pour le chauffage et la prparation des aliments');
+INSERT INTO gifi (accno,description) VALUES ('9225','Tlphone et tlcommunications - tlphone cellulaire, tlcopieur et tlavertisseur');
 INSERT INTO gifi (accno,description) VALUES ('9270','Autres dpenses');
 INSERT INTO gifi (accno,description) VALUES ('9271','carts de caisse');
 INSERT INTO gifi (accno,description) VALUES ('9272','Remboursement de frais de la socit mre');
@@ -618,10 +659,12 @@ INSERT INTO gifi (accno,description) VALUES ('9541','Subventions laitires');
 INSERT INTO gifi (accno,description) VALUES ('9542','Assurance-rcolte');
 INSERT INTO gifi (accno,description) VALUES ('9543','Paiements du CSRN');
 INSERT INTO gifi (accno,description) VALUES ('9544','Paiements provenant du programme d\'aide en cas de catastrophe');
+INSERT INTO gifi (accno,description) VALUES ('9545','Indemnit du programme Agri-stabilit ou Agri-investissement');
+INSERT INTO gifi (accno,description) VALUES ('9546','Indemnit de prime d\'assurance-production');
 INSERT INTO gifi (accno,description) VALUES ('9570','Remises');
-INSERT INTO gifi (accno,description) VALUES ('9571','Remises  carburant');
-INSERT INTO gifi (accno,description) VALUES ('9572','Remises  intrt');
-INSERT INTO gifi (accno,description) VALUES ('9573','Remises  taxes foncires');
+INSERT INTO gifi (accno,description) VALUES ('9571','Remises - carburant');
+INSERT INTO gifi (accno,description) VALUES ('9572','Remises - intrt');
+INSERT INTO gifi (accno,description) VALUES ('9573','Remises - taxes foncires');
 INSERT INTO gifi (accno,description) VALUES ('9574','Reventes, Remises de la TPS pour les dpenses');
 INSERT INTO gifi (accno,description) VALUES ('9575','Remises de la TPS pour les dpenses non');
 INSERT INTO gifi (accno,description) VALUES ('9600','Autres revenus/pertes agricoles');
@@ -641,6 +684,7 @@ INSERT INTO gifi (accno,description) VALUES ('9613','Contrat-location (carburant
 INSERT INTO gifi (accno,description) VALUES ('9614','Location de machines');
 INSERT INTO gifi (accno,description) VALUES ('9615','Revenus/pertes des socits de personnes agricoles');
 INSERT INTO gifi (accno,description) VALUES ('9616','Revenus/pertes des coentreprises agricoles');
+INSERT INTO gifi (accno,description) VALUES ('9617','Revenu li  l\'engraissement  faon');
 INSERT INTO gifi (accno,description) VALUES ('9650','Revenu non agricole');
 INSERT INTO gifi (accno,description) VALUES ('9659','Total des revenus agricoles');
 INSERT INTO gifi (accno,description) VALUES ('9660','Dpenses lies aux rcoltes');
@@ -706,9 +750,11 @@ INSERT INTO gifi (accno,description) VALUES ('9832','Amortissement des biens inc
 INSERT INTO gifi (accno,description) VALUES ('9833','Amortissement de quote-part de lait');
 INSERT INTO gifi (accno,description) VALUES ('9834','Frais de dplacement');
 INSERT INTO gifi (accno,description) VALUES ('9835','Taxes d\'affaires/sur le capital');
+INSERT INTO gifi (accno,description) VALUES ('9836','Commissions et redevances');
 INSERT INTO gifi (accno,description) VALUES ('9850','Dpenses non agricoles');
 INSERT INTO gifi (accno,description) VALUES ('9870','Rgularisation des stocks');
-INSERT INTO gifi (accno,description) VALUES ('9899','Total des revenus agricoles nets');
+INSERT INTO gifi (accno,description) VALUES ('9898','Total des dpenses agricoles');
+INSERT INTO gifi (accno,description) VALUES ('9899','Total-autres revenus tendus');
 INSERT INTO gifi (accno,description) VALUES ('9970','Revenu net/perte nette avant impts et lments');
 INSERT INTO gifi (accno,description) VALUES ('9975','lment(s) extraordinaire(s)');
 INSERT INTO gifi (accno,description) VALUES ('9976','Rglements juridiques');
@@ -716,4 +762,5 @@ INSERT INTO gifi (accno,description) VALUES ('9980','Profits/pertes non matriali
 INSERT INTO gifi (accno,description) VALUES ('9985','lments inhabituels');
 INSERT INTO gifi (accno,description) VALUES ('9990','Impts sur le revenu exigibles de l\'exercice');
 INSERT INTO gifi (accno,description) VALUES ('9995','Provision pour impts sur le revenu diffrs');
+INSERT INTO gifi (accno,description) VALUES ('9998','Total-autres revenus tendus');
 INSERT INTO gifi (accno,description) VALUES ('9999','Revenu net/perte nette aprs impts et lments');
